##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""ZConfig data types

$Id: datatypes.py,v 1.2 2004/01/09 14:05:32 johanc Exp $
"""

from ZODB.config import BaseConfig
from Zope.Startup.datatypes import ZopeDatabase
from apelib.zope2.mapper import createMapper
from apelib.zodb3 import storage, db, resource


class Storage(BaseConfig):

    def open(self):
        config = self.config
        conns = {}
        for c in config.connections:
            conns[c.name] = c.open()
        mapper = createMapper(config.mapper_variation, search_products=1)
        r = resource.StaticResource(mapper)
        return storage.ApeStorage(r, conns, name=self.name)


class Database(ZopeDatabase):

    def createDB(self):
        config = self.config
        if config.mapper_variation:
            mapper = createMapper(config.mapper_variation, search_products=1)
            r = resource.StaticResource(mapper)
        else:
            r = None
        s = config.storage.open()
        kw = {}
        for name in ('scan_interval', 'pool_size', 'cache_size',
                     'version_pool_size', 'version_cache_size'):
            if hasattr(config, name):
                kw[name] = getattr(config, name)
        d = db.ApeDB(s, r, **kw)
        return d


def getParams(config):
    kw = {}
    for name in config.__dict__.keys():
        if not name.startswith('_'):
            kw[name] = getattr(config, name)
    return kw


class FSConnection(BaseConfig):

    def open(self):
        from apelib.fs.connection import FSConnection as impl
        return impl(**getParams(self.config))


class DBAPIConnection:

    def open(self):
        from apelib.sql.dbapi import DBAPIConnector as impl
        config = self.config
        return impl(module_name=config.module_name,
                    prefix=config.prefix,
                    params=config.params,
                    kwparams=config.kwparams)

