##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Standard classifiers.

$Id: classifiers.py,v 1.2 2004/01/09 14:05:32 johanc Exp $
"""

from apelib.core.interfaces import IClassifier
from apelib.core.exceptions import SerializationError, DeserializationError



class FixedClassifier:
    """Classifies objects based purely on the keychain."""

    __implements__ = IClassifier

    def __init__(self):
        self._key_to_res = {}
        self._default_res = None

    def register(self, key, mapper_name):
        self._key_to_res[key] = ({}, mapper_name)

    def registerDefault(self, mapper_name):
        self._default_res = ({}, mapper_name)

    def getResult(self, k):
        res = self._key_to_res.get(k)
        if res is None:
            res = self._default_res
            if res is None:
                raise KeyError("Key %s is not known to fixed classifier %s" %
                               (repr(k), repr(self)))
        return res

    def classifyObject(self, value, keychain):
        return self.getResult(keychain[-1])

    def classifyState(self, event):
        return self.getResult(event.getKeychain()[-1])

    def store(self, event, classification):
        pass


class NullClassifier:
    """A null classifier refuses to classify anything."""

    __implements__ = IClassifier

    def classifyObject(self, value, keychain):
        raise SerializationError("Null classifier")

    def classifyState(self, event):
        raise DeserializationError("Null classifier")

    def store(self, event, classification):
        pass

