##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Filesystem classification section.

$Id: classification.py,v 1.2 2004/01/09 14:05:32 johanc Exp $
"""

from apelib.core.interfaces import IGateway
from apelib.core.schemas import FieldSchema

from base import FSGatewayBase


class FSClassificationSection(FSGatewayBase):
    """Gateway for storing classification data."""

    __implements__ = IGateway

    schema = FieldSchema('classification', 'classification')

    def load(self, event):
        fs_conn = self.getConnection(event)
        p = event.getKey()
        classification = {'node_type': fs_conn.readNodeType(p)}
        text = fs_conn.readSection(p, 'classification', '')
        if text:
            lines = text.split('\n')
            for line in lines:
                if '=' in line:
                    k, v = line.split('=', 1)
                    classification[k.strip()] = v.strip()
        classification['extension'] = fs_conn.getExtension(p)
        return classification, text.strip()

    def store(self, event, state):
        # state is a classification
        fs_conn = self.getConnection(event)
        p = event.getKey()
        items = state.items()
        items.sort()
        text = []
        for k, v in items:
            if k == 'extension':
                fs_conn.suggestExtension(p, v)
            else:
                text.append('%s=%s' % (k, v))
        text = '\n'.join(text)
        fs_conn.writeSection(p, 'classification', text)
        return text.strip()
