##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Filesystem-specific interfaces

$Id: interfaces.py,v 1.2 2004/01/09 14:05:32 johanc Exp $
"""

from Interface import Interface


class IFSConnection (Interface):
    """Simple filesystem connection (with textual annotations).
    """

    def writeSection(subpath, section_name, data):
        """Writes a text-based metadata section for a filesystem node."""

    def writeNodeType(subpath, data):
        """Writes the node type for a filesystem node.

        'd' (directory) and 'f' (file) are supported.
        """

    def writeData(subpath, data, as_text=0):
        """Writes data to a filesystem node.

        In the case of directories, expects a tuple containing the names
        of the files that should be in the directory.  In the case of
        files, expects a string.

        If as_text is true, the file is written in text mode.  The
        as_text flag is ignored for directories.
        """

    def suggestExtension(subpath, ext):
        """Suggests a filename extension for a filesystem node.

        The IFSConnection may use this information to store the file
        with an automatically appended filename extension.
        """

    def readSection(subpath, section_name, default=None):
        """Reads a text-based metadata section.
        """

    def readNodeType(subpath):
        """Reads the node type of a filesystem node.
        """

    def readData(subpath, allow_missing=0, as_text=0):
        """Reads the data from a filesystem node.

        For files, this reads the main data stream.  For directories,
        this returns a list of names.  If the allow_missing flag is
        specified, this method returns None if no filesystem node is
        found.

        If as_text is true, the file is read in text mode.  The
        as_text flag is ignored for directories.
        """

    def getExtension(subpath):
        """Returns the filename extension used for a filesystem node.
        """

    def getModTime(subpath, default=0):
        """Returns the modification time of a file.
        """

    def getSources(subpath):
        """Returns source information for a subpath.

        The source information is a mapping that maps
        (source_repository, path) to a state object.  The contents of
        path and state are specific to a source repository.  The
        source repository (an ISourceRepository) may be polled
        periodically to freshen the state of objects in caches.
        """
