##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""SQL classification gateway

$Id: classification.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

from apelib.core.schemas import FieldSchema
from apelib.core.exceptions import NoStateFoundError
from sqlbase import SQLGatewayBase


class SQLClassification (SQLGatewayBase):

    __implements__ = SQLGatewayBase.__implements__

    schema = FieldSchema('classification', 'classification')

    table_base_name = 'classification2'  # Version 2

    column_defs = (
        ('class_name', 'string', 0),
        ('mapper_name', 'string', 0),
        )

    def load(self, event):
        key = long(event.getKey())
        items = self.execute(event, 'read', 1, key=key)
        classification = {}
        if items:
            rec = items[0]
            if rec[0]:
                classification['class_name'] = rec[0]
            if rec[1]:
                classification['mapper_name'] = rec[1]
        else:
            raise NoStateFoundError(key)
        return classification, rec

    def store(self, event, classification):
        key = long(event.getKey())
        items = self.execute(event, 'read', 1, key=key)
        cn = classification.get('class_name', '')
        mn = classification.get('mapper_name', '')
        kw = {'key': key, 'class_name': cn, 'mapper_name': mn}
        if items:
            self.execute(event, 'update', **kw)
        else:
            self.execute(event, 'insert', **kw)
        return (cn, mn)

