##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Abstract gateways

$Id: sqlbase.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

from apelib.core.interfaces import IGateway, IDatabaseInitializer
from interfaces import ISQLConnection


class SQLGatewayBase:
    """SQL gateway base class"""

    __implements__ = IGateway, IDatabaseInitializer

    table_base_name = '(override this)'
    schema = None       # override
    column_defs = None  # optional override

    def __init__(self, conn_name='db'):
        self.conn_name = conn_name
        if self.column_defs is None:
            schema = self.getSchema()
            if schema is not None:
                self.column_defs = tuple(schema.getColumnDefs())

    def getSchema(self):
        return self.schema

    def init(self, event):
        conn = event.getConnection(self.conn_name)
        assert ISQLConnection.isImplementedBy(conn)
        try:
            if event.clearing():
                self.execute(event, 'clear')
            else:
                self.execute(event, 'check')
        except conn.error:
            conn.db.rollback()
            self.execute(event, 'create')
            conn.db.commit()

    def execute(self, event, operation, *args, **kw):
        conn = event.getConnection(self.conn_name)
        query = conn.getQuery(
            self.table_base_name, self.column_defs, operation)
        if query == '':
            # No query needed for this operation
            return
        return conn.execute(query, *args, **kw)

    def load(self, event):
        raise NotImplementedError, "abstract method"

    def store(self, event, obj):
        raise NotImplementedError, "abstract method"

    def getSources(self, event):
        return None
