##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests of gateway_fs.params

$Id: testparams.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

import unittest

from apelib.fs.params import stringToParams, paramsToString


class ParamsTests(unittest.TestCase):

    def testStringToParams(self):
        s = 'abc def="123 456\\n \\"done\\" " ghi=4 j567 \n'
        params = stringToParams(s)
        self.assertEqual(tuple(params), (
            ('abc', ''),
            ('def', '123 456\n "done" '),
            ('ghi', '4'),
            ('j567', ''),
            ))

    def testParamsToString(self):
        params = (
            ('abc', ''),
            ('def', '123 456\n "done" '),
            ('ghi', '4'),
            ('j567', ''),
            )
        s = paramsToString(params)
        self.assertEqual(s, 'abc def="123 456\\n \\"done\\" " ghi="4" j567')

    def testInvalidKeys(self):
        paramsToString((('abc_-09ABC', ''),))
        self.assertRaises(ValueError, paramsToString, (('a bc', ''),))
        self.assertRaises(ValueError, paramsToString, (('a\nbc', ''),))
        self.assertRaises(ValueError, paramsToString, (('', ''),))
        self.assertRaises(ValueError, paramsToString, ((' abc', ''),))
        self.assertRaises(ValueError, paramsToString, (('abc ', ''),))
        self.assertRaises(ValueError, paramsToString, (('a\tbc', ''),))
        self.assertRaises(ValueError, paramsToString, (('a\rbc', ''),))
        self.assertRaises(ValueError, paramsToString, (('a"bc', ''),))
        self.assertRaises(ValueError, paramsToString, (('0abc', ''),))


if __name__ == '__main__':
    unittest.main()

