##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test of storing objects in relational databases via ZODB

$Id: testzope2sql.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

import unittest
import sys

from apelib.sql.dbapi import DBAPIConnector
from apelib.zodb3.db import ApeDB
from apelib.zodb3.storage import ApeStorage
from apelib.zodb3.resource import StaticResource
from apelib.zope2.mapper import createMapper
from zope2testbase import Zope2TestBase


root_mapper = createMapper('sql')

class Zope2SQLTests (Zope2TestBase):

    dbapi_module = None  # Name of the Database API module (required)
    dbapi_params = ()    # Positional args for connect()
    dbapi_kwparams = {}  # Keyword args for connect()

    def getConnector(self):
        return DBAPIConnector(self.dbapi_module, self.dbapi_params,
                              self.dbapi_kwparams, prefix='test_temp')

    def setUp(self):
        conn = self.getConnector()
        self.root_mapper = root_mapper
        resource = StaticResource(self.root_mapper)
        self.conns = {'db': conn}
        storage = ApeStorage(resource, self.conns, clear_all=1)
        self.storage = storage
        self.db = ApeDB(storage, resource)

    def clear(self):
        self.storage.initDatabases(clear_all=1)
        for conn in self.conns.values():
            conn.db.commit()

    def tearDown(self):
        self.clear()
        self.db.close()

    def testConnect(self):
        # Tests the setUp/tearDown methods
        pass


class PsycopgTests (Zope2SQLTests, unittest.TestCase):
    dbapi_module = 'psycopg'
    dbapi_params = ('',)


class MySQLTests (Zope2SQLTests, unittest.TestCase):
    dbapi_module = 'MySQLdb'
    dbapi_kwparams = {'db': 'ape'}


def test_suite():
    """Makes a test suite for the available databases."""
    suite = unittest.TestSuite()
    for k, v in globals().items():
        mname = getattr(v, 'dbapi_module', None)
        if mname is not None:
            try:
                __import__(mname, {}, {}, ('__doc__',))
            except ImportError:
                sys.stderr.write('Warning: could not import %s. '
                                 'Skipping %s.\n'
                                 % (repr(mname), k))
            else:
                case = v('testConnect')
                connector = case.getConnector()
                try:
                    connector.connect()
                    connector.close()
                except connector.error:
                    sys.stderr.write('Warning: could not open a '
                                     'connection using %s. Skipping %s.\n'
                                     % (repr(mname), k))
                else:
                    suite.addTest(unittest.makeSuite(v, 'test'))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')

