##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Gateways specific to ZODB3.

$Id: gateways.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

from apelib.core.interfaces import IGateway
from apelib.core.schemas import RowSequenceSchema


class ReadOnlyItems:
    """Pseudo-storage of persistent mapping items.

    This is generally used for the database root.
    """

    __implements__ = IGateway

    schema = RowSequenceSchema()
    schema.addField('key', 'string', 1)
    schema.addField('keychain', 'keychain')

    def __init__(self, keychains=None):
        if keychains is None:
            keychains = {}
        self.keychains = keychains

    def set(self, key, keychain):
        self.keychains[key] = keychain

    def getSchema(self):
        return self.schema

    def load(self, event):
        items = self.keychains.items()
        items.sort()
        return items, None

    def store(self, event, data):
        expect = self.load(event)
        if data != expect:
            raise exceptions.StoreError(
                "Attempted to store %s in a read-only gateway.  Should be %s."
                % (repr(data), repr(expect)))
        return None

    def getSources(self, event):
        return None

