##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Monkey-patches necessary to make apelib work in Zope.

$Id: patches.py,v 1.2 2004/01/09 14:05:54 johanc Exp $
"""

from Acquisition import aq_base

from apelib.zodb3.utils import copyOf


def applyCopySupportPatch():
    # Fall back to copying by pickle when ZEXP export/import is not
    # implemented.
    def _getCopy(self, container):
        try:
            return self._real_getCopy(container)
        except NotImplementedError:
            return copyOf(aq_base(self))

    from OFS.CopySupport import CopySource
    CopySource._real_getCopy = CopySource._getCopy
    CopySource._getCopy = _getCopy


def applySetObPatch():
    # Fall back to copying when move/rename is not possible.
    def _setOb(self, id, object):
        if object._p_oid is not None:
            if (getattr(self, '_use_fixed_oids_', 0)
                or (object._p_jar is not None
                    and object._p_jar is not self._p_jar)):
                old = object
                # Forget changes to the original object
                old._p_changed = 0
                object = copyOf(object)
        setattr(self, id, object)

    from OFS.ObjectManager import ObjectManager
    ObjectManager._setOb = _setOb


def applyTmpStorePatch():
    from TmpStore import TmpStore as patched_TmpStore
    import ZODB.TmpStore
    ZODB.TmpStore.TmpStore = patched_TmpStore

def applyPatches():
    applyCopySupportPatch()
    applySetObPatch()
    applyTmpStorePatch()

