##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""Modifications to the control panel for managing multiple databases.

$Id: ControlPanelPatch.py,v 1.2 2004/01/09 14:05:14 johanc Exp $
"""

import Acquisition
from Acquisition import aq_base
import Globals
from OFS.SimpleItem import SimpleItem
from App.CacheManager import CacheManager
from App.ApplicationManager import ApplicationManager, DatabaseManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from MainConfiguration import configuration


class AltDatabaseManager(DatabaseManager, CacheManager):
    """Database management
    """
    db_name = ApplicationManager.db_name
    db_size = ApplicationManager.db_size
    manage_pack = ApplicationManager.manage_pack

Globals.InitializeClass(AltDatabaseManager)


class FakeConnection:
    # Supports the methods of Connection that CacheManager needs

    def __init__(self, db, parent_jar):
        self._db = db
        self.version = parent_jar.getVersion()

    def db(self):
        return self._db

    def getVersion(self):
        return self.version


class DatabaseChooser (SimpleItem):
    """Lets you choose which database to view
    """
    meta_type = 'Database Management'
    name = title = 'Database Management'
    icon = 'p_/DatabaseManagement_icon'
    isPrincipiaFolderish = 1

    manage_options=(
        {'label':'Databases', 'action':'manage_main'},
        )

    manage_main = PageTemplateFile('www/chooseDatabase.pt', globals())

    def __init__(self, id):
        self.id = id

    def getDatabaseNames(self):
        names = configuration.listDatabaseNames()
        names.sort()
        return names

    def __getitem__(self, name):
        db = configuration.getDatabase(name=name)
        m = AltDatabaseManager()
        m.id = name
        m._p_jar = FakeConnection(db, self.getPhysicalRoot()._p_jar)
        return m.__of__(self)

    def __bobo_traverse__(self, request, name):
        if configuration.hasDatabase(name):
            return self[name]
        return getattr(self, name)

    def tpValues(self):
        names = self.getDatabaseNames()
        res = []
        for name in names:
            m = AltDatabaseManager()
            m.id = name
            # Avoid opening the database just for the tree widget.
            m._p_jar = None
            res.append(m.__of__(self))
        return res

Globals.InitializeClass(DatabaseChooser)


if 1:
    # Replace the "Database" control panel with a database chooser
    chooser = DatabaseChooser('Database')
    ApplicationManager.Database = chooser
