# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyApplication/EasyApplicationCatalogAware.py,v 1.6 2003/07/22 07:48:20 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyApplicationCatalogAware."""
__version__='$Revision: 1.6 $'[11:-2]


from ExtensionClass import Base
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwarePropertyManager, VirtualPathCatalogAware

from Products.EasyCore import MLOG_TAB

class EasyApplicationCatalogAware(Base):

    _extended_catalog_aware=1

    def reindex_object(self):
        self.index_object()

    def index_object(self):
        """index this objects once for each language, with different keys."""
        catalog=self.getCatalog()
        if catalog is not None:
            catalog.catalog_object(self)

    def unindex_object(self):
        """index this objects once for each language, with different keys."""
        catalog=self.getCatalog()
        if catalog is not None:
            for language in self.getLanguages():
                catalog.uncatalog_object( (self.physicalURL, language) )

    def manage_afterAdd(self, item, container, external=0):
        if not external and not getattr(self, '_v_possible_move',0):
            resetPublishHistory=getattr(self, 'resetPublishHistory', None)
            if resetPublishHistory: resetPublishHistory()
        self.index_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterAdd(item, container, external=external)
            else:
                object.manage_afterAdd(item, container)
            if s is None: object._p_deactivate()

    def manage_beforeDelete(self, item, container, external=0):
        self.unindex_object()
        self._v_possible_move=1
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_beforeDelete(item, container, external=external)
            else:
                object.manage_beforeDelete(item, container)
            if s is None: object._p_deactivate()

    def manage_afterClone(self, item, external=0):
        self.index_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterClone(item, external=external)
            else:
                object.manage_afterClone(item)
            if s is None: object._p_deactivate()



class EasyApplicationConfCatalogAware(EasyCatalogAwarePropertyManager, VirtualPathCatalogAware):

    _extended_catalog_aware=1

    def manage_afterAdd(self, item, container, external=0):
        if not external: pass
        self.index_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterAdd(item, container, external=external)
            else:
                object.manage_afterAdd(item, container)
            if s is None: object._p_deactivate()

    def manage_beforeDelete(self, item, container, external=0):
        self.unindex_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_beforeDelete(item, container, external=external)
            else:
                object.manage_beforeDelete(item, container)
            if s is None: object._p_deactivate()

    def manage_afterClone(self, item, external=0):
        self.index_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterClone(item, external=external)
            else:
                object.manage_afterClone(item)
            if s is None: object._p_deactivate()

