 # $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyApplication/EasyApplicationConf.py,v 1.31 2003/07/23 06:19:58 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyApplicationConf'''
__version__='$Revision: 1.31 $'[11:-2]

from string import split, strip

#Zope
import Globals, Products
from Acquisition import aq_base, aq_parent, aq_inner, aq_chain, ImplicitAcquisitionWrapper

from Globals import DTMLFile

from AccessControl import ClassSecurityInfo, getSecurityManager
from ComputedAttribute import ComputedAttribute
from OFS.Folder import Folder

#CMFCore
from Products.CMFCore.SkinsContainer import SkinsContainer

#EasyCore
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs
from Products.EasyCore.Utils.PCounter import PCounter
from Products.EasyCore.Utils import Tutils
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

#EasyPublisher
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflowBase import *
from Products.EasyNotification.EasyNotifiable import EasyNotifiable
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers

#EasyApplication
from Products.EasyApplication.EasyApplicationGlobals import EasyAppConfigId, EasyObjectTemplateType, EasyPropertySheetType
from Products.EasyApplication.EasyObject import EasyObject
from Products.EasyApplication.EasySecurityConf import EasyApplicationAdd, EasyApplicationClassManage
from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectView

#Interfaces
from Products.EasyApplication.interfaces.EasyApplication import EasyApplicationConfInterface


from Products.EasyCore import MLOG_TAB

DEFAULT_INDEXES = {}
DEPRECATED_INDEXES=('PrincipiaSearchSource','path')
REQUIRED_INDEXES = {
      'id':'FieldIndex'
    , 'title':'FieldIndex'
    , 'meta_type':'FieldIndex'
    , 'bobobase_modification_time':'FieldIndex'
    , 'getWorkflowState':'FieldIndex'
    , 'getWorkflowStateLastChangedDate':'FieldIndex'
    , 'getWorkflowStateLastChangedBy':'FieldIndex'
    , 'creation_date':'FieldIndex'
    , 'effective_date':'FieldIndex'
    , 'expiration_date':'FieldIndex'
    , 'last_modified_by':'FieldIndex'
    , 'getCurrentLanguage':'FieldIndex'
    , 'subclass':'FieldIndex'
    , 'virtualURL':'FieldIndex'
    , 'virtualPath':'FieldIndex'
    , 'physicalURL':'FieldIndex'
    , 'physicalPath':'FieldIndex'
    , 'physicalParentURL':'FieldIndex'
    , 'physicalPathIndex':'PathIndex'
    , 'skin_path':'FieldIndex'
    , 'is_a_folder':'FieldIndex'
    , 'is_creatable_in':'KeywordIndex'
    , 'enable_workflow_view':'FieldIndex'
    , 'template_attributes':'KeywordIndex'
    , 'isVisible':'FieldIndex'
}


DEFAULT_COLUMNS = ()
DEPRECATED_COLUMNS=('summary',)
REQUIRED_COLUMNS= (
      'id'
    , 'title'
    , 'meta_type'
    , 'bobobase_modification_time'
    , 'getWorkflowState'
    , 'getWorkflowStateLastChangedDate'
    , 'getWorkflowStateLastChangedBy'
    , 'creation_date'
    , 'effective_date'
    , 'expiration_date'
    , 'last_modified_by'
    , 'getCurrentLanguage'
    , 'skin_path'
    , 'virtualURL'
    , 'virtualPath'
    , 'physicalURL'
    , 'physicalPath'
    , 'physicalParentURL'
    , 'getIcon'
    , 'id_prefix'
    , 'is_a_folder'
    , 'is_creatable_in'
    , 'enable_workflow_view'
    , 'template_attributes'
    , 'isVisible'
)

class EasyApplicationConf(ProtectedCatalog, SkinsContainer, VirtualPathBase):
    """Web application configuration object"""

    # NB! This application has no add interface, because it is only added
    # on creation of an EasyApplication object, can only exist once in each
    # EasyApplication and can not be deleted, hence manage_add* is of no use.

    security = ClassSecurityInfo()

    meta_type = 'EasyApplicationConfigure'

    _extended_catalog_aware=1

    ##############
    # This object has it's own list of classes to be shown in the add box.
    #ApplicationClasses = {}

    __interfaces__=(EasyApplicationConfInterface,)


    def __init__(self):
        ProtectedCatalog.__init__(self, EasyAppConfigId, '')

        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)


    security.declareProtected(EasyApplicationClassManage, 'manage_main')
    #EasyApplicationClassManage
    security.declareProtected(EasyApplicationClassManage, 'manage_addProperty')
    security.declareProtected(EasyApplicationClassManage, 'manage_editProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_delProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_changeProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_changePropertyTypes')

    #ZCatalog security redefinition: EasyApplicationClassManage
    ## 'Manage ZCatalog Entries':['Manager']
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogObject')
    security.declareProtected(EasyApplicationClassManage, 'manage_uncatalogObject')
    security.declareProtected(EasyApplicationClassManage, 'catalog_object')
    security.declareProtected(EasyApplicationClassManage, 'uncatalog_object')
    security.declareProtected(EasyApplicationClassManage, 'refreshCatalog')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogView')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogFind')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogSchema')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogIndexes')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogAdvanced')
    security.declareProtected(EasyApplicationClassManage, 'manage_objectInformation')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogReindex')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogFoundItems')
    security.declareProtected(EasyApplicationClassManage, 'manage_catalogClear')
    security.declareProtected(EasyApplicationClassManage, 'manage_addColumn')
    security.declareProtected(EasyApplicationClassManage, 'manage_delColumn')
    security.declareProtected(EasyApplicationClassManage, 'manage_addIndex')
    security.declareProtected(EasyApplicationClassManage, 'manage_delIndex')
    security.declareProtected(EasyApplicationClassManage, 'manage_clearIndex')
    security.declareProtected(EasyApplicationClassManage, 'manage_reindexIndex')
    security.declareProtected(EasyApplicationClassManage, 'availableSplitters')

    ## 'Search ZCatalog':['Anonymous', 'Manager']
    security.declareProtected(EasyApplicationObjectView, 'searchResults')
    security.declareProtected(EasyApplicationObjectView, '__call__')
    security.declareProtected(EasyApplicationObjectView, 'uniqueValuesFor')
    security.declareProtected(EasyApplicationObjectView, 'getpath')
    security.declareProtected(EasyApplicationObjectView, 'schema')
    security.declareProtected(EasyApplicationObjectView, 'indexes')
    security.declareProtected(EasyApplicationObjectView, 'index_objects')
    security.declareProtected(EasyApplicationObjectView, 'all_meta_types')
    security.declareProtected(EasyApplicationObjectView, 'valid_roles')
    security.declareProtected(EasyApplicationObjectView, 'resolve_url')
    security.declareProtected(EasyApplicationObjectView, 'getobject')


    # This shows only the relevant classes
    def filtered_meta_types(self,user=None):
        f=lambda x: x['name'] in (EasyObjectTemplateType, EasyPropertySheetType,)
        return filter(f, Products.meta_types)

    # This function is used to register classes that only should
    # be displayed in this object
    def registerApplicationClass(self, instance_class=None, meta_type='',
                              permission=None, constructors=(),
                              icon=None, permissions=None, legacy=(),):
        """Register a class for use only in this object"""

        result = self.registerClass(instance_class,meta_type,
            permission,constructors,icon,permissions,legacy)

        meta = Products.meta_types[-1]
        Products.meta_types = Products.meta_types[:-1]

        if len(constructors)>1:
            method = constructors[1]
            if type(method) is type(()):
                name,method = method
            else: name = method.__name__
            meta['template_add_method'] = name

        kind = instance_class.meta_type

        l = PartnerClasses.get(kind, [])
        l.append(meta)
        PartnerClasses[kind] = l

        return result


    ###Application Configure Interface

    def getAllTemplates(self, parent_template=None):
        if parent_template==None:
            return self.searchResults(meta_type=EasyObjectTemplateType)
        else:
            return self.searchResults(meta_type=EasyObjectTemplateType, is_creatable_in=parent_template)

    def getPlacesForCreation(self):
        places = ['[Root]']
        for each in self.searchResults(meta_type=EasyObjectTemplateType, is_a_folder='on'):
            places.append(each['id'])
        return places

    def getPropertySheetsFor(self, template):
        query = {}
        query['meta_type'] = { 'query': EasyPropertySheetType }
        query['physicalParentURL'] = { 'query': self.physicalURL }
        globalsheets = self.searchResults( query)
        query['physicalParentURL'] = { 'query': self.physicalURL + template + '/' }
        localsheets = self.searchResults( query)
        return globalsheets + localsheets


    def getDataSheetsFor(self, template): return ()

    def getDataViewsFor(self, template): return ()


    def getFactoryFor(self, template, tag=None):
        raise "Not Implemented", "Not Implemented: getFactoryFor(self, template, tag=None):"

    def getLastTemplateChange(self, template):
        sheets = self.getPropertySheetsFor(template)
        #dates = map( lambda n: getattr(aq_base(n), 'bobobase_modification_time', None), sheets)
        dates = map( lambda n: n['bobobase_modification_time'], sheets)
        if len(dates) > 0: return max(dates)
        else: return 0


    def getObjectManageOptions(self, template, options_left=None, options_right=None):
        MLOG_TAB("Deprecated: getObjectManageOptions")
        template_object = self.getTemplateBy(template)
        if template_object is not None:
            return template_object.getManageOptions(options_left,options_right)
        return ()

    security.declarePrivate('getTemplateByName')
    def getTemplateByName(self, template, default=None):
        query = {}
        query['meta_type'] = { 'query': EasyObjectTemplateType }
        query['id'] = { 'query': template }
        templates = self.searchResults( query)
        if len(templates)>0:
            return aq_base(templates[0].getObject()).__of__(self)
        return default



    ###CMF Skin Container Interface
    security.declarePublic('getSkinPath')
    def getSkinPath(self, name):
        '''
        Converts a skin name to a skin path.
        '''
        query = {}
        query['meta_type'] = { 'query': EasyObjectTemplateType }
        query['id'] = { 'query': name }
        templates = self.searchResults( query)
        if len(templates)>0:
            try: return templates[0].skin_path
            except: pass
        return None

    security.declarePublic('getDefaultSkin')
    def getDefaultSkin(self):
        '''
        Returns the default skin name.
        '''
        raise 'Not implemented'

    security.declarePublic('getRequestVarname')
    def getRequestVarname(self):
        '''
        Returns the variable name to look for in the REQUEST.
        '''
        raise 'Not implemented'


    security.declarePrivate('getSkinByPath')
    def getSkinByPath(self, path, raise_exc=0):
        '''
        Returns a skin at the given path.  A skin path is of the format:
        "some/path, some/other/path, ..."  The first part has precedence.
        '''
        #Original: baseself = aq_base(self)
        baseself = aq_inner(self)
        #Original: skinob = baseself
        skinob = aq_parent(aq_parent(self))
        #Put the skin context on top of the skinfolder container context
        ###skinob = aq_inner(self)
        ###skinob = baseself
        parts = list(split(path, ','))
        parts.reverse()
        for part_path in parts:
            partob = baseself
            for name in split(strip(part_path), '/'):
                if name == '':
                    continue
                if name[:1] == '_':
                    # Not allowed.
                    partob = None
                    if raise_exc:
                        raise SkinPathError('Underscores are not allowed')
                    break
                # Allow acquisition tricks.
                partob = getattr(partob, name, None)
                if partob is None:
                    # Not found.  Cancel the search.
                    if raise_exc:
                        raise SkinPathError('Name not found: %s' % part_path)
                    break
            if partob is not None:
                # Now partob has containment and context.
                # Build the final skinob by creating an object
                # that puts the former skinob in the context
                # of the new skinob.
                skinob = partob.__of__(skinob)

        return skinob
    #w_self = ImplicitAcquisitionWrapper(aq_base(self), parent)

    security.declarePrivate('getSkinByName')
    def getSkinByName(self, name):
        path = self.getSkinPath(name)
        if path is None:
            return None
        return self.getSkinByPath(path)

    #New interface to look up attributes of the skin before loading it.
    security.declarePrivate('getSkinAttributeIds')
    def getSkinAttributeIds(self, skinname):
        query = {}
        query['meta_type'] = { 'query': EasyObjectTemplateType }
        query['id'] = { 'query': skinname }
        templates = self.searchResults(query)
        if len(templates)>0 and hasattr(templates[0], 'template_attributes'):
            return templates[0].template_attributes
        return ()


    ##Object Manager Support Interface

    def manage_afterAdd(self, item, container, external=0):
        self._catalog.clear()
        if not external: pass
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterAdd(item, container, external=external)
            else:
                object.manage_afterAdd(item, container)
            if s is None: object._p_deactivate()

    def manage_beforeDelete(self, item, container, external=0):
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_beforeDelete(item, container, external=external)
            else:
                object.manage_beforeDelete(item, container)
            if s is None: object._p_deactivate()

    def manage_afterClone(self, item, external=0):
        self._catalog.clear()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterClone(item, external=external)
            else:
                object.manage_afterClone(item)
            if s is None: object._p_deactivate()



    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7

        self._object_version = 1.8

        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages


Globals.InitializeClass(EasyApplicationConf)
InitializeBodyTabs(EasyApplicationConf)


#manage_addEasyApplicationConfForm = DTMLFile('dtml/manage_addEasyApplicationForm', globals())

def manage_addEasyApplicationConf(self, REQUEST=None):
    """Add a EasyApplicationConf object"""

    ob=EasyApplicationConf()
    self._setObject(EasyAppConfigId, ob)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyApplicationConf,
        permission=EasyApplicationAdd,
        icon='icons/EasyApplicationConfiguration.gif',
        constructors = ( manage_addEasyApplicationConf, manage_addEasyApplicationConf ),
        interfaces=(EasyApplicationConfInterface,),
        meta_type = EasyApplicationConf.meta_type,
        visibility = None,
    )



