# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyApplication/EasyObject.py,v 1.61 2004/01/23 15:23:01 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyObject'''
__version__='$Revision: 1.61 $'[11:-2]

import Globals, Products, DateTime

from Globals import DTMLFile
from urllib import quote
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager, Unauthorized

from OFS.Folder import Folder
from OFS.PropertySheets import Virtual, View, FixedSchema, DefaultPropertySheets, vps, PropertySheet
from Products.EasyCore.Utils import Tutils
from Products.EasyCore.Utils.PCounter import PCounter
from ComputedAttribute import ComputedAttribute

from Acquisition import Implicit, aq_chain, aq_base, aq_inner, aq_parent, ImplicitAcquisitionWrapper

from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyLanguageService.EasyLanguageHelpers import getLanguageByRFC3066
from Products.EasyNotification.EasyNotifiable import EasyNotifiable

#EasyCore
from Products.EasyCore.WorkflowLocks.LockedPropertyManager import LockedPropertyManager
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.Utils.Utils import distinct

#EasyApplication
from Products.EasyApplication.EasyAppWorkflowManagement import EasyAppWorkflowManagement
from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectView, EasyApplicationObjectAdd, EasyApplicationObjectChange, EasyApplicationObjectView
from Products.EasyApplication.EasySkinnable import EasySkinnableAppObjectManager, VeryTinyItem
from Products.EasyApplication.EasySkinnable import DRAFT_VER
from Products.EasyApplication.EasyApplicationCatalogAware import EasyApplicationCatalogAware
from Products.EasyApplication.EasyFixedSchema import EasyFixedSchema
from Products.EasyApplication.EasyCSVSupport import EasyCSVSupport


#Interfaces
from Products.EasyApplication.interfaces.EasyObject import EasyObjectInterface

import os
DEBUG_MODE = os.environ.get('Z_DEBUG_MODE','')
from Products.EasyCore import MLOG_TAB, C_MLOG_TAB

from App.version_txt import getZopeVersion
zope_version=getZopeVersion()


_marker = []  # Create a new marker object.


class EasyPropertySheets(DefaultPropertySheets):

    def manage_options(self):
        return aq_parent(self).manage_options()

    def delPropertySheet(self, name):
        result=[]
        for propset in aq_parent(self).__propsets__:
            if propset.getId() != name and  propset.xml_namespace() != name:
                result.append(propset)
        aq_parent(self).__propsets__=tuple(result)


class EasyObject(LockedPropertyManager, EasyAppWorkflowManagement, EasyApplicationCatalogAware,
                 EasyCatalogAwareObjectAndPropertyManager, EasyCSVSupport,
                 EasySkinnableAppObjectManager, VirtualPathBase, Publishable, Folder):

    """Web application object class"""

    security = ClassSecurityInfo()

    meta_type = 'EasyObject'


    _v_sheet_ids=None

    _properties=()
    propertysheets=vps(EasyPropertySheets)

    isWorkflowable=1
    _last_template_update = -1

    manage_options_left = ()
    manage_options_right = ()
    manage_main_options = ({'label': 'Content', 'action': 'manage_main' },)

    # This is here as a "private" meta type. It will show up in "all_meta_types",
    # which is necessary to be able to copy EasyObjects into this EasyObject, but
    # it will not appear in filtered_meta_types, and hence not in the add box.
    # This is all pretty much an undocemented and magical part of Zope.

    __interfaces__=(EasyObjectInterface,)
    _product_meta_types = ({'name': 'EasyObject', 'meta_type':'EasyObject', 'action': ''}, )
    _product_interfaces = (EasyObjectInterface, )


    def __init__(self, id, subclass, subclass_title):
        EasySkinnableAppObjectManager.__init__(self)
        Publishable.__init__(self)
        self.id =id
        self.subclass = subclass
        self.subclass_title = subclass_title
        self._object_links = {}


    def _init(self):
        if not self._language_data:
            self.addDefaultObjectVersion()

    security.declareProtected(EasyApplicationObjectChange, 'manage_main')

    if zope_version[0:3]>=(2,5,1) and zope_version[0:3]<(2,6,1):
        manage_main = DTMLFile('dtml/251/manage_main', globals())
    elif zope_version[0:3]>=(2,6,1) and zope_version[0:3]<(2,6,2):
        manage_main = DTMLFile('dtml/261/manage_main', globals())
    elif zope_version[0:3]>=(2,6,2) and zope_version[0:3]<(2,7,0):
        manage_main = DTMLFile('dtml/262/manage_main', globals())
    elif zope_version[0:3]>=(2,7,0) and zope_version[0:3]<(2,8,0):
        manage_main = DTMLFile('dtml/270/manage_main', globals())
    else:
        manage_main = DTMLFile('dtml/262/manage_main', globals())


    #Properties security redefinition: EasyApplicationClassManage
    security.declareProtected(EasyApplicationObjectChange, 'manage_addProperty')
    security.declareProtected(EasyApplicationObjectChange, 'manage_editProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_delProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_changeProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationObjectChange, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationObjectChange, 'manage_changePropertyTypes')

    def title(self):
        return self.id

    def this(self):
        return self

    #Energy+ hack
    def manage_afterAdd(self, item, container, external=0):
        self._init()
        #Energy-plus hack to set the model_reference from the id.
        if getattr(self, 'model_reference', None)!=self.getId():
            try:
                model_reference=self.id.replace('_', ' ').strip()
                current_propsheet = self.propertysheets.get('general_info')
                if hasattr(current_propsheet, 'do_editProperties'):
                    current_propsheet.manage_changeProperties(model_reference=model_reference)
            except:
                pass
        EasyApplicationCatalogAware.manage_afterAdd(self, item, container, external=external)

    #Energy+ hack

    manage_main = DTMLFile('dtml/energy_plus/manage_main', globals())
    security.declareProtected(EasyApplicationObjectChange, 'manage_workflow')
    manage_workflow = DTMLFile('dtml/energy_plus/manage_workflow', globals())
    security.declareProtected(EasyApplicationObjectChange, 'manage_publishform')
    manage_publishform = DTMLFile('dtml/energy_plus/manage_publishform', globals())

    security.declareProtected(EasyApplicationObjectChange, 'wizard_tabs')
    wizard_tabs = DTMLFile('dtml/energy_plus/manage_tabs', globals())


    def getIcon(self):
        Icon=getattr(self, 'Icon', None)
        if Icon is not None:
            return Icon.virtualURL
        return '/misc_/EasyApplication/EasyObject.gif'

    def renderIcon(self, link=None, icon=None, target=None, alt=None, title=None, meta_type=None, content_type=None, hide_link=0):
        img_tmpl='<img src="%(icon)s" alt="%(meta_type)s [%(content_type)s]" title="%(meta_type)s" border="0"/>'
        target_tmpl=' target="%s"'
        link_tmpl='<a href="%s"%s>%s</a>'

        if not icon: icon=self.getIcon()
        result=img_tmpl % {'icon':icon,'meta_type':self.subclass_title,'content_type':self.subclass,}

        if not hide_link:
            if target: target=target_tmpl%target
            else: target=''
            if not link: link=self.virtualURL+'manage_workspace'
            result=link_tmpl % (link, target, result)

        return result


    def manage_options(self):
        """Set up property sheets and return the manage_options stuff"""
        # Sync the property sheets.
        template=self.getObjectTemplate(containment=1)
        return self.getObjectTemplate(containment=1).getManageOptions(self.manage_options_left, self.manage_options_right)

    def filtered_meta_types(self,user=None):
        gmt = []
        entry = None
        for each in Products.meta_types:
            if each['name'] == 'EasyObject':
                entry = each.copy()
                break

        if not entry: return []
        action = entry['action']
        for each in self.getAllEasyProducts(parent_template=self.subclass):
            entry['action'] = 'manage_addProduct/EasyApplication/' + each['id']
            entry['name'] = each['title']
            gmt.append(entry.copy())

        return gmt


    ##Seach Interface
    def getSubObjectsByClass(self, class_name, language=None, revision=None, tags=None, query=None, **kw):
        application=self.getApplicationObject()
        if query is None: query={}
        if language is None : language=self.getCurrentLanguage(self.default_language)
        query['physicalPathIndex']={'query': aq_inner(self).physicalURL}
        query.update(kw)
        results=application.getObjectsByClass('product', language=language, revision=revision, tags=tags, query=query)
        return results
        #Set Brain:
        #return self.searchResults(query)
        #physicalPathIndex=aq_inner(self).physicalURL)


    #Energy+ Wizard

    def getNextView(self, current_view, number_of_views, manage_options):
        if current_view<number_of_views and number_of_views+1==len(manage_options):
            return manage_options[current_view+1]
        return None

    def getPreviousView(self, current_view, number_of_views, manage_options):
        if current_view>0 and number_of_views+1==len(manage_options):
            return manage_options[current_view-1]
        return None

    def getLastView(self, current_view, number_of_views, manage_options):
        if current_view>0 and number_of_views+1==len(manage_options):
            return manage_options[-1]
        return None

    #Energy+ hack
    security.declareProtected(EasyApplicationObjectChange, 'manage_editProperties')
    def manage_editProperties(self, usevalues=None, ep_hack=0, REQUEST=None):
        """\
        Proxy method for the propertiesheets manage_editProperties.
        Uses the method do_editProperties on EasyFixedSchema to change the properties.
        """
        is_manage=REQUEST is not None
        if REQUEST is None: REQUEST=self.REQUEST

        if usevalues==None:
            usevalues=REQUEST #This is default.
        current_sheet=REQUEST.get('current_sheet', None)
        if not ep_hack:
            if current_sheet == None:
                #do all sheets.
                for sheet in self.propertysheets.items():
                    current_propsheet = self.propertysheets.get(sheet[0])
                    if hasattr(current_propsheet, 'do_editProperties'):
                        current_propsheet.do_editProperties(usevalues)
            else:
                current_propsheet = self.propertysheets.get(current_sheet)
                if hasattr(current_propsheet, 'do_editProperties'):
                    current_propsheet.do_editProperties(usevalues)

            self.bobo_base_modification_time = DateTime.DateTime()

        next=None
        if REQUEST.has_key('move_to'):
            move_to=REQUEST.get('move_to', None)
            current_view=REQUEST.get('current_view', None)
            number_of_views=REQUEST.get('number_of_views', None)
            manage_options=REQUEST.get('manage_options', None)
            if move_to.strip()=='Next':
                next=self.getNextView(current_view, number_of_views, manage_options)
            elif move_to.strip()=='Previous':
                next=self.getPreviousView(current_view, number_of_views, manage_options)
            elif move_to.strip()=='Finished':
                next=self.getLastView(current_view, number_of_views, manage_options)
            if next:
                REQUEST.RESPONSE.redirect(next+"?manage_tabs_message=Your%20changes%20have%20been%20saved.&newly_created_object:int=1")
                return

        if is_manage:
            return current_propsheet.manage_main(self, REQUEST,
                   manage_tabs_message='Your changes have been saved.')

    _manage_versions = DTMLFile('dtml/manage_versions', globals())
    def manage_versions(self, client, REQUEST=None):
        """\
        Manage language versions.
        """
        return self._manage_versions(client, REQUEST)

    #



    ###########
    #CMF Skinnable and Easy Object Versioning inferface

    security.declareProtected(EasyApplicationObjectChange, 'manage_select_language')
    def manage_select_language(self, new_language_setting, REQUEST=None):
        """Change the current language setting."""

        if self.hasObjectVersion(DRAFT_VER, new_language_setting):
            self.changeCurrentVersion(DRAFT_VER, new_language_setting)
            message="Current language change to %s" %new_language_setting
        else:
            message="Can't change language to %s" %new_language_setting


        # Removed the getting sheet by REQUEST['URL1'].split('/')[-1]
        # to use an explicilly set current_sheet argument, set as an
        # hidden field in the properties.dtml
        # current_sheet is set in REQUEST['current_sheet'] by the __bobo_traverse__.
        current_propsheet = self.propertysheets.get(REQUEST['current_sheet'])
        #message='Current language change to %s' %new_language_setting
        if REQUEST is not None:
            #Work around to fix the problem with disapearing management tabs.
            #  ... Unleast until we figured out why it happens,
            #      I really want to get rid of the ugly QueryString thing.
            #REQUEST.RESPONSE.redirect(URL1+'/manage_main?manage_tabs_message=%s' % message)
            return current_propsheet.manage_main(
                      client=self
                    , REQUEST=REQUEST
                    , management_view=REQUEST['current_sheet']
                    , manage_tabs_message=message
                )


    security.declarePublic('acquireAllLanguages')
    def acquireAllLanguages(self):
        """Override this to add other languages sources."""
        return self.getApplicationObject().getLanguages()

    def acquireDefaultLanguage(self, default=None):
        """Override this to add other languages sources."""
        application=self.getApplicationObject()
        if application:
            lang=application.getLanguages()
            if lang:
                return lang[0]
        return default


    def _setupPropertySheets(self, obj=None):
        """Sets up the object to have the propsheets that are defined in the SkinFolder"""
        if obj is None: obj = self
        config = self.getSkinsFolder(None)
        if config is None: return
        # reset
        if config.getLastTemplateChange(self.subclass) > self._last_template_update:
            self.__propsets__=()
            sheets = config.getPropertySheetsFor(self.subclass)
            for i in sheets:
                ps=EasyFixedSchema(i.id, i.getObject(), obj, management_view=i.title)
                self.propertysheets.addPropertySheet(ps)
            self._last_template_update = DateTime.DateTime()


    def __bobo_traverse__(self, REQUEST, name):
        # Checks to see if name can be found, if not
        # it checks for a name starting with manage_ps_
        # This is our virtual sheet method, which contains
        # the sheets name. parse the sheet name and look it up
        # in the self.propertysheets.
        # As we know that the virtual sheet method always should be
        # 'manage_main' we return that attribute of the sheet.
        if hasattr(self, name):
            return getattr(self, name)
        if name[0:10]=="manage_ps_":
            sheet_command = name[10:]
            if sheet_command[0:7]=="manage_":
                REQUEST['current_sheet'] = sheet_name = sheet_command[7:]
                sheet = self.propertysheets.get(sheet_name)
                # This object that is returned contains full acqusition
                # It seems to be the correct thing to return.
                # But be aware that the aq_chain also contains the
                # self.propertysheets, the Sheet it self, and the DTMLFile
                return sheet.manage_main
            if sheet_command[0:15]=="editProperties_":
                REQUEST['current_sheet'] = sheet_name = sheet_command[15:]
                sheet = self.propertysheets.get(sheet_name)
                return sheet.manage_editProperties


    security.declareProtected(EasyApplicationObjectAdd,'addObject')
    def addObject(self, subclass, id=None):
        """Add an object."""
        subclass_ob = self.getSkinsFolder().getTemplateByName(subclass)
        if subclass_ob is None:
            raise AttributeError, "Can't add an object of the template %s" % subclass
        subclass_title = subclass_ob.title

        if id is None:
            id = str(self.getUniqueId())
            while len(id) < 4: id = '0' + id

        id=str(id)
        id=id.strip()
        id = subclass_ob.id_prefix + id
        id=Tutils.xlate2url(id)

        ob=EasyObject(id, subclass, subclass_title)
        self._setObject(id, ob)
        ob=self._getOb(id)
        ob.index_object()
        return ob

    security.declarePrivate('getCatalog')
    def getCatalog(self):
        return self.getApplicationObject()


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base=aq_base(self)
        if getattr(base,'_object_version', None)!='1.7': self._object_version = '1.7'
        if not hasattr(base,'_object_links'): self._object_links = {} ; changed=1
        if getattr(base,'default_language', None) is None:
            if hasattr(base,'_language_data') and self._language_data:
                self.default_language=self._language_data.keys()[0]
                changed=1
        #if not hasattr(self, '_id_serial'):
        #    self._id_serial=PCounter()
        #    changed=1
        #elif self._id_serial.__class__.__module__=="Shared.Torped.PCounter":
        #    p = PCounter(self._id_serial)
        #    del self._id_serial
        #    self._id_serial=p
        #    changed=1
        if hasattr(self, '_id_serial'):
            del self._id_serial
            changed=1

        ##Upgrade from language to revisions
        #if not hasattr(self, '_revision_counter') or not hasattr(self, '_revision_data'):
        #    self._revision_counter=PCounter()
        #    self._revision_data = IOBTree()

        #if hasattr(self, '_language_data'):
        #    data=self._language_data.get('en', None)
        #    self.addRevision(data)

        self._object_version = 1.8

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

    manage_test_object_dtml=DTMLFile('manage_test_object_dtml', globals())

    def language_versions(self, REQUEST=None):
        """ttw"""
        return "<h1>language_versions.: %s</h1>" % (str(self._language_data.keys()), )

    security.declareProtected('Easy Upgrade Manage', 'just_english')
    def just_english(self, REQUEST=None):
        """ttw"""
        temp=self._language_data.get('en', None)
        if temp is None:
            if REQUEST:
                return "<h1>No data.: %s</h1>" % (str(self._language_data.keys()), )
        else:
            from ZODB.PersistentMapping import PersistentMapping
            language_data = PersistentMapping()
            language_data['en']=temp
            self._language_data=language_data
        for object in self.objectValues(spec='EasyObject'):
            try: s=object._p_changed
            except: s=0
            object.just_english()
            if s is None: object._p_deactivate()
        if REQUEST:
            return "<h1>Just English</h1>"

    def manage_test(self, REQUEST=None):
        """ttw"""
        template=self.getObjectTemplate()
        return  "Hej this is manage_test in EasyObject: template_id: %s" % (template.getId(),)

Globals.InitializeClass(EasyObject)




manage_addEasyObjectForm = DTMLFile('dtml/manage_addEasyObjectForm', globals())

def manage_addEasyObject(self, id, subclass, subclass_title, REQUEST=None):
    """Add an EasyObject"""
    id=Tutils.xlate2url(id)
    ob=EasyObject(id, subclass, subclass_title)
    self._setObject(id, ob)


    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        u="%s/%s" % (u,quote(id))

        ob=self._getOb(id)
        manage_option='manage_workspace'
        manage_options=ob.manage_options
        if callable(manage_options):
            manage_options=manage_options()
        if len(manage_options)>0:
            manage_option=manage_options[0].get('action','manage_workspace')

        REQUEST.RESPONSE.redirect(u+'/%s?newly_created_object:int=1'%manage_option)
    else:
        return ob


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyObject,
        permission=EasyApplicationObjectAdd,
        icon='icons/EasyObject.gif',
        constructors = ( manage_addEasyObjectForm, manage_addEasyObject ),
        meta_type = 'EasyObject',
        interfaces=(EasyObjectInterface,),
        visibility = None,
    )

    context.registerEasyApplicationType(EasyObject)


