# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCatalog/EasyZCatalog.py,v 1.9 2003/07/18 15:57:25 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""\
A override version of Zope ZCatalog.ZCatalog.
"""
__version__='1.7.6'

import Globals
from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog
from Products.ZCatalog.Catalog import CatalogError
from Products.EasyCatalog.EasyCatalog import EasyCatalog
from Products.EasyCatalog.EasyCatalog import EasyVocabulary
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes

DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {
          'id':'FieldIndex'
        , 'title':'TextIndex'
        , 'bobobase_modification_time':'FieldIndex'
        , 'summary':'TextIndex'
        , 'physicalPathIndex':'PathIndex'
        , 'meta_type':'FieldIndex'

}
DEPRECATED_INDEXES=('id','title','meta_type','bobobase_modification_time','PrincipiaSearchSource','path')

DEFAULT_COLUMNS = ()
REQUIRED_COLUMNS = (
          'id'
        , 'title'
        , 'meta_type'
        , 'bobobase_modification_time'
        , 'summary'

)
DEPRECATED_COLUMNS=('summary', 'bobobase_modification_time', 'meta_type', 'title', 'id')

class EasyZCatalog(ProtectedCatalog):
    """\
    A patch of the ProtectedCatalog.
    """


    def __init__(self, id, title='', vocab_id=None, container=None):
        if vocab_id is not None and container is None:
            raise CatalogError, ("You cannot specify a vocab_id without "
                                     "also specifying a container.")
        if container is not None:
            self=self.__of__(container)
        self.id=id
        self.title=title

        self.vocabulary = None

        self.threshold = 10000
        self._v_total = 0

        if vocab_id is None:
            v = EasyVocabulary('Vocabulary', 'Vocabulary', globbing=1)
            self._setObject('Vocabulary', v)
            self.vocabulary = v
            self.vocab_id = 'Vocabulary'
        else:
            self.vocab_id = vocab_id


        self._catalog = EasyCatalog(vocabulary=self.vocab_id)

    def init_default(self):

        #self._catalog.delIndex(name)

        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)




    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7


        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS)
        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        self._object_version = 1.75

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages


Globals.InitializeClass(EasyZCatalog)
