# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyContactForm/EasyContactForm.py,v 1.30 2004/01/27 10:13:42 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyContactForm Object"""
__version__='$Revision: 1.30 $'[11:-2]


import sys, traceback
from types import ListType, TupleType
from cStringIO import StringIO
import Globals
from Globals import HTML, DTMLFile
import DateTime
from string import join, strip
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.EasyCore.EasyCorePermissions import *
import re
from Acquisition import aq_base, aq_self, aq_chain

from MultiMapping import MultiMapping
from copy import deepcopy
from smtplib import SMTP

from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.Utils import Tutils
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID, DEFAULTVIEWNAME, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyEditor.EasyEditable import EasyEditable
from Products.EasyCore.ManageOptionBase import ManageOptionBase

from Products.EasyNavigation.EasyNavigatable import Navigatable
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflowBase import NullDate

from Products.Formulator.Form import PythonForm, FormValidationError
from Products.EasyCore.EasyContactFormConf import FORMULATORID

#DUBLIN CORE
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyMailService.EasyMailService import EASYMAILSERVICE
from Products.EasyCore.EasyNavigationConf import NAVIGATION_CATALOG_ID

from zLOG import LOG, INFO
from Products.EasyCore import MLOG_TAB

linebreaks = re.compile(r"\n\r|\n|\r")

class EasyContactForm(ManageOptionBase
                     , Navigatable, EasyDublinCore
                     , EasyEditable, Publishable):
    """A EasyContactForm Object """

    security = ClassSecurityInfo()

    meta_type = 'EasyContactForm'

    use_easymailservice = 0
    __catalogs__=(NAVIGATION_CATALOG_ID,)

    _workflowable=1

    manage_options_easycontactform =  (
        {'label':'E-mail Options', 'action':'manage_emailoptions_form',}, )

    use_manage_options = EasyEditable.use_manage_options + \
                        ((None, manage_options_easycontactform, None),) + \
                        Publishable.use_manage_options + \
                        Navigatable.use_manage_options

    mailsubject = ""
    _auto_add_to_navigation = 0
    skey_default='id'

    _properties = Navigatable._properties + EasyDublinCore._properties + \
        (
            {'id':'on_submit_handlers', 'label': 'On Submit Handlers:', 'type': 'lines', 'mode': 'w',},
            {'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'wd','select_variable':'possible_obj_sort_columns'},
        )


    # default attributes that holds view ids.
    _view_defaults = {
                 '_viewview': ('viewview','Form View', 0),
                 '_thankview': ('thankview','Thank View', 1),
                 '_errorview': ('errorview','Error View', 2),
                 '_printview': ('printview','Print View', 3),
            }

    #Safty catch
    on_submit_handlers=['sendmail']

    def __init__(self, id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation=0
                    , use_easymailservice=0
                    , smtphost = "your.mailserver.com"
                    , smtp_port = 25
                    ):
        EasyEditable.__init__(self, id, title, template) #added to include Sync activation!! 03/01/05
        Publishable.__init__(self)
        EasyDublinCore.__init__(self
                                , title=title
                                , description=description
                                , language=language
                                , category=category
                                , subject=subject
                                , effective_date=effective_date
                                , expiration_date=expiration_date
                                , format=format
                                )
        self.id = id
        self._auto_add_to_navigation = add_to_navigation
        #self.title = title
        self._template = template
        self._viewview = 'viewview'
        self._thankview = 'thankview'
        self._errorview = 'errorview'
        self._printview = 'printview'
        self.navigation_title = navtitle
        #self.effective_date = NullDate
        #self.expiration_date = NullDate
        self.comments = []
        #self.creation_date = DateTime.now()
        self.last_modified_by = getSecurityManager().getUser().getId()
        self.last_modified_by = self.last_modified_by
        self.email_from = 'mail@address.domain'
        self.email_to = 'mail@address.domain'
        self.email_cc = ''
        self.email_bcc = ''
        self.dtml_message = "Mail Message (DTML allowed)."
        self.on_submit_handlers=['sendmail']
        self.smtphost = smtphost
        self.smtp_port = smtp_port
        self.subject_prefix = ''
        self.use_easymailservice = use_easymailservice
        self._object_version = '1.7'

    # OVERRIDE EASYEDITABLE
    #security.declareProtected(EasyTemplateManage, 'manage_template')
    #manage_template = DTMLFile('dtml/manage_template_EasyContactForm',globals())

    security.declareProtected(View, 'default_form_render')
    default_form_render = DTMLFile('dtml/default_form_render',globals())


    security.declareProtected(EasyContactFormsSend, 'send')
    def send(self,client=None,REQUEST=None):
        """Send an E-mail and show the result page (_thankview)"""
        editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
        template = editor.getTemplate(self.getTemplateId())
        browserfolder = editor.getBrowserFolderByHUA(REQUEST.HTTP_USER_AGENT)
        viewid=self._thankview
        view =  template.getView(viewid)
        REQUEST.set('browserfolder',browserfolder)
        REQUEST.set('viewid',viewid)
        REQUEST.set('templateid',template.id)
        REQUEST.set('viewmode','view')
        REQUEST.set('view',view)
        REQUEST.set('here',self)
        REQUEST.set('template',template)
        REQUEST.set('editor',editor)
        REQUEST.set('SITEROOTURL',editor.getSiteRootURL())

        try:
            context=template.__of__(editor) #Should also handle Editor and Browser folders
            if browserfolder:
                context=browserfolder.__of__(context) #Should also handle Editor and Browser folders
            context=aq_self(self).__of__(context)
            formObj = getattr(context, FORMULATORID, None)
            result = formObj.validate_all_to_request(REQUEST)
            for handler in self.on_submit_handlers:
                handlerObj=getattr(context,handler,None)
                if handlerObj and callable(handlerObj):
                    try:
                        result = handlerObj(result,REQUEST)
                    except:
                        get_transaction().abort()
                        raise
                else:
                    LOG('EasyContactForm', INFO, 'send', 'Handler [%s] not present in object %s.\n' %(handler, self.virtualURL))
            return view(client=self,REQUEST=REQUEST)
        except FormValidationError, e:
            view =  template.getView(self._viewview)
            errors = {}
            errcount = 0
            for err in e.errors:
                REQUEST['field_error_' + err.field_id] = err.error_text
                errcount = errcount + 1
            REQUEST['form_errors'] = errcount
            REQUEST.set('viewid',self._viewview)
            REQUEST.set('view',view)
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            REQUEST.set('error_type', str(sys.exc_info()[0]))
            REQUEST.set('error_value', sys.exc_info()[1])
            REQUEST.set('error_tb', error_tb)
            return view(client=self,REQUEST=REQUEST)
        except:
            view =  template.getView(self._errorview)
            REQUEST.set('viewid',self._errorview)
            REQUEST.set('view',view)
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            REQUEST.set('error_type', str(sys.exc_info()[0]))
            REQUEST.set('error_value', sys.exc_info()[1])
            REQUEST.set('error_tb', error_tb)
            return view(client=self,REQUEST=REQUEST)

    security.declarePublic('print_view')
    def print_view(self,client=None,REQUEST=None,viewname=None):
        """Show a print version of the e-mail form but only if it validates."""
        editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
        template = editor.getTemplate(self.getTemplateId())
        browserfolder = editor.getBrowserFolderByHUA(REQUEST.HTTP_USER_AGENT)
        viewid=self._printview
        view =  template.getView(viewid)

        REQUEST.set('browserfolder',browserfolder)
        REQUEST.set('templateid',template.id)
        REQUEST.set('viewmode','view')
        REQUEST.set('viewid',viewid)
        REQUEST.set('view',view)
        REQUEST.set('here',self)
        REQUEST.set('template',template)
        REQUEST.set('editor',editor)
        REQUEST.set('SITEROOTURL',editor.getSiteRootURL())

        try:
            context=template.__of__(editor) #Should also handle Editor and Browser folders
            if browserfolder:
                context=browserfolder.__of__(context)
            context=aq_self(self).__of__(context)
            formObj = getattr(context,FORMULATORID,None)
            result = formObj.validate_all_to_request(REQUEST)
            return view(client=self,REQUEST=REQUEST)
        except FormValidationError, e:
            view =  template.getView(self._viewview)
            REQUEST.set('viewid',self._viewview)
            REQUEST.set('view',view)
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            REQUEST.set('error_type', str(sys.exc_info()[0]))
            REQUEST.set('error_value', sys.exc_info()[1])
            REQUEST.set('error_tb', error_tb)
            return view(client=self,REQUEST=REQUEST)
        except:
            view =  template.getView(self._errorview)
            REQUEST.set('viewid',self._errorview)
            REQUEST.set('view',view)
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            REQUEST.set('error_type', str(sys.exc_info()[0]))
            REQUEST.set('error_value', sys.exc_info()[1])
            REQUEST.set('error_tb', error_tb)
            return view(client=self,REQUEST=REQUEST)

    security.declarePrivate('sendmail')
    def sendmail(self, client=None, REQUEST=None):
        if REQUEST:
            multimap=MultiMapping()
            multimap.push(REQUEST)
            multimap.push(client)
        else:
            multimap=client
        fromLst, toLst, ccLst, bccLst = self.cookHeader(multimap)
        messageText = HTML(self.dtml_message,globals())(self,multimap)
        subject = ""
        if self.subject_prefix!='':
            subject = "[%s] " % self.subject_prefix
        subject = subject + HTML(self.mailsubject,globals())(self,multimap)
        easymailservice = self.getEasyMailService()
        if self.use_easymailservice and easymailservice:
            smtp_host= easymailservice.getSMTPHost()
            smtp_port= easymailservice.getSMTPPort()
        else:
            smtp_host = self.smtphost
            smtp_port = self.smtp_port
        smtpserver = SMTP( smtp_host, smtp_port )
        to_all_Lst = filter(None, toLst + ccLst + bccLst)
        for from_adress in fromLst:
            headers = self.renderHeaders(subject, from_adress, toLst, ccLst)
            messageText = headers + '\n' + messageText
            smtpserver.sendmail( from_adress, to_all_Lst, messageText)
        smtpserver.quit()
        return client


    security.declarePrivate('cookHeader')
    def cookHeader(self, multimap):
        """Renders From, To, Cc, and Bcc templates, each resulting in a
        comma seperated list which is converted to a python list and
        all are returned as a tuple;

            return ([from], [to], [cc], [bcc])

        """
        email_from_rendered = HTML(self.email_from,globals())(self,multimap)
        email_from = map(strip, email_from_rendered.split(','))
        email_to_rendered = HTML(self.email_to,globals())(self,multimap)
        email_to = map(strip, email_to_rendered.split(','))
        email_cc_rendered = HTML(self.email_cc, globals())(self,multimap)
        email_cc = map(strip, email_cc_rendered.split(','))
        email_bcc_rendered = HTML(self.email_bcc, globals())(self,multimap)
        email_bcc = map(strip, email_bcc_rendered.split(','))
        return email_from, email_to, email_cc, email_bcc

    def renderHeaders(self, subject='', fromStr='', toLst=[], ccLst=[]):
        result=''
        result = "Subject: " + subject + '\n'
        if fromStr: result = result + "From: " + fromStr + '\n'
        if toLst: result = result + "To: " + ','.join(toLst) + '\n'
        if ccLst: result = result + "Cc: " + ','.join(ccLst) + '\n'
        return result

    # Helper for rendering fields

    def getFieldReturnValue(self,id, REQUEST=None, html=None, listconvertion=None, ):
        if REQUEST is None: REQUEST = self.REQUEST
        if REQUEST.has_key(id):
            value = REQUEST[id]
        elif REQUEST.has_key('field_' + id):
            value = REQUEST['field_' + id]
        else:
            raise AttributeError, id
        if html is not None: linebreak = "<br>"
        else: linebreak = "\n"
        if type(value) == type([]):
            if listconvertion is not None and listconvertion == 'space':
                value = join(value," ")
            else:
                value = join(value,linebreak)
        elif type(value) == type(()):
            if listconvertion is not None and listconvertion == 'lines':
                value = join(value,linebreak)
            else:
                value = join(value," ")
        elif type(value) == type('') and html is not None:
            value = join(linebreaks.split(value),linebreak)
        return value

    def getFieldReturnValueHidden(self, id, REQUEST=None, listconvertion=None):
        value = self.getFieldReturnValue(id, REQUEST,listconvertion)
        return """<INPUT type="hidden" name="field_%(id)s" value="%(value)s">""" % {'id': id,'value': value}


    # Manage Options
    security.declareProtected(EasyContactFormChange, 'manage_emailoptions_form')
    manage_emailoptions_form = DTMLFile('dtml/manage_emailoptions_EasyContact',globals())

    security.declareProtected(EasyContactFormChange, 'manage_emailoptions')
    def manage_emailoptions(self
                            ,   email_from
                            ,   email_to
                            ,   email_cc
                            ,   email_bcc
                            ,   dtml_message
                            ,   mailsubject
                            ,   smtphost=None
                            ,   smtp_port=25
                            ,   subject_prefix=''
                            ,   use_easymailservice=0
                            ,   REQUEST=None):
        """Change the email option for this form"""
        management_view='E-mail Options'
        manage_tabs_message=''
        self.email_from = email_from
        self.email_to = email_to
        self.email_cc = email_cc
        self.email_bcc = email_bcc

        self.mailsubject = mailsubject
        self.subject_prefix = subject_prefix
        self.dtml_message = dtml_message

        if hasattr(self, EASYMAILSERVICE):
            self.use_easymailservice = use_easymailservice
        easymailservice = self.getEasyMailService()
        if self.use_easymailservice and easymailservice:
            self.smtphost= easymailservice.getSMTPHost()
            self.smtp_port= easymailservice.getSMTPPort()
        else:
            if smtphost is not None:
                self.smtphost = smtphost
            try: self.smtp_port = int(smtp_port)
            except: pass
            if self.smtp_port=='': self.smtp_port=25

        if REQUEST is not None:
            return self.manage_emailoptions_form(self,REQUEST
                , management_view=management_view
                , manage_tabs_message=manage_tabs_message
                )

    def getEasyMailService(self):
        return getattr(self, EASYMAILSERVICE, None)

    def getUseEasyMailService(self):
        return self.getEasyMailService() and self.use_easymailservice



    def addNewDocument(self , id
                            , template = None
                            , title=''
                            , navtitle=None
                            , description=''
                            , language=''
                            , category=()
                            , subject=()
                            , effective_date=NullDate
                            , expiration_date=NullDate
                            , format='text/html'
                            , use_easymailservice=None #0
                            , smtphost=None #'your.mailserver.domain'
                            , smtp_port=None #25
                            , add_to_navigation=None
                            , REQUEST=None,RESPONSE=None
                        ):
        if add_to_navigation is None:
            add_to_navigation=self._auto_add_to_navigation
        if template is None:
            template=self._template
        if navtitle is None:
            navtitle=id
        if use_easymailservice is None:
            use_easymailservice=self.use_easymailservice
        if smtphost is None:
            smtphost=self.smtphost
        if smtp_port is None:
            smtp_port=self.smtp_port
        try:
            ob=EasyContactForm(id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                            , subject=subject
                            , effective_date=effective_date
                            , expiration_date=expiration_date
                            , format=format
                            , add_to_navigation=add_to_navigation
                            , use_easymailservice=use_easymailservice
                            , smtphost=smtphost
                            , smtp_port=smtp_port
                          )
            self._setObject(id, ob)
            ob = self._getOb(id) # get object in context
            ob.initcontent()
            ob._setObject(FORMULATORID, PythonForm(FORMULATORID, FORMULATORID))
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyContactForm', ERROR, "Can't create sub document. %s (%s)" % exc[:2], error=exc)


    def cloneSubobjects( self, clone, inv_spec=('EasySlaveCopy','EasyMasterCopy')):
        filtered_ids=[]
        for id, obj in self.objectItems():
            if not getattr(aq_base(obj), 'isAnObjectManager', 0) and obj.meta_type not in inv_spec:
                filtered_ids.append(id)
        cp=self.manage_copyObjects(filtered_ids)
        clone.manage_pasteObjects(cp)

    security.declarePrivate('cloneTo')
    def cloneTo( self, container ):
        """\
        Make a copy of this objekt, including any sub objekt
        not being an ObjectManager.
        """
        excluded_ids=['notification_list','creation_date','EasySlaveCopy','EasyMasterCopy',
                      'publish_history', '_workflow_history', 'created_by', 'approved_by',
                      'effective_date', 'effective_date', 'effective_date_set_by',
                      'expiration_date_set_by', '_template'
                      '_master_data_copy', '_objects', 'last_modified_by']
        self_id = self.getId()
        excluded_ids.extend(self.objectIds())
        clone=EasyContactForm( id = self_id
                            , template = self._template
                            , title=self.title
                            , navtitle=self.navigation_title
                            , description=self.description
                            , language=self.language
                            , category=self.category
                            , subject=self.subject
                            , effective_date=self.effective_date
                            , expiration_date=self.expiration_date
                            , format=self.format
                            , add_to_navigation=self._auto_add_to_navigation
                            , use_easymailservice=self.use_easymailservice
                            , smtphost=self.smtphost
                            , smtp_port=self.smtp_port
                            )
        for id, obj in self.__dict__.items():
            if id not in excluded_ids:
                if hasattr(aq_base(obj), 'inheritedAttribute'):
                    clone.__dict__[id] = aq_base(obj)
                else:
                    clone.__dict__[id] = deepcopy(aq_base(obj))
        container._setObject(self_id, clone)
        clone = container._getOb(self_id)
        self.cloneSubobjects(clone)


    #EVENT TRIGGERS
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        Navigatable.manage_afterAdd(self, item, container)
        EasyEditable.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        Navigatable.manage_beforeDelete(self, item, container)
        EasyEditable.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Navigatable.manage_afterClone(self, item)
        EasyEditable.manage_afterClone(self, item)



    #Skinnable
    #NEW#__of__=EasyEditable.__of__
    #NEW#__getattr__=EasyEditable.__getattr__
    #NEW#_checkId=EasyEditable._checkId
    #NEW#valid_property_id=EasyEditable.valid_property_id


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """
        1.7. EasyContactForms used to have mailrecipient/sender properties
        as lists of recipients.
        They are now instead comma separated texts.
        """
        if isinstance(aq_base(self).email_from, ListType) or isinstance(self.email_from, TupleType):
            self.email_from = ', '.join(filter(None, self.email_from)) ; changed=1
        if isinstance(aq_base(self).email_to, ListType) or isinstance(self.email_to, TupleType):
            self.email_to  = ', '.join(filter(None, self.email_to)) ; changed=1
        if isinstance(aq_base(self).email_cc, ListType) or isinstance(self.email_cc, TupleType):
            self.email_cc  = ', '.join(filter(None, self.email_cc)) ; changed=1
        if isinstance(aq_base(self).email_bcc, ListType) or isinstance(self.email_bcc, TupleType):
            self.email_bcc  = ', '.join(filter(None, self.email_bcc)) ; changed=1
        if (not hasattr(aq_base(self), 'smtp_port')) or self.smtp_port == '': self.smtp_port = 25 ; changed=1
        if getattr(aq_base(self),'_object_version', None)!='1.7': self._object_version = '1.7' ; changed=1
        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8' ; changed=1
        if not getattr(aq_base(self),'_object_links', None): self._object_links = {} ; changed=1

        if hasattr(self, EASYMAILSERVICE) and not hasattr(self,'DONT_USE_MAIL_SERVICE'):
            self.use_easymailservice = 1 ; changed=1

        upgradeProperties(self, {'skey_default':'position',}, ())

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyContactForm)


manage_add_EasyContactForm = DTMLFile('dtml/manage_add_EasyContactForm', globals())


def manage_add_EasyContact(self,  id
                                , template=None
                                , title=''
                                , navtitle=''
                                , description=''
                                , language=''
                                , category=()
                                , subject=()
                                , effective_date=NullDate
                                , expiration_date=NullDate
                                , format='text/html'
                                , add_to_navigation=0
                                , use_easymailservice=0
                                , smtphost='your.mailserver.domain'
                                , smtp_port=25
                                , REQUEST=None
                          ):
    """Add EasyContactForm"""

    id=Tutils.xlate2url(id)

    # BUG
    # Something isn't right here, for instance if a EasyEditor
    # has a none default viewname the created EasyContactForm
    # would try to render DEFAULTVIEWNAME instead of EasyContactForm.default_view
    if template=='**default':
        template = DEFAULTTEMPLATEID
        ob=EasyContactForm(id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                            , subject=subject
                            , effective_date=effective_date
                            , expiration_date=expiration_date
                            , format=format
                            , add_to_navigation=0
                            , use_easymailservice=use_easymailservice
                            , smtphost=smtphost
                            , smtp_port=smtp_port
                          )
        ob._isEasyRoot = 1
        ob._view = DEFAULTVIEWNAME
        self._setObject(id, ob)
        ob = self._getOb(id)
    else:
        if template=='__ACQUIRE' or template is None:
            template = ACQUIRETEMPLATE
        ob=EasyContactForm(id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                            , subject=subject
                            , effective_date=effective_date
                            , expiration_date=expiration_date
                            , format=format
                            , add_to_navigation=add_to_navigation
                            , use_easymailservice=use_easymailservice
                            , smtphost=smtphost
                            , smtp_port=smtp_port
                          )
        self._setObject(id, ob)
        ob = self._getOb(id) # get object in context
        ob.initcontent()
    formid = ob._setObject(FORMULATORID, PythonForm(FORMULATORID, FORMULATORID))
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)




def initialize(context):
    """Initialize EasyEditable Product"""

    context.registerClass (
            EasyContactForm,
            permission=EasyContactFormAdd,
            icon='icons/EasyContactForm.gif',
            constructors=(manage_add_EasyContactForm,
                          manage_add_EasyContact),
            meta_type = EasyContactForm.meta_type
            )

    context.registerViewDefault(
                    EasyContactForm._view_defaults,
                    meta_type = EasyContactForm.meta_type
                )
    context.registerEasyType(EasyContactForm)