# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/ClassWrappers.py,v 1.2 2003/07/22 07:48:20 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''Products.EasyCore.ClassWrappers'''
__version__='$Revision: 1.2 $'[11:-2]


from DocumentTemplate.DT_Util import TemplateDict,InstanceDict
from ExtensionClass import Base
from Acquisition import Implicit
from Acquisition import Implicit, aq_parent, aq_inner, aq_chain


from Products.EasyCore import MLOG_TAB

from ComputedAttribute import ComputedAttribute


class DTMLFileWrapper(Implicit):
    """\
    Products.EasyCore.ClassWrappers.DTMLFileWrapper

    Wraps a DTMLFile object and sets new vars for it.
    This makes it possible to redefine DTMLFiles with new vars.
    """


    isDocTemp=1

    # Prevent acquisition from our container
    index_html=None

    # Templates masquerade as functions:
    class func_code: pass
    func_code=func_code()
    func_code.co_varnames='self','REQUEST'
    func_code.co_argcount=2
    func_defaults=()

    # Only used for wrapping trusted content
    validate = None

    def __init__(self, fn, globals, **kw):
        self._fn = fn
        self.initvars(globals, kw)

    initvars__roles__=()
    def initvars(self, globals, vars):
        if globals:
            for k in globals.keys():
                if k[:1] != '_' and not vars.has_key(k): vars[k]=globals[k]
        self._globals=vars

    def __call__(self, client=None, context=None, guarded_getattr=None, **extras):
        if type(context)!=type({}): context={}
        fn = self._fn
        md = TemplateDict()
        if guarded_getattr is None:
            md.guarded_getattr=getattr(fn, 'guarded_getattr', None)
        else:
            md.guarded_getattr=guarded_getattr
        md.validate = self.validate
        md._push(self._globals)
        if context:
            md._push(context)
        if client is None:
            client=aq_parent(aq_inner(self))
        if extras:
            md._push(extras)
        try:
            return apply(self._fn, (client , md), self._globals)
        finally:
            while md:
                md._pop()


class FunctionTemplate(Implicit):
    """A wrapper to use a python function as a Document Template.
    This is useful if you want to promote a DTMLMethod into a python function
    (to better handle complex logic) but to not want to loose context-oriented
    nature of DTML.
    """

    isDocTemp=1

    # Prevent acquisition from our container
    index_html=None

    # Templates masquerade as functions:
    class func_code: pass
    func_code=func_code()
    func_code.co_varnames='self','REQUEST'
    func_code.co_argcount=2
    func_defaults=()

    # Only used for wrapping trusted content
    validate = None

    def __init__(self, fn, globals, **kw):
        self._fn = fn
        self.initvars(globals, kw)

    initvars__roles__=()
    def initvars(self, globals, vars):
        if globals:
            for k in globals.keys():
                if k[:1] != '_' and not vars.has_key(k): vars[k]=globals[k]
        self._globals=vars

    def __call__(self,client=None,context=None,guarded_getattr=None,**extras):
        fn = self._fn
        if type(context)!=type({}): context={}
        md = TemplateDict()
        if guarded_getattr is None:
            md.guarded_getattr=getattr(fn, 'guarded_getattr', None)
        else:
            md.guarded_getattr=guarded_getattr
        md.validate = self.validate
        md._push(self._globals)
        if context:
            md._push(context)
        if client is not None:
            md._push(InstanceDict(client, md)) # Circ. Ref. 8-|
        if extras:
            md._push(extras)
        try:
            #Call with: method(self, client, context)
            return apply(fn, (aq_parent(aq_inner(self)), client, md), extras)
        finally:
            while md:
                md._pop()


