# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/EasyHTMLDiff.py,v 1.1 2003/03/28 07:20:10 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""EasyHTMLDiff

$Colliberty: Storage.py,v 0.0.1 2001/05/14 22:06:00 johanc Exp $"""
__version__='$Revision: 1.1 $'[11:-2]
import sys, os
from types import StringType, ListType, TupleType

from Acquisition import Implicit, Explicit, Acquired, aq_get, aq_parent, aq_chain
#import ExtensionClass, Globals
from OFS import ZDOM
from Products.EasyCore.ExtendedHTMLParser import ExtendedHTMLParser
from formatter import NullFormatter

#from Products.EasyCore.Utils.PCounter import PCounter
#elem_counter=PCounter()
elem_counter=0
def getUniqueId():
    global elem_counter
    elem_counter=elem_counter+1
    return str(elem_counter)


class Element:
    tag=''
    is_paragraph=0
    is_container=0
    pre_rendered_content=None
    do_pre_render=1
    expects_paragraph=0
    #attrs={}
    #content=[]
    text_content=''
    single_tag=0
    text_node=0

    def __init__(self, tag=None, attrs=()):
        self.id=getUniqueId()
        if type(tag) is StringType:
            self.tag=tag.lower()
        self.content=[]
        self.set_attrs(attrs)

    def type(self):
        if not self.tag: return self.alt_tag
        return self.tag

    def append(self, node):
        if type(node)==StringType: # and node:
            node=Text(node)
        self.content.append(node)
        return node.__of__(self)

    def __len__(self):
        if self.tag=='' or not self.is_container:
            return len(self.text_content)
        return len(self.content)

    def set_attrs(self, attrs):
        self.attrs={}
        for tup in attrs:
            if len(tup)>0:
                key=tup[0]
                if len(tup)>1: value=tup[1]
                else: value=''
                self.attrs[key]=value

    def get_attr(self, key):
        return self.attrs[key]

    def getChildren(self):
        content=[]
        for node in self.content:
            content.append(node.__of__(self))
        return content

    def getText(self):
        return self.render(textmode=1)

    def getHTML(self):
        return self.render()

    def find_paragraph(self):
        if self.is_paragraph: return self
        parent=aq_parent(self)
        if parent is not None:
            return parent.find_paragraph()
        return None

    def find_container(self):
        if self.is_container: return self
        parent=aq_parent(self)
        if parent is not None:
            return parent.find_container()
        return None

    def render_attrs(self):
        result=''
        for key, value in self.attrs.items():
            result=result+' '+key+'="'+value+'"'
        return result

    def render_content(self, textmode=0):
        if self.tag=='' or not self.is_container: return self.text_content
        if type(self.content)!=ListType: raise "Content of Element is not Type"
        result=''
        for elem in self.getChildren():
            result=result+elem.render(textmode=textmode)
        return result

    def render(self, textmode=0):
        if self.pre_rendered_content: return self.pre_rendered_content
        if self.text_node:
            return self.text_content
        if textmode: return " %s " % self.render_content(textmode=1)
        output={'tag':self.tag,'attrs':self.render_attrs(),'content':self.render_content()}
        if self.single_tag:
            return """<%(tag)s%(attrs)s/>""" % output
        return """<%(tag)s%(attrs)s>%(content)s</%(tag)s>""" % output

    def pre_render(self):
        if not self.pre_rendered_content:
            self.pre_rendered_content=self.render()

    def normalize(self):
        new_children=[]
        for elem in self.getChildren():
            if elem.text_node and elem:
                result=result+elem.render(textmode=textmode)


class Text(Element):
    text_node=1
    expects_paragraph=0
    alt_tag="TextNode"
    def __init__(self,  content):
        Element.__init__(self, tag='', attrs=())
        self.text_content=content


class Container(Element):
    is_container=1
    expects_paragraph=0

    def __init__(self,  tag=None, attrs=()):
        Element.__init__(self, tag=tag, attrs=attrs)

class Document(Container):
    tag='document'
    do_pre_render=0
    expects_paragraph=1

    def __init__(self):
        Container.__init__(self, 'document')

    def render(self):
        return self.render_content()

class TABLE(Container):
    tag='table'
    expects_paragraph=0

class TR(Container):
    tag='tr'
    expects_paragraph=0

class TD(Container):
    tag='td'
    expects_paragraph=0

class TH(Container):
    tag='th'
    expects_paragraph=1

class UL(Container):
    tag='ul'
    expects_paragraph=0

class OL(Container):
    tag='ol'
    expects_paragraph=0

class LI(Container):
    tag='li'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=1

class DIV(Container):
    tag='div'
    expects_paragraph=1

class SPAN(Container):
    tag='span'
    expects_paragraph=0

class P(Container):
    tag='p'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=1

class A(Container):
    tag='a'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=1

class IMG(Element):
    tag='a'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=1
    single_tag=1

class BR(Element):
    tag='br'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=0
    single_tag=1

class HR(Element):
    tag='hr'
    do_pre_render=1
    expects_paragraph=0
    is_paragraph=1
    single_tag=1


class MyHTMLParser(ExtendedHTMLParser):

    def __init__(self, verbose=0):
        ExtendedHTMLParser.__init__(self, NullFormatter(), verbose=verbose)
        self.document=Document()
        #self.current_paragraph=None
        self.current_node=self.document

    def handle_data(self, data):
        self.current_node.append(data)

    def unknown_starttag(self, tag, attrs): pass

    def unknown_endtag(self, tag): pass

    def pop_paragraph(self):
        if self.current_node.do_pre_render:
            self.current_node.pre_render()
            self.current_node=aq_parent(self.current_node)

    def start_p(self, attrs):
        if self.current_node.is_paragraph:
            self.pop_paragraph()
        paragraph=P(attrs=attrs)
        self.current_node=self.current_node.append(paragraph)

    def end_p(self):
        self.pop_paragraph()

    def start_br(self, attrs):
        br_node=BR()
        self.current_node.append(br_node)
    def start_end(self): pass


    def start_div(self, attrs): self.start_default('div', attrs)
    def end_div(self): self.end_default('div')
    def start_span(self, attrs): self.start_default('span', attrs)
    def end_span(self): self.end_default('span')
    def start_li(self, attrs): self.start_default('li', attrs)
    def end_li(self): self.end_default('li')
    def start_ol(self, attrs): self.start_default('ol', attrs)
    def end_ol(self): self.end_default('ol')
    def start_ul(self, attrs): self.start_default('ul', attrs)
    def end_ul(self): self.end_default('ul')
    def start_table(self, attrs): self.start_default('table', attrs)
    def end_table(self): self.end_default('table')
    def start_tr(self, attrs): self.start_default('tr', attrs)
    def end_tr(self): self.end_default('tr')
    def start_td(self, attrs): self.start_default('td', attrs)
    def end_td(self): self.end_default('td')
    def start_th(self, attrs): self.start_default('th', attrs)
    def end_th(self): self.end_default('th')

    def start_default(self, tag, attrs):
        #if self.current_node.is_paragraph:
        node=Container(tag, attrs)
        self.current_node=self.current_node.append(node)

    def end_default(self, tag):
        if self.current_node.tag==tag:
            self.current_node=aq_parent(self.current_node)

    def render(self):
        return self.document.render()

