# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/EasyIcon.py,v 1.1 2003/07/08 09:15:27 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyIcon'''
__version__='$Revision: 1.1 $'[11:-2]

import Globals
import string
from types import IntType,TupleType, ListType
from OFS.Image import Image, Pdata, getImageInfo
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo

from ComputedAttribute import ComputedAttribute
from Acquisition import aq_base

from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyCore.interfaces.EasyIcon import EasyIconInterface


from zLOG import LOG, INFO, WARNING

from webdav.common import rfc1123_date
import os
try:
    import PIL
    import PIL.PngImagePlugin
    import PIL.ImageFile
    PIL.ImageFile.MAXBLOCK = 1000000 # default is 64k
    from StringIO import StringIO
    _PIL_is_present = 1
except:
    _PIL_is_present = 0
    LOG("EasyImage", INFO, "PIL (Python Image Library) is not installed on this system")



ICONID='Icon'

class EasyIcon(VirtualPathBase,Image):
    """A EasyImage Object """

    security = ClassSecurityInfo()

    meta_type = 'EasyIcon'

    _properties=({'id':'title', 'type': 'string', 'mode':'w'},)


    def resize_image(self, REQUEST=None):
        """ttw"""
        file = StringIO(str(self.data))
        im = PIL.Image.open(file)

        if im.mode!='RGBA': im = im.convert("RGBA")
        im.load()
        if im.size[0]!=16 or im.size[1]!=16:
            im = im.resize((16, 16), PIL.Image.BICUBIC)

        newfile = StringIO()
        im.save(newfile, 'PNG', optimize=1)
        self.data=newfile.getvalue()
        self.content_type = 'image/png'
        self.size=len(self.data)
        self.width=16
        self.height=16
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def __init__(self, id, title, file):
        Image.__init__(self, id, title, file)
        if _PIL_is_present:
            self.resize_image()

    def getIcon(self):
        return self.virtualURL[1:]

    icon=getIcon

    def update_data(self, data, content_type=None, size=None):
        Image.update_data(self, data, content_type, size)
        if _PIL_is_present: self.resize_image()


    manage_editForm  =DTMLFile('dtml/EasyIcon/manage_editForm',globals(),
                               Kind='Image',kind='image')
    view_image_or_file =DTMLFile('dtml/EasyIcon/manage_imageView',globals())
    manage_editForm._setName('manage_editForm')
    manage=manage_main=manage_editForm
    manage_uploadForm=manage_editForm

    def manage_edit(self, title, content_type, precondition='',
                    filedata=None, REQUEST=None):
        """
        Changes the title and content type attributes of the File or Image.
        """
        if _PIL_is_present: self.resize_image()
        return Image.manage_edit(self, title, content_type, precondition=precondition,filedata=filedata, REQUEST=REQUEST)

Globals.InitializeClass(EasyIcon)




manage_addEasyIconForm = DTMLFile('dtml/EasyIcon/manage_addEasyIconForm', globals())

def manage_addEasyIcon(self, file, title='', REQUEST=None):
    """Add an EasyIcon"""
    ob=EasyIcon(ICONID, title, file)
    self._setObject(ICONID, ob)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyIcon,
        permission=EasyTemplateAdd,
        icon='icons/EasyIcon/EasyIcon.gif',
        constructors = ( manage_addEasyIconForm, manage_addEasyIcon ),
        meta_type = 'EasyIcon',
        interfaces=(EasyIconInterface,),
        visibility = None,
    )



