# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/EasyObservable.py,v 1.3 2003/03/02 20:00:02 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyObservable"""
__version__='$Revision: 1.3 $'[11:-2]


import Globals
from types import StringType

from ExtensionClass import Base
from OFS.DefaultObservable import DefaultObservable
from AccessControl import ClassSecurityInfo
from AccessControl import Permissions

# General Zope permissions
View = Permissions.view
AccessContentsInformation = Permissions.access_contents_information

import os
DEBUG_OBSERVABLE = os.environ.has_key('DEBUG_OBSERVABLE')


class NotifyException(Exception):
    pass

class Event(Base):
    event_type="Event"


class EasyObservable(DefaultObservable, Base):
    """

    """

    security = ClassSecurityInfo()

    _observers = {}

    def __init__(self):
        self._observers = {}

    def _normalize( self, observer ):
        # Assert that observer is not a string.
        if type( observer ) == StringType:
            observer = observer.split('/')
        return observer

    security.declarePrivate('registerObserver')
    def registerObserver( self, event_channel, observer, unique=1):
        normal = self._normalize( observer )
        if self.restrictedTraverse( normal, None ):
            observed = self._observers.get( event_channel, [] )
            observed=[]
            if (unique and normal not in observed) or not unique:
                observed.append( normal )
                self._observers[event_channel] = observed
                self._p_changed=1

    security.declarePrivate('unregisterObserver')
    def unregisterObserver( self, event_channel, observer ):
        observed = self._observers.get( event_channel, None )
        if observed:
            observed.remove(observer)
            self._observers[event_channel] = observed
            self._p_changed=1


    security.declarePrivate('notify')
    def notify( self, event_channel, event ):

        bozos = []

        for observer in self._observers.get(event_channel, []):

            obj = self.restrictedTraverse( observer, None )

            if obj is not None:
                try:
                    obj( self, event )
                except:
                    bozos.append( observer ) # Veto not allowed!
                    if DEBUG_OBSERVABLE:
                        import traceback
                        traceback.print_exc()
            else:
                bozos.append( observer )

        for bozo in bozos:
            try: # avoid race condition if unregister() called before now
                self._observers.remove( bozo )
            except:
                pass

    security.declareProtected(AccessContentsInformation, 'getObserved')
    def getObserved(self, event_channel=None):
        if event_channel is None or event_channel=='*':
            result=[]
            append=result.append
            for event_channel, observed in self._observers.items():
                for link in observed:
                    append( (event_type, link) )
            return result
        return map(lambda x, ec=event_channel: (ec, x) ,self._observers.get(event_channel,[]))

    security.declareProtected(AccessContentsInformation, 'isObserved')
    def isObserved( self, event_channel=None, observer=None):
        if observer:
            if event_channel is None or event_channel =='*':
                for observed in self._observers.values():
                    if observer in observed: return 1
            else:
                if observer in self._observers[event_channel]:
                    return 1
        return 0

    security.declarePrivate('registerBeforeDelete')
    def registerBeforeDelete( self, path):
        self.registerObserver('onBeforeDelete', path)

    security.declarePrivate('unregisterBeforeDelete')
    def unregisterBeforeDelete( self, path ):
        self.unregisterObserver('onBeforeDelete', path )

    security.declarePrivate('registerAfterAdd')
    def registerAfterAdd( self, path):
        self.registerObserver('AfterAdd', path)

    security.declarePrivate('unregisterAfterAdd')
    def unregisterAfterAdd( self, path ):
        self.unregisterObserver('AfterAdd', path)

    def manage_afterAdd(self, item, container):
        pass
        #self.notify(MasterCopyAdded.event_channel, MasterCopyAdded(self.getPhysicalPath()))

    def manage_beforeDelete(self, item, container):
        pass
        #self.notify(MasterCopyDeleted.event_channel, MasterCopyDeleted())



Globals.InitializeClass(EasyObservable)


