


from sgmllib import SGMLParser
from formatter import NullFormatter

class StringCache:

    def __init__(self):
        self._hash=[]
        self._cache={}

    def addEntry(self, text):
        h=hash(text)
        while h in self._hash:
            if text==self._cache[h]:
                #Add to hash count if string has been seen before
                self._hash.append(h)
                return h
            h=h+1
        self._hash.append(h)
        self._cache[h]=text
        return h

    def getEntry(self, h):
        return self._cache.get(h)

    def getAll(self):
        return self._cache

    def getAllHashes(self):
        return self._cache.keys()

    def getHistogram(self):
        hist={}
        for key in self._cache.keys():
            hist[key]=self._hash.count(key)
        return hist

class TabBase: pass

class Tag(TabBase):
    isContainer=1
    type='Tag'
    def __init__(self, tag, text, attrs=(), nc_tags=[]):
        self.tag=tag
        self.text=text
        self.attrs=attrs
        self.nc_tags=nc_tags
        kw={}
        for key, value in attrs: kw[key]=value
        self.kw=kw
        self.container=[]
        self.inner_hash=None
        self.outer_hash=None
        self.style=None

    def getString(self, cache=None, get_inner=0):
        if cache is not None:
            try:
              if get_inner:
                if self.inner_hash is not None:
                    return cache.getEntry(self.inner_hash)
              else:
                if self.outer_hash is not None:
                    return cache.getEntry(self.outer_hash)
            except: pass
        inner=''
        for elem in self.container:
            inner=inner+elem.getString(cache)
        if self.kw.has_key('class'):
            rtag='<%s class="%s">'%(self.tag, self.kw['class'])
        else:
            rtag="<%s>"%self.tag
        outer=rtag+inner
        if self.tag not in self.nc_tags:
            outer=outer+"</%s>"%self.tag
        if cache is not None:
            self.inner_hash = cache.addEntry(inner)
            self.outer_hash = cache.addEntry(outer)
        if get_inner: return inner
        return outer

    def render(self):
        inner=''
        for elem in self.container:
            inner=inner+elem.render()
        if self.tag=='root': return inner
        attr=''
        if self.kw.has_key('class'):
            attr=attr+' class="%s"'%(self.kw['class'],)
        if self.style is not None:
            attr=attr+' style="%s"'%(self.style,)
        return "<%s%s>%s</%s>" % (self.tag, attr, inner, self.tag)

class Text(TabBase):
    isContainer=0
    tag=None
    type='Text'
    def __init__(self, text):
        self.text=text
        self.inner_hash=None
        self.outer_hash=None
        self.style=None

    def getString(self, cache=None, get_inner=0):
        if cache is not None:
            try:
              if get_inner:
                if self.inner_hash is not None:
                    return cache.getEntry(self.inner_hash)
              else:
                if self.outer_hash is not None:
                    return cache.getEntry(self.outer_hash)
            except: pass
        outer = self.text
        if cache is not None:
            self.inner_hash = cache.addEntry(outer)
            self.outer_hash = cache.addEntry(outer)
        return outer

    def render(self):
        if self.style is not None:
            attr=' style="%s"'%(self.style,)
            return "<span%s>%s</span>"% (attr, self.text)
        return self.text

class CharRef(TabBase):
    isContainer=0
    tag=None
    type='CharRef'

    def __init__(self, ref):
        self.ref=ref
        self.inner_hash=None
        self.outer_hash=None
        self.style=None

    def getString(self, cache=None, get_inner=0):
        if cache is not None:
            try:
              if get_inner:
                if self.inner_hash is not None:
                    return cache.getEntry(self.inner_hash)
              else:
                if self.outer_hash is not None:
                    return cache.getEntry(self.outer_hash)
            except: pass
        outer = "&#%s;" % self.ref
        if cache is not None:
            self.inner_hash = cache.addEntry(self.ref)
            self.outer_hash = cache.addEntry(outer)
        if get_inner: return self.ref
        return outer

    def render(self):
        inner="&#%s;" % self.ref
        if self.style is not None:
            return '<span style="%s">%s</span>'%(self.style,inner)
        return inner

class EntityRef(TabBase):
    isContainer=0
    tag=None
    type='EntityRef'

    def __init__(self, ref):
        self.ref=ref
        self.inner_hash=None
        self.outer_hash=None
        self.style=None

    def getString(self, cache=None, get_inner=0):
        if cache is not None:
            try:
              if get_inner:
                if self.inner_hash is not None:
                    return cache.getEntry(self.inner_hash)
              else:
                if self.outer_hash is not None:
                    return cache.getEntry(self.outer_hash)
            except: pass
        outer = "&%s;" % self.ref
        if cache is not None:
            self.inner_hash = cache.addEntry(self.ref)
            self.outer_hash = cache.addEntry(outer)
        if get_inner: return self.ref
        return outer

    def render(self):
        inner="&%s;" % self.ref
        if self.style is not None:
            return '<span style="%s">%s</span>'%(self.style,inner)
        return inner

NO_CONTAINMENT_TAGS=['br','hr','img']

CANCEL_TAGS={
    'p':  ('p',)
  , 'h1': ('h1','h2','h3','h4','h5','h6',)
  , 'h2': ('h1','h2','h3','h4','h5','h6',)
  , 'h3': ('h1','h2','h3','h4','h5','h6',)
  , 'h4': ('h1','h2','h3','h4','h5','h6',)
  , 'h5': ('h1','h2','h3','h4','h5','h6',)
  , 'h6': ('h1','h2','h3','h4','h5','h6',)
  , 'tr': ('tr',)
  , 'td': ('td',)
}

class ExtendedHTMLParser(SGMLParser):

    def __init__(self, formater=None, verbose=0
                , no_containment_tags=NO_CONTAINMENT_TAGS
                , cancel_tags=CANCEL_TAGS
                ):
        SGMLParser.__init__(self, verbose=verbose)
        if formater is not None: pass
        self.no_containment_tags=no_containment_tags
        #Needs to handle the unXML like features of HTML (or use Tidy?)
        self.cancel_tags=cancel_tags
        self.has_seen_tag={}
        self.active_tags=[]
        self.root=Tag('root','<root>')
        self.current=[]
        self.current.append(self.root)

    def find_active_tag(self, tags):
        return filter(lambda x, atags=self.active_tags: x in atags, tags)

    def unknown_starttag(self, tag, attrs):
        new_tag=Tag(tag, self.get_starttag_text(), attrs, self.no_containment_tags)
         #This new tag cansels the current container (or any above) tag.
        pop_tags=self.find_active_tag(self.cancel_tags.get(tag, ()))
        while len(pop_tags):
            if self.current[-1].tag in pop_tags:
                pop_tags.remove(self.current[-1].tag)
            self.current.pop()
        self.current[-1].container.append(new_tag)
        if tag not in self.no_containment_tags:
            self.active_tags.append(tag)
            self.current.append(new_tag)

    def unknown_endtag(self, tag):
        if self.current and self.current[-1]==self.root:
            raise "InvalideTag", "Can't pop more"
        if tag not in self.no_containment_tags:
            self.active_tags.remove(tag)
        self.current.pop()

    def handle_charref(self, ref):
        self.current[-1].container.append(CharRef(ref))

    def handle_entityref(self, ref):
        self.current[-1].container.append(EntityRef(ref))

    def getDOM(self):
        return [self.root]

    def report_unbalanced(self, tag):
        #Needs to handle the unXML like features of HTML (or use Tidy?)
        pass #print "report_unbalanced </%s>" % ref

    def handle_data(self, data):
        self.current[-1].container.append(Text(data))

HALTED=0
INSERT_MODE=1
REMOVE_MODE=2
COMPARING=3

ADDED='ADDED'
REMOVED='REMOVED'
CHANGED='CHANGED'
CLASS_CHANGED='CLASS_CHANGED'
CHANGE_STYLES={
    'default': {
                 ADDED:'text-decoration:underline;color:#0C0;'
               , REMOVED:'text-decoration:line-through;color:#F00;'
               , CHANGED:'text-decoration:underline;color:#60c;'
               , CLASS_CHANGED:'text-decoration:overline;color:#60c;'
               },
}
CRLF=Text('\n')

class HTMLDiff:

    def __init__(self, prev=None, next=None
                , change_styles=CHANGE_STYLES
                , no_containment_tags=NO_CONTAINMENT_TAGS
                , cancel_tags=CANCEL_TAGS
    ):
        self.prev=prev
        self.next=next
        #self.state=HALTED
        self.cache=StringCache()
        self.is_in_prev=[]
        self.is_in_next=[]
        self.has_seen_in_prev=[]
        self.has_seen_in_next=[]
        self.change_styles=change_styles
        self.no_containment_tags=no_containment_tags
        self.cancel_tags=cancel_tags
        self.root=Tag('root','<root>')
        self.current=[]
        self.current.append(self.root)

    def AddToOutput(self, new_tag):
        self.current[-1].container.append(new_tag)
        #if new_tag.tag not in self.no_containment_tags:
        #    self.current.append(new_tag)

    def PushLastTagOnStack(self):
        if len(self.current) and len(self.current.container):
            tag=self.current[-1].container[-1]
            self.current.append(tag)

    def PopLastTag(self, new_tag):
        if len(self.current):
            self.current.pop()


    def getStyle(self, tag, type_of_change):
        definition=self.change_styles.get(tag,None)
        if definition is None:
            definition=self.change_styles.get('default')
        return definition.get(type_of_change, '')

    def collectHash(self, container, is_in_append):
        for elem in container:
            elem.getString(self.cache)
            is_in_append(elem.outer_hash)
            if elem.isContainer:
                self.collectHash(elem.container, is_in_append)

    def create_hash_indexes(self):
        self.collectHash(self.prev,self.is_in_prev.append)
        self.collectHash(self.next,self.is_in_next.append)

    def markNextAsSeen(self, tag):
        self.has_seen_in_next.append(tag.outer_hash)
        self.has_seen_in_next.append(tag.inner_hash)

    def markPrevAsSeen(self, tag):
        self.has_seen_in_prev.append(tag.outer_hash)
        self.has_seen_in_prev.append(tag.inner_hash)

    def isInPrev(self, hash_id):
        return hash_id in self.is_in_prev

    def isInNext(self, hash_id):
        return hash_id in self.is_in_next

    def hasBeenSeenPrev(self, hash_id):
        return hash_id in self.has_seen_in_prev

    def hasBeenSeenNext(self, hash_id):
        return hash_id in self.has_seen_in_next

    def compare_tag(self, prev_col, next_col):
        pointer_prev=0
        pointer_next=0
        while pointer_next<len(next_col) and pointer_prev<len(prev_col):
            next=next_col[pointer_next]
            prev=prev_col[pointer_prev]
            if next.outer_hash==prev.outer_hash:
                #Tag equal
                self.AddToOutput(next)
                self.markNextAsSeen(next); pointer_next+=1
                self.markPrevAsSeen(prev); pointer_prev+=1
            elif next.inner_hash==prev.inner_hash:
                #Inner Text equal
                next.style=self.getStyle(next.tag, CLASS_CHANGED)
                self.AddToOutput(next)
                self.markNextAsSeen(next); pointer_next+=1
                self.markPrevAsSeen(prev); pointer_prev+=1
            elif not self.isInPrev(next.outer_hash):
                #New TAG
                next.style=self.getStyle(next.tag, ADDED)
                self.AddToOutput(next)
                self.markNextAsSeen(next); pointer_next+=1
            elif not self.isInNext(prev.outer_hash):
                #Removed
                prev.style=self.getStyle(prev.tag, REMOVED)
                self.AddToOutput(CRLF)
                self.AddToOutput(prev)
                self.markPrevAsSeen(prev); pointer_prev+=1
            else:
                #differ
                self.markNextAsSeen(next); pointer_next+=1
                self.markPrevAsSeen(prev); pointer_prev+=1
        while pointer_next<len(next_col):
            next=next_col[pointer_next]
            next.style=self.getStyle(next.tag, ADDED)
            self.AddToOutput(next)
            self.markNextAsSeen(next); pointer_next+=1
        while pointer_prev<len(prev_col):
            prev=prev_col[pointer_prev]
            prev.style=self.getStyle(prev.tag, REMOVED)
            self.AddToOutput(prev)
            self.markPrevAsSeen(prev); pointer_prev+=1
        #while pointer_next<len(next_col) or pointer_prev<len(prev_col):



    def process(self):
        self.create_hash_indexes()
        self.compare_tag(self.prev[0].container, self.next[0].container)

    def render(self):
        return self.root.render()


def recursive(container, cache):
    for each in container:
        #print each.type, each.tag, each.getString(cache)
        if each.isContainer:
            recursive(each.container, cache)


DEBUG_CHANGE_STYLES={
    'default': {
                 ADDED:ADDED
               , REMOVED:REMOVED
               , CHANGED:CHANGED
               , CLASS_CHANGED:CLASS_CHANGED
               },
}
def main(arg):

    loada=open("tests\\htmltest\\Test1a.html", "r")
    loadb=open("tests\\htmltest\\Test1b.html", "r")
    savefile=open("tests\\htmltest\\Test1out.html", "w")
    loada_html=loada.read()
    loadb_html=loadb.read()
    prev=ExtendedHTMLParser()
    prev.feed(loada_html)
    prev.close()
    container_prev=prev.getDOM()
    next=ExtendedHTMLParser()
    next.feed(loadb_html)
    next.close()
    container_next=next.getDOM()

    if 1:
        diffObj=HTMLDiff(container_prev,container_next,change_styles=CHANGE_STYLES)
        diffObj.process()
        savefile.write(diffObj.render())

    else:
        diffObj=HTMLDiff(container_prev,container_next,change_styles=DEBUG_CHANGE_STYLES)
        diffObj.process()
        #print diffObj.render()

    loada.close()
    loadb.close()
    savefile.close()

if __name__=='__main__':
    import sys
    main(sys.argv)

