# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/ObjectManager/OrderedObjectManagerIF.py,v 1.5 2003/12/02 08:44:06 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyCore.ObjectManager.ObjectManagerIF"""
__version__='$Revision: 1.5 $'[11:-2]

import Globals
from Globals import DTMLFile
from types import StringType, ListType, TupleType
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager

#import Interface

from OFS.ObjectManager import IFAwareObjectManager, BreakoutException

from Products.EasyCore.ObjectManager.OrderedObjectManager import OrderedObjectManager


from App.version_txt import getZopeVersion
zope_version=getZopeVersion()


from Products.EasyCore import MLOG_TAB


ViewManagementScreens='View management screens'


class OrderedObjectManagerIF(IFAwareObjectManager, OrderedObjectManager):

    security = ClassSecurityInfo()

    manage_options=({'label':'Contents', 'action':'manage_main',},)


    security.declareProtected(ViewManagementScreens, 'manage_main')
    if zope_version[0:3]>=(2,5,1) and zope_version[0:3]<(2,6,1):
        manage_main=DTMLFile('dtml/251/ordered_if_manage_main', globals())
    elif zope_version[0:3]>=(2,6,1) and zope_version[0:3]<(2,7,0):
        manage_main=DTMLFile('dtml/261/ordered_if_manage_main', globals())
    elif zope_version[0:3]==(2,6,2):
        manage_main=DTMLFile('dtml/262/ordered_if_manage_main', globals())
    elif zope_version[0:3]==(2,7,0):
        manage_main=DTMLFile('dtml/270/ordered_if_manage_main', globals())
    else:
        manage_main=DTMLFile('dtml/261/ordered_if_manage_main', globals())



    def getInterfaceIds(self):
        interfaces=getInterfaces
        return map(lambda x: x.getName(), interfaces)


    def getInterfaces(self):
        return getattr(self, __interfaces__, ())


    def filtered_meta_types(self, user=None, spec=None, interfaces=None):
        # Return a list of the types for which the user has
        # adequate permission to add that type of object.
        user=getSecurityManager().getUser()
        if type(spec) == StringType:
            spec=[spec]
        elif type(spec) not in (ListType, TupleType):
            spec=None
        meta_types=[]
        if callable(self.all_meta_types):
            all=self.all_meta_types(interfaces)
        else:
            all=self.all_meta_types
        for meta_type in all:
            if spec is None or meta_type in spec:
                if meta_type.has_key('permission'):
                    if user.has_permission(meta_type['permission'],self):
                        meta_types.append(meta_type)
                else:
                    meta_types.append(meta_type)
        return meta_types


    def objectIds(self, spec=None, interfaces=None):
        # Returns a list of subobject ids of the current object.
        # If 'spec' is specified, returns objects whose meta_type
        # matches 'spec'.

        if interfaces is not None:
            if type(interfaces) not in (ListType, TupleType):
                interfaces=[interfaces]
            meta_types=map(lambda x: x['name'], self.all_meta_types(interfaces))
        else:
            meta_types=[]
        if spec is None: spec=[]
        elif type(spec)==type('s'):
            spec=[spec]
        meta_types.extend(list(spec))
        if meta_types:
            set=[]
            for ob in self._objects:
                if ob['meta_type'] in meta_types:
                    set.append(ob['id'])
            return set
        return map(lambda i: i['id'], self._objects)


    def objectValues(self, spec=None, interfaces=None):
        # Returns a list of actual subobjects of the current object.
        # If 'spec' is specified, returns only objects whose meta_type
        # match 'spec'.
        return map(self._getOb, self.objectIds(spec, interfaces))


    def objectItems(self, spec=None, interfaces=None):
        # Returns a list of (id, subobject) tuples of the current object.
        # If 'spec' is specified, returns only objects whose meta_type match
        # 'spec'
        r=[]
        a=r.append
        g=self._getOb
        for id in self.objectIds(spec, interfaces): a((id, g(id)))
        return r


Globals.InitializeClass(OrderedObjectManagerIF)

