# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/PropertyTextParser.py,v 1.5 2003/04/13 12:04:31 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""PropertyTextParser
Manage text representation of properties and
other types of objects.

$Colliberty: PropertyTextParser.py,v 0.0.1 2001/06/04 22:33:00 johanc Exp $"""
__version__='$Revision: 1.5 $'[11:-2]

import sys
from Acquisition import aq_base
from types import DictType, ListType, StringType, FloatType, IntType, LongType, TupleType
from DateTime import DateTime
from cStringIO import StringIO
from ConfigParser import ConfigParser

import re
trailing_digits=re.compile(r'[0-9]*$')

from ZPublisher.Converters import type_converters
all_types=type_converters.keys()

_marker=[]

def pset(parser, section, key, value, p_type):
    if p_type in ('lines','text'):
        lines = value.split('\n')
        counter=0
        for line in lines:
            counter=counter+1
            parser.set(section, key+str(counter), line)
    else:
        parser.set(section, key, value)

def guess_property_type(value, default='string', name=None):
    """\
    Guesses the property type based on:
      - If name is submitted and name contains a '.' seperating
        the "real" name and a valid type
      - If the value matches any of the standard Types
        a coresponding property type is returned
      - The value of default is returned
    """
    if name and type(name) is StringType and name.count('.'):
        name, ntype=name.split('.')[0:2]
        if ntype in all_types: return ntype
    if type(value) == ListType: return 'lines'
    elif type(value) == FloatType: return 'float'
    elif type(value) == IntType: return 'int'
    elif type(value) == LongType: return 'long'
    elif type(value) == type(DateTime()): return 'date'
    elif type(value) == TupleType: return 'tokens'
    elif hasattr(value,'find') and callable(value.find) and value.find('\n')!=-1:
        return 'text'
    elif type(value) == StringType: return 'string'
    return default


def norm_precistance_list(value):
    storage=[]
    if type(value) not in (TupleType,ListType):
        value = [value]
    for p_store in value:
        if len(p_store)==2:
            storage.append(p_store +(None, None))
        elif len(p_store)==3:
            storage.append(p_store +(None,))
        elif len(p_store)==4:
            storage.append(p_store)
    return tuple(storage)

class PropertyTextParser:

    _result=None
    _errors=[]

    def readBody(self, body, defaults=None):
        #Doesn't really manage properties, this should be changed.
        try:
            result={}
            if defaults is None: defaults={}
            file=StringIO(body)
        except:
            self._errors.append("%s: %s" % sys.exc_info()[:2])
        try:
            parser=ConfigParser(defaults)
            parser.readfp(file)
        except:
            self._errors.append("%s: %s" % sys.exc_info()[:2])
        try:
            for section in parser.sections():
                opts={}
                for option in parser.options(section):
                    value=parser.get(section, option)
                    digit_start=trailing_digits.search(option).start()
                    row=None
                    try: row=int(option[digit_start:])
                    except: pass
                    if row is not None or opts.has_key(option[:digit_start]):
                        new_option=option[:digit_start]
                        if not new_option: new_option=0
                        lines=opts.get(option[:digit_start],{})
                        if type(lines)!=DictType: lines={0:lines}
                        lines[row]=value
                        opts[option[:digit_start]]=lines
                    else:
                        opts[option]=value
                multirows=filter(lambda value: type(value[1])==DictType, opts.items())
                for prop_name,lines in multirows:
                    rows_ids=lines.keys()
                    rows_ids.sort()
                    opts[prop_name]='\n'.join(map(lambda row, lines=lines: lines[row], rows_ids))
                    opts['multiline']=1
                if not opts.has_key('type'):
                    opts['type']='string'
                result[section]=opts
        except:
            self._errors.append("%s: %s" % sys.exc_info()[:2])

        self._result=result
        return result

    def parseToInstance(self, instance, definitions, body, defaults=None):
        #Doesn't really manage properties, this should be changed.
        instance=aq_base(instance)
        dict = self.readBody(body, defaults=defaults)
        for definition in definitions:
            p_id = definition.get('id',None)
            p_label = definition.get('label',None)
            #p_store = definition.get('store',None)
            p_type = definition.get('type',None)
            section = dict.get(p_label, None)
            if section is None: section = dict.get(p_id, None)
            if p_id and section is not None:
                #if not p_store:
                p_store=(('value',p_id,p_type,None),)
                p_storages=norm_precistance_list(p_store)
                for v_id, v_key, v_type, v_default in p_storages:
                    v_value=section.get(v_id, v_default)
                    setattr(instance, v_key, v_value)

    def getCode(self):
        return self._result

    def getErrors(self):
        return self._errors




class PropertyTextGenerator:

    _result=None
    _errors=[]

    def renderFromInstance(self, instance, definitions, **kw):
        """
        definition = (
                        {
                            'id':'name',
                            'label':'Possible Languages',
                            'type': 'lines',
                            'mode': 'w',
                            'multiline':1
                         },
                     )
        """
        #Doesn't really manage properties, this should be changed.
        parser=ConfigParser()
        reserved_keys=('store','label')#default?
        for definition in definitions:
            keys = definition.keys()
            p_id = definition.get('id',None)
            p_label = definition.get('label',None)
            p_type = definition.get('type','string')
            #p_store = definition.get('store',None)
            if p_id:
                section=p_label or p_id
                parser.add_section(section)
                if p_type: parser.set(section, 'type', p_type)
                for key in keys:
                    d_value = definition.get(key,None)
                    if d_value and key not in reserved_keys:
                        pset(parser, section, key, d_value, guess_property_type(d_value))
                #if not p_store:
                p_store=(('value',p_id,p_type,None),)
                p_storages=norm_precistance_list(p_store)
                for l_id, l_key, l_type, l_default in p_storages:
                    value = getattr(instance, l_key, _marker)
                    if value is _marker: value = kw.get(l_key, _marker)
                    if value is _marker: value = definition.get(l_default,None)
                    pset(parser, section, l_id, value, l_type)

        file=StringIO()
        parser.write(file)
        self._result=file.getvalue()
        return self._result


    def getCode(self):
        return self._result

    def getErrors(self):
        return self._errors
