# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/TagUtils.py,v 1.3 2003/07/11 07:22:48 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''TagUtils
$Id: TagUtils.py,v 1.3 2003/07/11 07:22:48 johanc Exp $'''
__version__='$Revision: 1.3 $'[11:-2]

import Globals
import re
import string
from types import StringType, DictType
from DocumentTemplate.DT_Util   import Eval, ParseError, render_blocks, html_quote
from DocumentTemplate.DT_Util   import parse_params, InstanceDict
from Acquisition                import Implicit

from Products.EasyCore import MLOG_TAB


KEEP_SPACES=0
REMOVE_SPACES=1
INTEND_HTML=2

SILENT_PARAM='silent'
STRING_PARAM=''
EXEC_PARAM='exec'
SEXEC_PARAM='string'
CONDITIONAL_PARAM='conditional'
OPTIONAL_PARAM='optional'
REQUIRED_PARAM='required'


OPTIONAL=0
REQUIRED=1

_marker=[]

# attributes={key:{id:,value:,default:,required:,type:}
# attributes={
#    'key':
#       {
#       , 'id':         'the id of the attribute to add, if not awailable the key is used'
#                       # optional, defaults to the value of key
#       , 'value':      'value or exec or exec string'  # default to ''
#       , 'default':    'default value'                 # optional, default to ''
#       , 'required':   OPTIONAL or REQUIRED            # optional, defaults to OPTIONAL
#       , 'type':       STRING_PARAM or EXEC_PARAM or SEXEC_PARAM,    # defaults to STRING_PARAM
#       },
# }


class SessionData:

    def __init__(self, id, dict, **kw):
        self.id=id
        d={}
        if type(dict) is DictType:
            d.update(dict)
        d.update(kw)
        self.__dict__=d

    def get(self, key, default=None):
        return self.__dict__.get(key,default)

    def set(self, key, value):
        self.__dict__[key]=value



class SessionDataContainer:

    def __init__(self, id):
        self.id=id

    def add(self, key, dict, **kw):
        return self.__dict__.get(key,default)

    def remove(self, key):
        dict

    def get(self, key):
        return self.__dict__[key]



class SessionProxy: #(Implicit):

    def __init__(self, session_name):
        self.session_name=id
        self._sd=None

    def setSessionData(self, sd):
        self._sd=sd

    def getSessionProxy(self):
        return self

    def getSessionDataFromProxy(self, do_raise=1):
        if self._sd is None:
            if do_raise:
                raise "SessionProxy not initialized"
            else:
                return None
        return self._sd


class TagBase: #(Implicit):

    name="tag"
    _dont_save_ids=('name','expr','template',)
    _save_ids=('condition','space_policy', 'passby_counter', 'add_new_line', )
    _dont_render=('condition','space_policy', 'passby_counter', 'add_new_line', 'session', 'persist')
    expr=None
    condition=1
    template="<%(tag)s%(pre)s%(render)s/>"
    pre_rendered_attr=''
    space_policy=KEEP_SPACES
    passby_counter=0
    raw_attr=''
    add_new_line=1

    #session=None
    #persist=()

    attributes={}


    def __init__(self, attributes):
        """\
            __init__(self, attributes) --> Initialize tag object; return None
        """
        if hasattr(self, 'blockContinuations'):
            tname, attributes_text, section = attributes[0]
            self.name=tname
            self.section = section
            self.raw_attr=' '+attributes_text
            self.parse_params(attributes_text)
        else:
            self.section = None
            self.raw_attr=' '+attributes
            self.parse_params(attributes)

    def handle_reserved(self, vkey, value):
        if vkey=='condition':
            self.condition=value
        elif vkey=='space_policy':
            if value in (REMOVE_SPACES, INTEND_HTML):
                self.space_policy=value
            else:
                self.space_policy=KEEP_SPACES
        elif vkey=='passby_counter':
            try:
                self.passby_counter=int(value)
            except:
                self.passby_counter=0
        else:
            setattr(self, vkey, value)


    def parse_params(self, text,
                     parmre=re.compile('([\000- ]*([^\000- ="]+)=([^\000- ="]+))'),
                     qparmre=re.compile('([\000- ]*([^\000- ="]+)="([^"]*)")'),
                     sexecre=re.compile('([^{}]+)|{([^{}]+)}'),
                     attrcodingre=re.compile('(css:)|(:exec)|(:conditional)|(:string)|(:required)|(:optional)|(:silent)|([^:]+)'),
                     ):

        """\
        Parse tag parameters
        """
        attributes=self.attributes
        mo_p = parmre.findall(text)
        mo_q = qparmre.findall(text)
        #IGNORE_PARAM
        for text, key, value in mo_p+mo_q:
            skey=map(lambda x: filter(None, x)[0], attrcodingre.findall(key))
            sarg=filter(lambda x: x[0]==':', skey)
            spre=map(lambda x: x[:-1], filter(lambda x: x[-1]==':', skey))
            skey=filter(lambda x: x[-1]!=':' and x[0]!=':', skey)
            vkey='_'.join(spre+skey)


            attribute={'id':vkey, 'value':value, 'type':SILENT_PARAM, 'dont_include':vkey in self._dont_render}
            if ':'+EXEC_PARAM in sarg:
                attribute['type']=EXEC_PARAM
            elif ':'+SEXEC_PARAM in sarg:
                attribute['value']=tuple(sexecre.findall(value))
                attribute['type']=EXEC_PARAM
            elif ':'+CONDITIONAL_PARAM in sarg:
                attribute['type']=CONDITIONAL_PARAM
            else:
                if vkey in self._save_ids and vkey not in self._dont_save_ids:
                    self.handle_reserved(vkey, value)
                attribute['type']=STRING_PARAM

            if ':'+OPTIONAL_PARAM in sarg:
                attribute['required']=0
            elif ':'+REQUIRED_PARAM in sarg:
                attribute['required']=1

            if ':'+SILENT_PARAM in sarg:
                attribute['dont_include']=1

            attributes[vkey]=attribute

        if not self.attributes.has_key('id'):
            raise "TagParserError", "There must exist an argument called id in tag 'dtml-%s'" % self.name
        self.pre_render_attr(attributes)
        self.attributes=attributes


    def pre_render_attr(self, attributes):
        pre_rendered=''
        exec_in_order=[]
        for key, vargument in self.attributes.items():
            if key not in self._dont_render+self._save_ids:
                if vargument.get('type',STRING_PARAM)==STRING_PARAM:
                    vvalue=vargument.get('value', None)
                    if vvalue is not None and vargument.get('required', 0):
                        vvalue=vargument.get('default', None)
                    if vvalue is not None:
                        pre_rendered=pre_rendered+' %s="%s"' % (key, vvalue)
        self.pre_rendered_attr=pre_rendered


    def do_computed_attr(self, md, object):

        vinsert=''
        for vkey, argument in self.attributes.items():
            vtype=argument.get('type', STRING_PARAM)
            vvalue=argument.get('value', None)
            if vvalue is None and argument.get('required', 0):
                vvalue=argument.get('default', None)

            vrendered=''
            if vtype==EXEC_PARAM and vvalue is not None:
                try: vrendered=str(Eval(vvalue).eval(md))
                except:pass
            elif vtype==SEXEC_PARAM and vvalue is not None:
                try:
                    for svalue, sexec in value:
                        if svalue: vrendered=vrendered+svalue
                        elif sexec: vrendered=vrendered+str(Eval(sexec).eval(md))
                except:pass
            elif vtype==CONDITIONAL_PARAM and vvalue is not None:
                try:vrendered=Eval(vvalue).eval(md)
                except:pass
            if vrendered:
                if vkey in self._save_ids and vkey not in self._dont_save_ids:
                    setattr(self, vkey, vrendered)
                if not argument.get('dont_include', 1):
                    vinsert=vinsert+' %s="%s"' % (vkey, vrendered)

        return vinsert



    def setup_session(self, md, object):
        session=self.attributes.get('session', None)
        if session is not None and not hasattr(self, '_session_proxy'):
            sdm=getattr(object, 'session_data_manager', None)
            session_value=session.get('value', 'default')
            if session_value is not None and sdm is not None:
                vtype=session.get('type',STRING_PARAM)
                session_name=None
                if vtype in (EXEC_PARAM, CONDITIONAL_PARAM) and session_value is not None:
                    try: session_name = Eval(session_value).eval(md)
                    except: pass
                elif vtype==SEXEC_PARAM and session_value is not None:
                    try:
                        session_name=''
                        for svalue, sexec in session_value:
                            if svalue: session_name=session_name+svalue
                            elif sexec: session_name=session_name+str(Eval(sexec).eval(md))
                    except: pass
                else:
                    session_name=session_value
                if session_name is not None:
                    sd=sdm.getSessionData(create=1)
                    session_proxy = sd.get(session_name, None)
                    if session_proxy is None:
                        session_proxy=SessionProxy(session_name)
                    self._session_proxy=session_proxy

    def handle_session(self, md, object):
        s_proxy=getattr(md, '_session_proxy', None)
        if s_proxy is not None:
            persist=getattr(self, 'persist', None)
            if persist is not None:
                if type(persist) == StringType:
                    persist=persist.split(' ')
                for key in persist:
                    value=getattr(md, key, _marker)
                    if value is _marker:
                        value=getattr(object, key, _marker)
                    if value is _marker:
                        value=getattr(self, key, _marker)
                    if value is not _marker:
                        session_proxy.set(key, value)

            session_proxy=InstanceDict(session_proxy, md)


    def render(self, md, raw_re=re.compile('(.*passby_counter=")([0-9]+)(".*)', re.DOTALL)):
        """\
        render(self, md) --> Do the Translation; return string
        """
        try: eval_cond=Eval(str(self.condition)).eval(md)
        except: eval_cond=0
        if not eval_cond: return

        if self.passby_counter>0:
            raw=self.read_raw()
            pre, value, post = raw_re.findall(raw)[0]
            try: value=max(int(value)-1,0)
            except: value=0
            output=pre + str(value) + post

        else:
            object  = md.this
            self.md=md
            self.setup_session(md, object)
            if hasattr(self, '_session_proxy'):
                md._push(self._session_proxy)

            self.handle_session(md, object)

            output=self.do_render(md, object)

            if hasattr(self, '_session_proxy'):
                md._pop()

        return output


    __call__=render




    def __getattr__(self, name):
        md = getattr(self, 'md', None)
        if md is not None:
            vargument=self.attributes.get(name, None)
            if vargument is not None:
                vtype=vargument.get('type', STRING_PARAM)
                vvalue=vargument.get('value', None)
                if vvalue is None and argument.get('required', 0):
                    vvalue=argument.get('default', None)
                if vtype==EXEC_PARAM and vvalue is not None:
                    return Eval(vvalue).eval(md)
                elif vtype==SEXEC_PARAM and vvalue is not None:
                    vrendered=''
                    for svalue, sexec in vvalue:
                        if svalue: vrendered=vrendered+svalue
                        elif sexec: vrendered=vrendered+str(Eval(sexec).eval(md))
                    return vrendered
                elif vtype==CONDITIONAL_PARAM and vvalue is not None:
                    try:return Eval(vvalue).eval(md)
                    except: return None
                elif vvalue is not None:
                    return vvalue
        raise AttributeError, name

    #getitem=__getattr__

    def read_raw(self):
        return self.section.read_raw()


class SingleTagBase(TagBase):
    """\
    """

    name = 'single'
    condition=1

    template="<%(tag)s%(pre)s%(rendered_attr)s/>%(newline)s"

    def do_render(self, md, object):
        rendered_attr = self.do_computed_attr(md, object)
        return self.template % {
                  'tag':self.name
                , 'pre':self.pre_rendered_attr
                , 'rendered_attr': rendered_attr
                , 'newline':self.add_new_line and '\n' or ''
          }

    def read_raw(self):
        return self.template % {
                'tag':self.name
              , 'pre':self.raw_attr
              , 'rendered_attr':''
              , 'newline':self.add_new_line and '\n' or ''
          }



class ContainerTagBase(TagBase):
    """\
    """


    name = 'container'
    condition=1
    blockContinuations=()
    template="<%(tag)s%(pre)s%(rendered_attr)s>%(content)s</%(tag)s>%(newline)s"


    _dont_save_ids=TagBase._dont_save_ids+('blockContinuations',)


    def render_blocks(self, md):
        blocks=self.section.blocks
        if self.space_policy==REMOVE_SPACES:
            blocks=[]
            for block in self.section.blocks:
                if type(block)!=StringType or block.strip():
                    blocks.append(block)
            return string.strip(render_blocks(blocks, md))
        #if self.space_policy==INTEND_HTML:
        return render_blocks(blocks, md)

    def do_render(self, md, object):
        newline=self.add_new_line and '\n' or ''
        rendered_attr = self.do_computed_attr(md, object)
        content=self.render_blocks(md)
        return self.template % {
              'tag':self.name
            , 'pre':self.pre_rendered_attr
            , 'rendered_attr': rendered_attr
            , 'content':content
            , 'newline':self.add_new_line and '\n' or ''
            }


    def read_raw(self):
        return self.template % {
                  'tag':self.name
                , 'pre':self.raw_attr
                , 'rendered_attr':''
                , 'content':self.section.read_raw()
                ,'newline':self.add_new_line and '\n' or ''
                }


def strip_html(template, find_space_in_valid_rows=re.compile('([ ]*)[^ \n]+[^\n]*')):
    template=template.replace('\r\n','\n')
    template=template.replace('\n\r','\n')
    template=template.replace('\r','\n')
    srows=find_space_in_valid_rows.findall(template)
    if srows:
        ind=len(min(srows))
        temp=''
        for r in template.split('\n'):
            temp=temp+r[ind:]+'\n'
        return temp
    return template

def HTML(template, globals):
    return Globals.HTML(strip_html(template), globals)