# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/UpgradeSupport/Utils.py,v 1.13 2003/09/11 18:40:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''UpgradeSupport Utilities.'''
__version__='1.7.5'



from zLOG import LOG, INFO, ERROR

try: from DocumentTemplate.cDocumentTemplate import DictInstance
except: from DocumentTemplate.pDocumentTemplate import DictInstance

from Products.ZCatalog.Vocabulary import Vocabulary
from Products.EasyCore.Utils.DebugUtils import MLOG_TAB, C_MLOG_TAB

def upgrade_indexes(self,
          changed=0
        , default_indexes={}
        , deprecated_indexes=()
        , required_indexes=()
        , default_columns=()
        , deprecated_columns=()
        , required_columns=()
    ):
    upgraded_indexes=[]
    self_id=self.getId()
    self_path='/'.join(self.getPhysicalPath())

    if self.objectIds(spec=['Vocabulary']):
        for vocab in self.objectValues(spec=['Vocabulary']):
            if not hasattr(vocab,'useSplitter'):
                voc_id=vocab.id
                v = Vocabulary(voc_id, vocab.title, globbing=vocab.globbing)
                self._delObject(voc_id)
                self._setObject(voc_id, v)
                text="Vocabulary '%s' has been upgraded in (%s)." % (voc_id, self_path )
                LOG(self.__class__.__name__, INFO, "Vocabulary Upgraded", text)
                changed=1
                upgraded_indexes.append(text)
    else:
        v = Vocabulary('Vocabulary', 'Vocabulary', globbing=1)
        self._setObject('Vocabulary', v)
        text="Default Vocabulary 'Vocabulary' has been added to (%s)." % (self_path,)
        LOG(self.__class__.__name__, INFO, "Default Vocabulary Added", text)
        changed=1
        upgraded_indexes.append(text)

    indexes=self.index_objects()
    for index in indexes:
        index_type=index.__class__.__name__
        if index_type[0:2]=='Un':
            index_name=index.id
            index_type=index_type[2:]
            if index_type=='TextIndex':
                vocabulary_id=getattr(index, 'vocabulary_id', 'Vocabulary')
                extra=DictInstance({'vocabulary':vocabulary_id})
                text="In the object %s (%s) has the index %s of type %s has been upgraded to type %s with extra=%s. It will need reindex." %(self_id, self_path, index_name, index.__class__.__module__+'.'+index.__class__.__name__, index_type, extra)
                upgraded_indexes.append(text)
                self.delIndex(index_name, force=1, dont_raise=1)
                self.addIndex(index_name, index_type, extra)
            else:
                text="In the object %s (%s) has the index %s of type %s has been upgraded to type %s. It will need reindex." %(self_id, self_path, index_name, index.__class__.__module__+'.'+index.__class__.__name__, index_type)
                upgraded_indexes.append(text)
                self.delIndex(index_name, force=1, dont_raise=1)
                self.addIndex(index_name, index_type)
            changed=1

    LOG("Upgrade Indexes", INFO, "Begin", self_path)
    indexes=self.indexes()
    for id in deprecated_indexes:
        if id in indexes:
            self.delIndex(id, force=1, dont_raise=1)
            text="Removed index %s in %s" %(id, self_path)
            upgraded_indexes.append(text)
    schema=self.schema()
    for id in deprecated_columns:
        if id in schema:
            self.delColumn(id, force=1, dont_raise=1)
            text="Removed column %s in %s" %(id, self_path)
            upgraded_indexes.append(text)

    indexes=self.indexes()
    for id, type in default_indexes.items():
        if id in indexes and getattr(self._catalog.indexes[id],'meta_type', None)!=type:
            self.delIndex(id, force=1, dont_raise=1)
            changed=1
        if id not in indexes:
            self.addIndex(id, type)
            text="Added index %s of type %s in %s" %(id, type, self_path)
            upgraded_indexes.append(text)
            changed=1
    schema=self.schema()
    for id in default_columns:
        if id not in schema:
            self.addColumn(id)
            text="Added column %s in %s" %(id, self_path)
            upgraded_indexes.append(text)
            changed=1
    indexes=self.indexes()
    for id, type in required_indexes.items():
        if id in indexes and getattr(self._catalog.indexes[id],'meta_type', None)!=type:
            self.delIndex(id, force=1, dont_raise=1)
            changed=1
        if id not in indexes:
            self.addIndex(id, type)
            text="Added index %s of type %s in %s" %(id, type, self_path)
            upgraded_indexes.append(text)
            changed=1
    schema=self.schema()
    for id in required_columns:
        if id not in schema:
            self.addColumn(id)
            text="Added column %s in %s" %(id, self_path)
            upgraded_indexes.append(text)
            changed=1
    get_transaction().commit(1)
    self.refreshCatalog()
    get_transaction().commit(1)
    LOG("Upgrade Indexes", INFO, "Finished", self_path)

    return changed, upgraded_indexes


def changeProperty(properties, values):
    for prop in properties:
        if prop['id']==values['id']:
            for key, value in values.items():
                prop[key]=value

def upgradeProperties(obj, defaults=None, remove_ids=()):
    if defaults is None:defaults={}
    props=obj._properties
    props_keys=map(lambda x: x['id'], props)
    cprops=obj.__class__._properties
    for cprop in cprops:
        if cprop['id'] not in props_keys:
            if not defaults.has_key(cprop['id']) and not hasattr(obj, cprop['id']):
                raise "upgradeProperties: property '%s' needs a init value." % cprop['id']
            setattr(obj, cprop['id'], defaults[cprop['id']])
            props=(cprop,)+props
        else:
            changeProperty(props, cprop)
            if defaults.has_key(cprop['id']):
                setattr(obj, cprop['id'], defaults[cprop['id']])
    nprops=()
    for prop in props:
        if prop['id'] not in remove_ids:
            nprops=nprops+(prop,)

    obj._properties=nprops

