# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyCore/VirtualPathHelpers.py,v 1.8 2003/06/15 16:21:30 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""VirtualPathHelpers Module"""
__version__='$Revision: 1.8 $'[11:-2]

from string import join, split
from types import TupleType, StringType

pathtype=TupleType
urltype=StringType

def stripPath(path):
    return map(lambda x: x.strip(),path)

def exclude(path, not_allowed=('',)):
    include_root=0
    if path[0]=='':
        include_root=1
        path=path[1:]
    path = tuple(filter(lambda x,not_allowed=not_allowed: x not in not_allowed, path))
    if include_root:
        path=('',) + path
    return path

def removeEndSlash(path):
    while path[-1]=='': path=path[:-1]
    return path

def normalizePath(path):
    return exclude(path)

def mergePaths(path1, path2, not_allowed=None):
    if path2==('',): return path1
    if not path2: return path1
    if path2[0]=='' and len(path2)>1: path2 = path2[1:]
    if not_allowed:
        return exclude(path1 + path2,not_allowed=not_allowed)
    return path1 + path2


def absolute2relative(base_path, path):
    base_path=url2path(base_path)
    path=url2path(path)
    if isAbsolute(path) and base_path==path[0:len(base_path)]:
        path=path[len(base_path):]
    return path

def change_base(old_base, new_base, path):
    old_base=url2path(old_base)
    new_base=url2path(new_base)
    path=url2path(path)
    if isAbsolute(path) and old_base==path[0:len(old_base)]:
        path=path[len(old_base):]
    return mergePaths(new_base, path)

def url2path(url):
    if type(url)==pathtype: return url
    path = tuple(split(url,'/'))
    if (len(path) > 1 ) and (path[-1] == ''): path = path[:-1]
    return path

def path2url(path, ending_slash=1):
    """ path2url returns an URL from a path. It does this by merging the path tuple entries with a slash.
        There are a couple of special cases. Firstly, the root URL should be just a slash.
        Secondly, objects should end in a slash, to make it easy to append methodnames to them without
        taking special care of the root case.
        To handle this second case the ending_slash parameter is used.
        If you set ending_slash to 0, '' or None it will not end in a slash (unless it is root).
        If you set ending_slash to something else, it will always end in a slash.
        This is all slightly experimental..."""
    if type(path)==urltype:
        if path[-1]!='/' and ending_slash:
            return path + '/'
        elif len(path)>1 and path[-1]=='/' and not ending_slash:
            return path[:-1]
        return path

    url = join(path,'/')
    if url=='':
        return "/"
    elif ending_slash:
        return url + '/'
    elif len(url)>1 and url[-1]=='/' and not ending_slash:
        return url[:-1]
    return url

def isAbsolute(path):
    if type(path)==urltype:
        path=url2path(path)
    return len(path)>0 and path[0]==''

def isUrl(path):
    return type(path)==pathtype

def isPath(path):
    return type(path)==urltype

def isPathOrUrl(value):
    return type(value)==pathtype or type(value)==urltype

def isAbove(path1, path2):
     if len(path1) >= len(path2): return 0
     return path1==path2[:len(path1)]

def isBellow(path1, path2):
     if len(path2) >= len(path1): return 0
     return path2==path1[:len(path2)]

def isChild(path1, path2):
     return path1[:-1] == path2

def isCurrent(path1, path2):
     return path1==path2

def isSibling(path1, path2):
     return path1[:-1] == path2[:-1]

# Virtual to physical convertions:
# Move (Virtual to physical convertions) implementations to
# VirtualPathHelpers for unit testing reasons.
def virtualPathToPhysicalPath(virtual_root_path, path):
    return mergePaths(virtual_root_path, path)

def virtualPathToPhysicalURL(virtual_root_path, path):
    return join(mergePaths(virtual_root_path, path), '/')

def virtualURLToPhysicalURL(virtual_root_path, url):
    path = url2path(url)
    return join(mergePaths(virtual_root_path, path), '/')

def virtualURLToPhysicalPath(virtual_root_path, url):
    path = url2path(url)
    return mergePaths(virtual_root_path, path)


#Comparision Functions

def comparePaths(path1, path2):
    if len(path1)!=len(path2): return 0
    for i in range(0, len(path1)):
        if path1[i]!=path2[i]: return 0
    return 1


def extractDomainName(host_string):
    #host_string=request['HTTP_HOST']
    return split(host_string, ':')[0]


def extractPort(host_string):
    host = split(host_string, ':')
    try: return int(host[1])
    except: pass
    return 80



