
import re
from types import StringType
import string
from StringIO import StringIO
import difflib

from xml.dom import Node
from xml.dom.html.HTMLDocument import HTMLDocument
from xml.dom.ext.reader import HtmlLib
from xml.dom.ext import Print

from TextPrinter import TextVisitor

import CompNode
from DOMUtils import *


BEGIN_HTML="<html><body>"
END_HTML="</body></html>"

def insertNode(newnode, insert_point, parent, owner, tagName='INS'):
    newnode=owner.importNode(newnode,1)
    ins=owner.createElement(tagName)
    ins.appendChild(newnode)
    parent.insertBefore(ins, insert_point)

def appendNode(newnode, parent, owner):
    newnode=owner.importNode(newnode,1)
    ins=owner.createElement('INS')
    ins.appendChild(newnode)
    parent.appendChild(ins)

def deleteNode(deleted_object, owner, tagName='DEL'):
    isChildsDeleted(deleted_object)
    newnode=owner.createElement(tagName)
    deleted_object.parentNode.replaceChild(newnode, deleted_object)
    newnode.appendChild(deleted_object)

class AdvancedHTMLDiffer:

    dom1=None
    dom2=None
    seq1=None
    seq2=None


    def __init__(self, html1=None, html2=None):
        self.html1=html1
        self.html2=html2
        self._preProcess()

    def reset(self):
        self._preProcess()

    def _processHTML(self, html):
        reader = HtmlLib.Reader()
        dom = reader.fromString(BEGIN_HTML+html+END_HTML)
        return dom

    def _preProcess(self):
        self.dom1=self._processHTML(self.html1)
        self.dom2=self._processHTML(self.html2)
        #self.dom1=splitTextNodes(self.dom1)
        #self.dom2=splitTextNodes(self.dom2)
        self.dom1.normalize()
        self.dom2.normalize()

        self.seq1 = CompNode.SequentiateSimple(self.dom1)
        self.seq2 = CompNode.SequentiateSimple(self.dom2)


    def processDiff(self):

        differ=difflib.SequenceMatcher(None, self.seq1, self.seq2)

        owner=self.dom1.ownerDocument

        for tag, alo, ahi, blo, bhi in differ.get_opcodes():
            if tag == 'replace':
                insert_point=self.seq1[alo].node
                parent=insert_point.parentNode
                current=None

                for newline in self.seq2[blo:bhi]:
                   if not current or newline.node.parentNode is not current:
                        insertNode(newline.node, insert_point, parent, owner, 'INS')
                        current=newline.node

                for delline in self.seq1[alo:ahi]:
                    deleted_object=delline.node
                    newnode=owner.createElement('DEL')
                    deleted_object.parentNode.replaceChild(newnode, deleted_object)
                    newnode.appendChild(deleted_object)

            elif tag == 'delete':
                for delline in self.seq1[alo:ahi]:
                    deleted_object=delline.node
                    deleteNode(deleted_object, owner)

            elif tag == 'insert':
                if alo<len(self.seq1):
                    insert_point=self.seq1[alo].node
                    parent=insert_point.parentNode
                    current=None

                    for newline in self.seq2[blo:bhi]:
                        if not current or newline.node.parentNode is not current:
                            insertNode(newline.node, insert_point, parent, owner)
                            current=newline.node
                else:
                    if self.seq1:
                        parent=self.seq1[0].node.parentNode
                    else:
                        #If seq1 is empty, append to BODY
                        parent=self.dom1.getElementsByTagName('BODY')[0]
                    current=None

                    for newline in self.seq2[blo:bhi]:
                        if not current or newline.node.parentNode is not current:
                            appendNode(newline.node, parent, owner)
                            #newnode=owner.importNode(newline.node,1)
                            #ins=owner.createElement('INS')
                            #ins.appendChild(newnode)
                            current=newline.node

            elif tag == 'equal':
                pass

            else:
                self.fail("Tag '%s' doesn't exist." % tag)


    def getBodyContent(self):
        return PrintBodyContent(self.dom1)

    def Print(self):
        return doPrint(self.dom1)

