
from xml.dom import Node

from xml.dom.ext.reader import HtmlLib, Sax2
from xml.dom.ext import Print

from xml.dom.ext.c14n import Canonicalize

from StringIO import StringIO

from Products.ParsedXML.DOM.Traversal import NodeFilter, TreeWalker


XHTMLNS = "http://www.w3.org/1999/xhtml"    # XHTML name space

# Convenience map for assert messages
TYPE_NAME = {
        Node.ATTRIBUTE_NODE:                'Attribute',
        Node.CDATA_SECTION_NODE:            'CDATA Section',
        Node.COMMENT_NODE:                  'Comment',
        Node.DOCUMENT_FRAGMENT_NODE:        'Document Fragment',
        Node.DOCUMENT_NODE:                 'Document',
        Node.DOCUMENT_TYPE_NODE:            'DocumentType',
        Node.ELEMENT_NODE:                  'Element',
        Node.ENTITY_NODE:                   'Entity',
        Node.ENTITY_REFERENCE_NODE:         'Entity Reference',
        Node.NOTATION_NODE:                 'Notation',
        Node.PROCESSING_INSTRUCTION_NODE:   'Processing Instruction',
        Node.TEXT_NODE:                     'Text',
   }

from PatchXML.Element import match

class normalizeFilter(NodeFilter):

    def acceptNode(self, node):
        return self.FILTER_ACCEPT
        if node.nodeName == 'C':
            return self.FILTER_SKIP
        else:
            return self.FILTER_ACCEPT

INLINE_TAGS=[
  'A'
, 'ABBR'        #Abbreviation
, 'ACRONYM'     #Acronym
, 'B'           #Bold text
, 'BASEFONT'    #Base font change
, 'BDO'         #BiDi override
, 'BIG'         #Large text
, 'BR'          #Line break
, 'CITE'        #Citation
, 'CODE'        #Computer code
, 'DFN'         #Defined term
, 'EM'          #Emphasis
, 'FONT'        #Font change
, 'I'           #Italic text
, 'IMG'         #Inline image
, 'INPUT'       #Form input
, 'KBD'         #Text to be input
, 'LABEL'       #Form field label
, 'Q'           #Short quotation
, 'S'           #Strike-through text
, 'SAMP'        #Sample output
, 'SELECT'      #Option selector
, 'SMALL'       #Small text
, 'SPAN'        #Generic inline container
, 'STRIKE'      #Strike-through text
, 'STRONG'      #Strong emphasis
, 'SUB'         #Subscript
, 'SUP'         #Superscript
, 'TEXTAREA'    #Multi-line text input
, 'TT'          #Teletype text
, 'U'           #Underlined text
, 'VAR'         #Variable
]

def isInline(*arg):
    for node in arg[:]:
        if node.nodeType==Node.ELEMENT_NODE and node.tagName.upper() not in INLINE_TAGS:
            return 0
    return 1

def isDivAlone(nodelist):
    #returns true there only exists a single DIV node
    #in the nodelist. Whitespace textnodes are ignored.
    found_div=0
    #print
    #print "="*40
    #print "nodelist", nodelist, len(nodelist)
    for node in nodelist:
        #print "node", node, node.nodeType, node.nodeType==Node.ELEMENT_NODE and node.tagName.lower() or '--', node.nodeType==Node.TEXT_NODE and node.nodeValue.strip() or '--'
        if node.nodeType==Node.ELEMENT_NODE and node.tagName.lower()=='div':
            if found_div:
                return 0
            else:
                found_div=1
        elif node.nodeType!=Node.TEXT_NODE or node.nodeValue.strip():
            return 0
    return 1

def nextElement(walker):
    skiped_nodes=[]
    nextNode = walker.nextNode()
    while nextNode and nextNode.nodeType==Node.TEXT_NODE:
        skiped_nodes.append(nextNode)
        nextNode = walker.nextNode()
    return nextNode, skiped_nodes

def EasyNormalizeXHTML(stream, output=None, xml=0):
    if not hasattr(stream, 'write'):
        stream=StringIO(stream)
    if not output or not hasattr(output, 'write'):
        output=StringIO()

    if xml:
        reader = Sax2.Reader()
    else:
        reader = HtmlLib.Reader()
    doc_node = reader.fromStream(stream)

    out=StringIO()
    Canonicalize(doc_node, out)
    doc_node = reader.fromString(out.getvalue())
    out.close()

    walker = TreeWalker(doc_node, NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_TEXT | NodeFilter.SHOW_ENTITY | NodeFilter.SHOW_ENTITY_REFERENCE, normalizeFilter(), 0)

    nextNode, skiped_nodes = nextElement(walker)
    i=1
    while nextNode:
        thisNode = nextNode
        nextNode, whitespaces = nextElement(walker)
        if nextNode and thisNode:
            if nextNode.nodeType==Node.ELEMENT_NODE and thisNode.nodeType==Node.ELEMENT_NODE:
                if match(thisNode, nextNode) and thisNode.parentNode==nextNode.parentNode and isInline(thisNode, nextNode):
                    #Merge matching adjacent nodes
                    walker.currentNode=thisNode
                    current=walker.nextSibling()
                    while current and current!=nextNode:
                        if current.nodeType==Node.TEXT_NODE:
                            if current.nodeValue.strip():
                                break;
                            else:
                                thisNode.appendChild(current.cloneNode(1))
                                thisNode.parentNode.removeChild(current)
                        current=walker.nextSibling()
                    for child in nextNode.childNodes:
                        thisNode.appendChild(child.cloneNode(1))
                    thisNode.parentNode.removeChild(nextNode)
                    walker.currentNode=thisNode
                elif thisNode==nextNode.parentNode and nextNode.tagName.lower()=='div' and isDivAlone(thisNode.childNodes):
                    for n in thisNode.childNodes[:]:
                        thisNode.removeChild(n)
                    thisNode.setAttributeNS(None, 'class', nextNode.getAttributeNS(None, 'class'))
                    for child in nextNode.childNodes:
                        thisNode.appendChild(child.cloneNode(1))
                    nextNode=thisNode
                    walker.currentNode=thisNode

    if output and hasattr(output,'write'):
        Print(doc_node, output)
    return doc_node

def EasyNormalizeXML(stream, output=None):
    return EasyNormalizeXHTML(stream, output, xml=1)
