
import string, re
from xml.dom import Node
from xml.dom.ext.Visitor import Visitor, WalkerInterface

class TextVisitor(Visitor):
    def __init__(self, stream, strip=1, encoding='ISO-8859-1', indent='', plainElements=None,
                 nsHints=None, isXhtml=0, force8bit=0):
        self.stream = stream
        self.encoding = encoding
        # Namespaces
        self._namespaces = [{}]
        self._nsHints = nsHints or {}
        # PrettyPrint
        self._indent = indent
        self._depth = 0
        self._inText = 0
        self._plainElements = plainElements or []
        # HTML support
        self._html = None
        self._isXhtml = isXhtml
        self.force8bit = force8bit
        self.strip=strip
        return

    def _write(self, text):
        self.stream.write(text)
        return

    def visit(self, node):
        if self._html is None:
            # Set HTMLDocument flag here for speed
            self._html = hasattr(node.ownerDocument, 'getElementsByName')

        nodeType = node.nodeType
        if node.nodeType == Node.ELEMENT_NODE:
            return self.visitElement(node)

        elif node.nodeType == Node.ATTRIBUTE_NODE:
            return self.visitAttr(node)

        elif node.nodeType == Node.TEXT_NODE:
            return self.visitText(node)

        elif node.nodeType == Node.CDATA_SECTION_NODE:
            return self.visitCDATASection(node)

        elif node.nodeType == Node.ENTITY_REFERENCE_NODE:
            return self.visitEntityReference(node)

        elif node.nodeType == Node.ENTITY_NODE:
            return self.visitEntity(node)

        elif node.nodeType == Node.PROCESSING_INSTRUCTION_NODE:
            return self.visitProcessingInstruction(node)

        elif node.nodeType == Node.COMMENT_NODE:
            return self.visitComment(node)

        elif node.nodeType == Node.DOCUMENT_NODE:
            return self.visitDocument(node)

        elif node.nodeType == Node.DOCUMENT_TYPE_NODE:
            return self.visitDocumentType(node)

        elif node.nodeType == Node.DOCUMENT_FRAGMENT_NODE:
            return self.visitDocumentFragment(node)

        elif node.nodeType == Node.NOTATION_NODE:
            return self.visitNotation(node)

        # It has a node type, but we don't know how to handle it
        raise Exception("Unknown node type: %s" % repr(node))

    def visitNodeList(self, node, exclude=None):
        for curr in node:
            curr is not exclude and self.visit(curr)
        return

    def visitNamedNodeMap(self, node):
        for item in node.values():
            self.visit(item)
        return

    def visitAttr(self, node):
        return

    def visitProlog(self):
        self._inText = 0
        return

    def visitDocument(self, node):
        not self._html and self.visitProlog()
        node.doctype and self.visitDocumentType(node.doctype)
        self.visitNodeList(node.childNodes, exclude=node.doctype)
        return

    def visitDocumentFragment(self, node):
        self.visitNodeList(node.childNodes)
        return

    def visitElement(self, node):
        #for attr in node.attributes.values():
        #    self.visitAttr(attr)
        if len(node.childNodes):
            self._depth = self._depth + 1
            self.visitNodeList(node.childNodes)
            self._depth = self._depth - 1
        self._inText = 0
        return

    def visitText(self, node):
        text = node.data
        if self.strip:
            if not string.strip(text) and text:
                text=' '
        else:
            text = string.strip(text) and text
        if text:
            self.stream.write(text)
            self._inText = 1
        return

    def visitDocumentType(self, doctype):
        self._inText = 0
        return

    def visitEntity(self, node):
        return

    def visitNotation(self, node):
        """Visited from a NamedNodeMap in DocumentType"""
        return

    def visitCDATASection(self, node):
        #self._tryIndent()
        #self._write('<![CDATA[%s]]>' % (node.data))
        #self._inText = 0
        return

    def visitComment(self, node):
        #self._tryIndent()
        #self._write('<!--%s-->' % (node.data))
        #self._inText = 0
        return

    def visitEntityReference(self, node):
        self._write('&%s;' % node.nodeName)
        self._inText = 1
        return

    def visitProcessingInstruction(self, node):
        #self._tryIndent()
        #self._write('<?%s %s?>' % (node.target, node.data))
        self._inText = 0
        return

