import unittest

from xml.dom import Node
from xml.dom import implementation
from xml.dom.NodeFilter import NodeFilter

from xml.dom.ext.reader import HtmlLib, Sax2
from xml.dom.ext import StripXml, Print, PrettyPrint

from logilab.xmldiff.input import tree_from_stream, tree_from_dom
from logilab.xmldiff.objects import N_ISSUE, N_CHILDS
from logilab.xmldiff.format import XUpdatePrinter, InternalPrinter
from logilab.xmldiff.ezs import EzsCorrector
from logilab.xmldiff.fmes import FmesCorrector

#input import
from logilab.xmldiff.parser import SaxHandler
from xml.sax import make_parser, SAXNotRecognizedException
from xml.sax.handler import feature_namespaces
from xml.sax.handler import feature_external_ges, feature_external_pes
from xml.sax.handler import property_lexical_handler
from xml.dom.ext.c14n import Canonicalize


import CompNode
import DOMUtils
import difflib
import HTMLDiffer


from logilab.xmldiff.format import InternalPrinter, XUpdatePrinter, DOMXUpdateFormatter

from StringIO import StringIO

B_HTML="<HTML><BODY>"
E_HTML="</BODY></HTML>"

OFF=1
ON=0


def generic_test(name, HTML1, HTML2):

    reader = HtmlLib.Reader()
    dom1 = reader.fromString("<html><body>"+HTML1+"</body></html>")
    dom2 = reader.fromString("<html><body>"+HTML2+"</body></html>")

    dom1=DOMUtils.splitTextNodes(dom1)
    dom2=DOMUtils.splitTextNodes(dom2)

    seq1, seq2 = DOMUtils.Sequentiate(dom1), DOMUtils.Sequentiate(dom2)

    if name:
        print "="*40
        print name
        print
        print "HTML1\n", HTML1
        print
        print "HTML2\n", HTML2
        print

    DOMUtils.Process(seq1, seq2)
    DOMUtils.DiffFormat(dom1)
    return dom1

def simple_test(name, HTML1, HTML2):

    reader = HtmlLib.Reader()
    dom1 = reader.fromString("<html><body>"+HTML1+"</body></html>")
    dom2 = reader.fromString("<html><body>"+HTML2+"</body></html>")

    dom1=DOMUtils.splitTextNodes(dom1)
    dom2=DOMUtils.splitTextNodes(dom2)

    seq1, seq2 = CompNode.SequentiateSimple(dom1), CompNode.SequentiateSimple(dom2)

    if name:
        print "="*40
        print name
        print
        print "HTML1\n", HTML1
        print
        print "HTML2\n", HTML2
        print

    DOMUtils.ProcessSimple(seq1, seq2)
    #DOMUtils.DiffFormatSimple(dom1)
    PrettyPrint(dom1)
    return dom1


class TestXMLDIFF(unittest.TestCase):

    def testsplitTextNodes1(self):  ## test method names begin 'test*'

        if ON: return

        NAME="testsplitTextNodes1"

        HTML1="""
        <P>Paragraph1</P>
        <P>Paragraph2</P>
        <P>Paragraph3 </P>
        <P>Paragraph5 test this</P>
        <P>Paragraph6</P>
        """

        HTML2="""
        <P>Paragraph1</P>
        <P>Paragraph2</P>
        <P>Paragraph3 <STRONG>add</STRONG></P>
        <P>Paragraph4 test now this</P>
        <P>Paragraph6</P>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


    def testsplitTextNodes2(self):  ## test method names begin 'test*'

        if ON: return

        NAME="testsplitTextNodes2"

        HTML1="""
        <P class="p">Paragraph1</P>
        <P>Paragraph2</P>
        <P>Paragraph3 </P>
        <P>Paragraph5 test this</P>
        <P>Paragraph6</P>
        """

        HTML2="""
        <P class="normal">Paragraph1</P>
        <H1>Paragraph2</H1>
        <P>Paragraph3 <STRONG>add</STRONG></P>
        <P>Paragraph4 test now this</P>
        <P>Paragraph6</P>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


    def testsplitTextNodes3(self):  ## test method names begin 'test*'

        if ON: return

        NAME="testsplitTextNodes3"

        HTML1="""
        <div id="newdiv" class="divstyle">
        <P class="p">Paragraph1</P>
        <P>Paragraph2</P>
        <P>Paragraph3 </P>
        <P>Paragraph5 test this</P>
        <P>Paragraph6</P>
        </div>
        """

        HTML2="""
        <P class="p">Paragraph1</P>
        <H1>Paragraph2</H1>
        <P>Paragraph3 <STRONG>add</STRONG></P>
        <P>Paragraph4 test now this</P>
        <P>Paragraph6</P>"""

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


    def testsplitTextNodes4(self):  ## test method names begin 'test*'

        if ON: return

        NAME="Insert in front: no 1"

        HTML1="""
        <p>Insert in front</p>
        <p>test row</p>
        <p>Last line</p>
        """

        HTML2="""
        <p>Insert in front</p>
        <p>New text test row</p>
        <p>Last line</p>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)



    def testsplitTextNodes5(self):  ## test method names begin 'test*'

        if ON: return

        NAME="Insert in front: no 2"

        HTML1="""
        <p>Insert in front</p>test row
        <p>Last line</p>
        <p>Insert Test2</p>
        <p>Line 3</p>
        <p>Insert Test3</p>
        <p>Line 4</p>
        <p>Insert Test4</p>
        """

        HTML2="""
        <p>Insert in front</p><p>New text</p><P>test row</P>
        <p><span>Remove <em>us</em now.</span></p>
        <p>Last line</p>
        <p><b>Insert</b> Test2</p>
        <p>Line 3 new3</p>
        <p>Insert Test3</p>
        <p>Line 4</p> new4
        <p>Insert Test4</p>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)



    def testBreakHomogenSequence1(self):

        if ON: return

        NAME="testBreakHomogenSequence1"

        HTML1="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>Line number 3</p>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <p>Line number 7</p>
        <p>Line number 8</p>
        """

        HTML2="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>This is new 1</p>
        <p>Line number 4</p>
        <p>This is new 2</p>
        <p>Line number 8</p>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


    def testBreakHomogenSequence1(self):

        if ON: return

        NAME="testBreakHomogenSequence1"

        HTML1="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>Line number 3</p>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <p>Line number 7</p>
        <p>Line number 8</p>
        """

        HTML2="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>This is new 1</p>
        <p>Line number 3</p>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <p>Line number 8</p>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


    def testListTest1(self):

        if ON: return

        NAME="testListTest1"

        HTML1="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>Line number 3</p>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <p>Line number 7</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 2</LI>
        </UL>
        <p>Line number 8</p>
        <p>Line number 9</p>
        <UL>
        <LI>List 9 row 1</LI>
        <LI>List 9 row 2</LI>
        </UL>
        <p>Line number 10</p>
        """

        HTML2="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>This is new 1</p>
        <p>Line number 3</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 2</LI>
        </UL>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 3</LI>
        <LI>List row 4</LI>
        </UL>
        <p>Line number 8</p>
        <p>Line number 9</p>
        <p>Line number 10</p>
        """

        #dom=generic_test(NAME, HTML1, HTML2)
        dom=simple_test(NAME, HTML1, HTML2)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestXMLDIFF, 'test'))
    return suite
