# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/BrowserFolder/BrowserFolder.py,v 1.6 2003/05/08 17:55:23 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""BrowserFolder

$Colliberty: BrowserFolder.py,v 1.5.5 2001/11/07 17:53:00 johanc Exp $"""
__version__='$Revision: 1.6 $'[11:-2]


import Globals
from Globals import DTMLFile
from OFS.Folder import Folder
from urllib import quote
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
import re
from Globals import PersistentMapping
from Acquisition import aq_base, aq_chain, aq_inner, aq_self

class BrowserFolder(Folder):
    """A BrowserFolder"""

    security = ClassSecurityInfo()

    meta_type = "BrowserFolder"

    isPrincipiaFolderish = 1
    isSearchable = 0

    manage_options =  Folder.manage_options[0:1] + ( {'label':'Matches',   'action':'manage_regex'}, ) + Folder.manage_options[1:]

    def __init__(self,id, title='', regex=None):
        self.id = id
        self.title = title
        self._compile(regex)


    def _compile(self, regex):
        recomp = []
        self.regex = regex
        for reg in regex:
            if len(reg)>0:
                if reg[0]=='-':
                    recomp.append(('-', re.compile(reg[1:])))
                elif reg[0]=='!':
                    recomp.append(('!', re.compile(reg[1:])))
                elif reg[0]=='#':
                    recomp.append(('#', re.compile(reg[1:])))
                elif reg[0]=='+':
                    recomp.append(('+', re.compile(reg[1:])))
                else:
                    recomp.append(('+', re.compile(reg)))
        self._recomp = recomp


    security.declarePublic('match')
    def match(self, http_user_agent):
        result = 0
        for command, recomp in self._recomp:
            if command=='+' and recomp.match(http_user_agent) is not None:
                return 1
            elif command=='-' and recomp.match(http_user_agent) is not None:
                return 0
            elif command=='!' and recomp.match(http_user_agent) is None:
                return 1
            elif command=='#' and recomp.match(http_user_agent) is None:
                return 0
        return 0

    security.declarePrivate('_manage_main')
    _manage_regex = DTMLFile('dtml/manage_main_BrowserFolder', globals())

    manage_test_hua = DTMLFile('dtml/manage_test_hua', globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_regex')
    def manage_regex(self, title=None, regex=None, REQUEST=None):
        """Manage Regulare Expressions"""
        if title is not None:
            self.title = title
        if regex is not None:
            self._compile(regex)
            try:
                self.aq_parent.clearBrowserFolderCache()
            except:
                pass
        if REQUEST is not None:
            return self._manage_regex(self,REQUEST ,management_view='Matches')

Globals.InitializeClass(BrowserFolder)


manage_add_BrowserFolderForm = DTMLFile('dtml/manage_add_BrowserFolderForm', globals())


def manage_add_BrowserFolder(self,id, submit, title='', regex=None, REQUEST=None):
    """Add a BrowserFolder"""
    ob = BrowserFolder(id, title=title, regex=regex)
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')


class BrowserAware:
    """BrowserAware Base class, mainly for use in Template Repositories"""


    security = ClassSecurityInfo()
    _http_user_agent_cache = PersistentMapping()

    # MANAGE BROWSER CUSTOMIZERS

    def hasBrowserFolder(self,browserid):
        return hasattr(self, browserid)

    def getBrowserFolderById(self, browserid):
        return getattr(self, browserid, None)

    def browserIds(self,spec=None):
        return self.objectIds(spec=["BrowserFolder"])

    def browserValues(self,spec=None):
        return self.objectValues(spec=["BrowserFolder"])

    def browserItems(self,spec=None):
        return self.objectItems(spec=["BrowserFolder"])

    def browserMap(self):
        return self.objectMap(spec=["BrowserFolder"])

    def getBrowserFolderByHUA(self, hua):
        if self._http_user_agent_cache.has_key(hua):
            return aq_base(self.getBrowserFolderById(self._http_user_agent_cache[hua]))
        for bfolder in self.browserValues():
            if bfolder.match(hua):
                self._http_user_agent_cache[hua]= bfolder.id
                return aq_base(bfolder)
        return None


    def clearBrowserFolderCache(self):
        self._http_user_agent_cache = PersistentMapping()

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_clear_browser_folder_cache')
    def manage_clear_browser_folder_cache(self, REQUEST=None):
        """Clear the Browser Cache."""
        self.clearBrowserFolderCache()
        if REQUEST:
            return "<h1>BrowserFolder Cache Cleared</h1>"


Globals.InitializeClass(BrowserAware)


def initialize(context):
    """Initialize BrowserFolder Product"""

    context.registerEasyTemplateClass(
            BrowserFolder,
            meta_type = BrowserFolder.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_BrowserFolderForm, manage_add_BrowserFolder),
            icon='icons/BrowserFolder.gif',
        )


