# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/EasyNewsItem.py,v 1.15 2004/01/27 10:13:42 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#



__doc__="""EasyNewsItem Object"""
__version__='$Revision: 1.15 $'[11:-2]



import sys
import Globals
from Globals import DTMLFile
import DateTime
from AccessControl import getSecurityManager
from string import join
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from copy import deepcopy

from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.Utils import Tutils
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID,DEFAULTVIEWNAME, DEFAULTVIEWATTR, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyEditor.EasyEditable import EasyEditable
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyNavigation.EasyNavigatable import Navigatable
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyNewsService.EasyNewsAware import EasyNewsAware
from Products.EasyCore.EasyNavigationConf import NAVIGATION_CATALOG_ID
from Products.EasyCore.EasyNewsServiceConf import NEWS_SERVICE_ID

from AccessControl import ClassSecurityInfo

class EasyNewsItem(ManageOptionBase
                    , Navigatable, EasyNewsAware
                    , EasyDublinCore, EasyEditable, Publishable):
    """A EasyNewsItem Object """

    meta_type = 'EasyNewsItem'
    __catalogs__=(NAVIGATION_CATALOG_ID,NEWS_SERVICE_ID)


    security = ClassSecurityInfo()



    use_manage_options = EasyEditable.use_manage_options + \
                         Publishable.use_manage_options + \
                         Navigatable.use_manage_options

    #manage_metadata editMetadata

    _auto_add_to_navigation = 1
    _workflowable=1
    _isEasyRoot = 0

    skey_default='id'

    _properties = Navigatable._properties + EasyDublinCore._properties+({'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'wd','select_variable':'possible_obj_sort_columns'},)


    _view_defaults = {
             DEFAULTVIEWATTR: ('default', 'Default', 0),
             '_printview': ('printview', 'Print View', 1)
             }



    def __init__(self
                , id
                , template=None
                , title=''
                , navtitle=''
                , description=''
                , language=''
                , category=()
                , subject=()
                , effective_date=NullDate
                , expiration_date=NullDate
                , format='text/html'
                , add_to_navigation=1
                ):
        EasyDublinCore.__init__(self
                                , title=title
                                , description=description
                                , language=language
                                , category=category
                                , subject=subject
                                , effective_date=effective_date
                                , expiration_date=expiration_date
                                , format=format
                                )
        EasyEditable.__init__(self,id, title, template)
        Navigatable.__init__(self, title, navtitle)
        Publishable.__init__(self)
        self._auto_add_to_navigation = add_to_navigation
        self.setEffectiveDate(effective_date)
        self.navtitle = navtitle


    def addNewDocument(self , id
                            , template = None
                            , title=''
                            , navtitle=''
                            , description=''
                            , language=''
                            , category=()
                            , subject=()
                            , effective_date=NullDate
                            , expiration_date=NullDate
                            , format=''
                            , add_to_navigation=None
                            , REQUEST=None
                            , RESPONSE=None
                        ):
        if add_to_navigation is None:
            add_to_navigation=self._auto_add_to_navigation
        if template is None:
            template=self._template
        try:
            ob=EasyNewsItem( id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                            , subject=subject
                            , effective_date=effective_date
                            , expiration_date=expiration_date
                            , format=format
                            , add_to_navigation = add_to_navigation
                            )
            ob._isEasyRoot = 0
            self._setObject(id, ob)
            ob = self._getOb(id)
            ob.initcontent()
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyNewsItem', ERROR, "Can't create sub document. %s (%s)" % exc[:2], error=exc)

    #EVENT TRIGGERS
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        Navigatable.manage_afterAdd(self, item, container)
        EasyEditable.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        Navigatable.manage_beforeDelete(self, item, container)
        EasyEditable.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Navigatable.manage_afterClone(self, item)
        EasyEditable.manage_afterClone(self, item)


    def cloneSubobjects( self, clone, inv_spec=('EasySlaveCopy','EasyMasterCopy')):
        filtered_ids=[]
        for id, obj in self.objectItems():
            if not getattr(aq_base(obj), 'isAnObjectManager', 0) and obj.meta_type not in inv_spec:
                filtered_ids.append(id)
        cp=self.manage_copyObjects(filtered_ids)
        clone.manage_pasteObjects(cp)

    security.declarePrivate('cloneTo')
    def cloneTo( self, container ):
        """\
        Make a copy of this objekt, including any sub objekt
        not being an ObjectManager.
        """
        excluded_ids=['notification_list','creation_date','EasySlaveCopy','EasyMasterCopy',
                      'publish_history', '_workflow_history', 'created_by', 'approved_by',
                      'effective_date', 'effective_date', 'effective_date_set_by',
                      'expiration_date_set_by', '_template'
                      '_master_data_copy', '_objects', 'last_modified_by']
        self_id = self.getId()
        excluded_ids.extend(self.objectIds())
        clone=EasyNewsItem( id = self_id
                            , template = self._template
                            , title=self.title
                            , navtitle=self.navigation_title
                            , description=self.description
                            , language=self.language
                            , category=self.category
                            , subject=self.subject
                            , effective_date=self.effective_date
                            , expiration_date=self.expiration_date
                            , format=self.format
                            , add_to_navigation=self._auto_add_to_navigation)
        for id, obj in self.__dict__.items():
            if id not in excluded_ids:
                if hasattr(aq_base(obj), 'inheritedAttribute'):
                    clone.__dict__[id] = aq_base(obj)
                else:
                    clone.__dict__[id] = deepcopy(aq_base(obj))
        container._setObject(self_id, clone)
        clone = container._getOb(self_id)
        self.cloneSubobjects(clone)


    #Skinnable
    #NEW#__of__=EasyEditable.__of__
    #NEW#__getattr__=EasyEditable.__getattr__
    #NEW#_checkId=EasyEditable._checkId
    #NEW#valid_property_id=EasyEditable.valid_property_id


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8' ; changed=1
        if not getattr(aq_base(self),'_object_links', None): self._object_links = {} ; changed=1
        upgradeProperties(self, {'skey_default':'position',}, ())
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyNewsItem)


addEasyNewsItemForm=DTMLFile('dtml/addEasyNewsItemForm', globals())


def addEasyNewsItem(self, id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation = 1
                    , REQUEST=None):
    """Add EasyNewsItem"""

    id=Tutils.xlate2url(id)

    # BUG
    # Something isn't right here, for instance if a EasyEditor
    # has a none default viewname the created EasyNewsItem
    # would try to render DEFAULTVIEWNAME instead of EasyNewsItem.default_view
    if template=='**default':
        template = DEFAULTTEMPLATEID

        ob=EasyNewsItem(  id
                        , template=template
                        , title=title
                        , navtitle=navtitle
                        , description=description
                        , language=language
                        , category=category
                        , subject=subject
                        , effective_date=effective_date
                        , expiration_date=expiration_date
                        , format=format
                        , add_to_navigation = add_to_navigation
                        )

        ob._isEasyRoot = 1
        self._setObject(id, ob)
        ob = self._getOb(id) # get object in context
        ob.initViews()
    else:
        if template=='__ACQUIRE' or template is None:
            template = ACQUIRETEMPLATE
        ob=EasyNewsItem(id
                        , template=template
                        , title=title
                        , navtitle=navtitle
                        , description=description
                        , language=language
                        , category=category
                        , subject=subject
                        , effective_date=effective_date
                        , expiration_date=expiration_date
                        , format=format
                        , add_to_navigation = add_to_navigation
                        )
        self._setObject(id, ob)
        ob = self._getOb(id) # get object in context
        ob.initcontent()

    if REQUEST is not None:
        return self.manage_main(self,REQUEST)


def initialize(context):
    """Initialize EasyNewsItem Product"""

    context.registerClass (
            EasyNewsItem,
            permission=EasyNewsItemAdd,
            icon='icons/EasyNewsItem.gif',
            constructors=(addEasyNewsItemForm,
                          addEasyNewsItem),
             meta_type = EasyNewsItem.meta_type
            )

    context.registerEasyType(EasyNewsItem)
