# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/EasyPropertyContainer.py,v 1.19 2004/01/23 15:23:02 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyPropertyContainer Object

$Colliberty: EasyPropertyContainer.py,v 1.8 2001/08/21 11:58:00 johanc Exp $"""
__version__='$Revision: 1.19 $'[11:-2]


import sys
import Globals
from Globals import DTMLFile
import DateTime
from Acquisition import aq_base
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions

from Products.EasyCore.ManageOptionBase import ManageOptionBase

from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager

from string import join

from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyEditor.EasyWebDAVSupport import EasyWebDAVSupport
from Products.EasyWorkflow.EasySync import EasySyncAware
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager
from Products.EasyWorkflow.EasyPublishable import Publishable

from zLOG import LOG, ERROR

_missing=[]

class EasyPropertyContainer(ManageOptionBase, EasySyncAware,
    Folder, EasyWebDAVSupport, EasyCatalogAwareObjectAndPropertyManager, Publishable):
    """A EasyPropertyContainer Object"""

    security = ClassSecurityInfo()

    meta_type = 'EasyPropertyContainer'

    easysyncaware_options = ({'label':'SyncProperties', 'action':'manage_sync_properties',},)

    _properties=({'id':'language', 'type': 'selection', 'mode':'w', 'select_variable':'possible_languages'},)

    security.declareProtected(EasyCorePermissions.ViewManagementScreens, 'use_manage_options')
    def use_manage_options(self):
        options = ()
        options = options + ((EasyCatalogAwareObjectAndPropertyManager.manage_options[:1], None, EasyCatalogAwareObjectAndPropertyManager.manage_options[1:]),)
        options = options + ((None , None, Folder.manage_options[3:]),)
        if self.isSyncronized:
            options = options + ((None,self.easysyncaware_options,None),)
        options = options + Publishable.use_manage_options
        return options

    def PrincipiaSearchSource(self):
        #Very temporary search source for text indexes
        #This method should be made to look for attributes,
        #properties and subobjects defined for the object that
        #should be included in the PrincipiaSearchSource.
        result = ''
        return result

    def __init__(self, id, language='', init_sheet=None, **kw):
        EasySyncAware.__init__(self)
        Publishable.__init__(self)
        self.id = id
        self.language = language
        if init_sheet is None: init_sheet={}
        init_sheet.update(kw)
        self._initProperties(init_sheet)


    def _initProperties(self, dict, mode='w'):
        if dict:
            for name, value in dict.items():
                ntype=guess_property_type(value, name=name)
                if name.count('.'): name=name.split('.')[0]
                if type_converters.has_key(ntype):
                    value=type_converters[ntype](value)
                    self._initProperty(name, value, ntype, mode)

    def _initProperty(self, id, value, type='string', mode='w'):
        self._wrapperCheck(value)
        if not id or id[:1]=='_' or (id[:3]=='aq_') or (' ' in id):
            raise 'Bad Request', 'Invalid or duplicate property id'
        if type in ('selection', 'multiple selection'):
            self._properties=self._properties+({'id':id,'type':type, 'mode': mode, 'select_variable':'possible_'+id},)
        else:
            self._properties=self._properties+({'id':id,'type':type, 'mode': mode},)
        self._setPropValue(id, value)


    security.declareProtected(EasyCorePermissions.EasySimpleObjectEdit, 'initProperties')
    def initProperties(self, dict=None, mode='w', reset=0, icon_path=_missing, **kw):
        """\
        Initialize properties.
        Not that this checks for valid ids but ignores
        existing attributes. Which means they will be written over.
        Use this to set up object that are created ttw (e.g. Python Script)
        and can't call the constructor it self.
        For adding, deleting and changing properties use:
            addProperty, delProperty, and changeProperty
        """
        if icon_path is not _missing:
            self.icon_path = icon_path
        if type(dict)!=DictType: dict={}
        dict.update(kw)
        if reset: self._properties = ()
        self._initProperties(dict, mode=mode)


    # webDAV support

    def addNewDocument(self, id
                        , template = None
                        , title=''
                        , navtitle=''
                        , description=''
                        , language=''
                        , category=()
                        , subject=()
                        , effective_date=NullDate
                        , expiration_date=NullDate
                        , format='text/html'
                        , add_to_navigation=None
                        , REQUEST=None,RESPONSE=None
                    ):
        try:
            container=EasyPropertyContainer(id
                            , language=language
                        )
            self._setObject(id, container)
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyPropertyContainer', ERROR, "Can't create sub document. %s (%s)" % exc[:2], error=exc)

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if not hasattr(aq_base(self),'_object_links'): self._object_links = {} ; changed=1

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

    #EVENT TRIGGERS
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        EasySyncAware.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        EasySyncAware.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        EasySyncAware.manage_afterClone(self, item)



Globals.InitializeClass(EasyPropertyContainer)


manage_addEasyPropertyContainerForm=DTMLFile('dtml/manage_addEasyPropertyContainerForm', globals())

def manage_addEasyPropertyContainer(self, id
                        , template = None
                        , title=''
                        , navtitle=''
                        , description=''
                        , language=''
                        , category=()
                        , subject=()
                        , effective_date=NullDate
                        , expiration_date=NullDate
                        , format='text/html'
                        , add_to_navigation=None
                        , init_sheet=None
                        , REQUEST=None):
    """Add EasyPropertyContainer"""

    ob=EasyPropertyContainer(id, language=language, init_sheet=init_sheet)
    self._setObject(id, ob)
    ob = self._getOb(id) # get object in context

    if REQUEST is not None:
        return self.manage_main(self,REQUEST)


def initialize(context):
    """Initialize EasyPropertyContainer Product"""

    context.registerClass (
            EasyPropertyContainer,
            permission=EasyCorePermissions.EasyDocumentAdd,
            icon='icons/EasyPropertyContainer.gif',
            constructors=(manage_addEasyPropertyContainerForm,
                          manage_addEasyPropertyContainer),
             meta_type = EasyPropertyContainer.meta_type
            )


