# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/Features/Features.py,v 1.3 2003/04/14 12:35:33 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""FeatureBase Class

$Colliberty: Features.py,v 0.0.1 2001/05/14 22:06:00 johanc Exp $"""
__version__='$Revision: 1.3 $'[11:-2]


import Globals, Acquisition, Products, OFS, os
from App.ProductContext import ProductContext
from ExtensionClass import Base
from string import join,split
from App.ImageFile import ImageFile
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from OFS.SimpleItem import SimpleItem



FEATUREADDPERMISSION = 'Add EasyEditor Feature'



class FeatureBase(SimpleItem):
    """Base Class for Feature Objects"""

    elementid = ""
    command = ""
    handler = ""
    updateAppearenceHandler = ""
    usesLatched = ""
    feature_type = ""

    manage_options_left = (
            {'label':'Edit',   'action':'manage_main'},
    )

    manage_options_right = (
            {'label':'<< Features',   'action':'../Features/manage_workspace'},
    )

    def manage_options(self):
        return self.manage_options_left + self.manage_options_right

    def __init__(self,id,title=""):
        self.id = id
        self.title = title

    def getDialog(self,dialogid):
        pass


Globals.InitializeClass(FeatureBase)




# key: kind name; value: lists of meta-type dicts for classes of kind
EasyFeatures = {}
EasyFeatureButtonGroups = {}
EasyFeatureDropDownGroups = {}

class ButtonGroup:

    def __init__(self,id,order=None):
        self.id = id
        self.order = order
        self.buttons = {}

    def getButtonsOrdered(self):
        l = self.buttons.values()
        l.sort(lambda x, y: cmp(x.button_group_order,y.button_group_order))
        return l

class DropDownGroup:

    def __init__(self,id,order=None):
        self.id = id
        self.order = order
        self.dropdowns = {}

    def getDropDownsOrdered(self):
        l = self.dropdowns.values()
        l.sort(lambda x, y: cmp(x.dropdown_group_order,y.dropdown_group_order))
        return l


class _ProductContextPatch:

    def registerEasyFeature(self, instance_class=None, meta_type='',icon=None):
        """
        Register a plug-in class.  The class registered will not be included in the
        master Products.meta_types registry, but only in the EasyFeatures registry.
        To register a plug-in class, create an initialize() method in your product's
        __init__.py.  In it, call context.registerEasyFeature() with the parameters
        you would use for a context.registerClass() call.
        """

        # create and add a singleton
        feature = instance_class(meta_type)
        EasyFeatures[meta_type] = feature
        if icon:
            setupIcon(feature, icon, 'EasyEditor')

        button_group = getattr(instance_class,'button_group',None)
        if button_group is not None:
            if not EasyFeatureButtonGroups.has_key(button_group):
                bg = EasyFeatureButtonGroups[button_group] = ButtonGroup(button_group)
            else: bg = EasyFeatureButtonGroups[button_group]
            button_group_order = getattr(instance_class,'button_group_order',None)
            if not bg.buttons.has_key(meta_type):
                bg.buttons[meta_type] = feature

        dropdown_group = getattr(instance_class,'dropdown_group',None)
        if dropdown_group is not None:
            if not EasyFeatureDropDownGroups.has_key(dropdown_group):
                ddg = EasyFeatureDropDownGroups[dropdown_group] = DropDownGroup(dropdown_group)
            else: ddg = EasyFeatureDropDownGroups[dropdown_group]
            dropdown_group_order = getattr(instance_class,'dropdown_group_order',None)
            if not ddg.dropdowns.has_key(meta_type):
                ddg.dropdowns[meta_type] = feature


    def registerButtonGroup(self,id,order):
        """Registers a new Button Group and specifies its display order"""
        if not EasyFeatureButtonGroups.has_key(id):
            EasyFeatureButtonGroups[id] = ButtonGroup(id,order)

    def registerDropDownGroup(self,id,order):
        """Registers a new Button Group and specifies its display order"""
        if not EasyFeatureDropDownGroups.has_key(id):
            EasyFeatureDropDownGroups[id] = DropDownGroup(id,order)


def getEasyFeatureButtonGroupsOrdered():
    l = EasyFeatureButtonGroups.values()
    l.sort(lambda x, y: cmp(x.order,y.order))
    return l

def getEasyFeatureDropDownGroupsOrdered():
    l = EasyFeatureDropDownGroups.values()
    l.sort(lambda x, y: cmp(x.order,y.order))
    return l


#This method was borrowed from Martijn Faassen's (faassen@vet.uu.nl) Formulator
def setupIcon(klass, icon, repository):
    """Load icon into Zope image object and put it in Zope's
    repository for use by the ZMI, for a particular class.
    klass -- the class of the field we're adding
    icon  -- the icon
    """

    if not hasattr(OFS.misc_.misc_, repository):
        setattr(OFS.misc_.misc_,
                repository,
                OFS.misc_.Misc_(repository, {}))

    # get name of icon in the misc_ directory
    icon_name = os.path.split(icon)[1]

    icon_image = ImageFile(icon, globals())
    icon_image.__roles__ = None

    getattr(OFS.misc_.misc_, repository)[icon_name] = icon_image


    setattr(klass, 'icon', 'misc_/%s/%s' % (repository, icon_name))


# Patch ourselves in so we look like part of the Zope registration API
ProductContext.__dict__.update(_ProductContextPatch.__dict__)
