# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/Storage/MultiSelectionPropertyStorage.py,v 1.1 2004/01/14 14:38:55 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import HTML
from SelectionPropertyStorage import SelectionPropertyStorage
from ListTypeStorages import ListTypeStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'



class MultiSelectionPropertyStorage(ListTypeStorage, SelectionPropertyStorage):
    """A MultiSelectionPropertyStorage Object """

    property_type = 'multiple selection'
    storage_type="MultiSelectionPropertyStorage"
    value_type='list'

    security = ClassSecurityInfo()

    def validityCheck(self, editablecontainer, region):
        prop = self.getPropertyInfoFor(editablecontainer, region)
        value = hasattr(editablecontainer, region.storagecontainerid, _marker)
        return prop and value is not _marker and \
               type(value) in (ListType, TupleType) and \
               prop.has_key('') and hasattr(editablecontainer, prop['select_variable'])


    _default_render_html_view=HTML("""\
        <div class="form-element">
        <select name="&dtml-id;:list" size="&dtml-list_size;">
          <dtml-in select_list prefix="pre">
             <option value="&dtml-pre_item;" <dtml-var "value==value_item and 'SELECTED' or ''">><dtml-var pre_key><dtml-var pre_item html_quote></option>
          </dtml-in>
        </select>
        </div>
        """, globals())

    def render_html_view(self, editablecontainer, region):
        REQUEST=getattr(editablecontainer ,'REQUEST', None)
        select_variable=region.property_select_variable
        select_list=getattr(editablecontainer,'select_variable',None)
        if select_list:
            return """<div class="form-text">No value for %s.</div>""" % select_variable
        if type(select_list) not in (ListType, TupleType): select_list=[select_list]
        value=self.manage_load(editablecontainer, region, region.getDefault())
        return self._default_render_html_view(self, REQUEST=REQUEST
                            , value=value
                            , select_list=select_list
                            , list_size=min(7, len(select_list))
                            , id=region.storagecontainerid
                            )


    _default_property_config=HTML("""\
        <dtml-with region>
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table>
          <tr>
            <td class="form-label">Label</td>
            <td><input type="text" name="property_label" value="&dtml.missing-property_label;"></td>
          </tr>
          <tr>
            <td class="form-label">Select Variable</td>
            <td><input type="text" name="property_select_variable" value="&dtml.missing-property_select_variable;"></td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def manage_view_storage_conf(self, region, REQUEST):
        return self._default_property_config(self, region=region, REQUEST=REQUEST)

    def manage_save_storage_conf(self, region, REQUEST):
        region.property_mode = REQUEST.get('property_mode',getattr(region,'property_mode', self.property_mode))
        region.property_label = REQUEST.get('property_label',getattr(region,'property_label', self.property_label))
        region.property_select_variable = REQUEST.get('property_select_variable',getattr(region,'property_select_variable', self.property_select_variable))


Globals.InitializeClass(MultiSelectionPropertyStorage)


def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(MultiSelectionPropertyStorage,   'MultiSelectionPropertyStorage')


