# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/Storage/PropertyStorage.py,v 1.19 2004/03/20 16:24:18 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.8 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.19 $'[11:-2]


import Globals
from Globals import HTML

from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Storage import EasyStorageBase, getAllListValues, getActiveListValues
from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'


class PropertyStorage(EasyStorageBase):
    """A PropertyStorage Object """

    storage_type="PropertyStorage"
    value_type='string'
    property_type = 'None'
    property_mode='wd'
    property_label=''
    property_select_variable=''

    security = ClassSecurityInfo()

    def _updateProperty(self, container, region, id, value, force_update=0):
        ####MLOG_TAB("PropStorage::_updateProperty")
        if force_update or not hasattr(aq_base(container), id):
            setattr(container, id, value)

    def valid_property_id(self, container, region, id):
        if not id or id[:1]=='_' or (id[:3]=='aq_') or (' ' in id):
            return 0
        return 1

    def _has_property_entry(self, container, region, id):
        for prop in container._properties:
            if prop['id']==id: return 1
        return 0

    def _init_select_variable(self, container, region, id):
        ####MLOG_TAB("PropStorage::_init_select_variable")
        if self.property_type in ('selection', 'multiple selection'):
            select_variable=getattr(region, 'property_select_variable', '')
            if not select_variable:
                select_variable=POSSIBLE_LINES+id
            if not hasattr(container, select_variable):
                container._setProperty(select_variable, [], 'lines')
            return select_variable

    def _create_property_entry(self, container, region, id):
        ###MLOG_TAB("PropStorage::_create_property_entry")
        if self.property_type in ('selection', 'multiple selection'):
            select_variable=self._init_select_variable(container, region, id)
            container._properties=container._properties + (
                {   'id':               id,
                    'type':             self.property_type,
                    'label':            getattr(region, 'property_label', ''),
                    'mode':             getattr(region, 'property_mode', 'wd'),
                    'select_variable':  select_variable
                },)

        else:
            container._properties=container._properties+(
                {
                    'id':       id,
                    'type':     self.property_type,
                    'label':    getattr(region, 'property_label', ''),
                    'mode':     getattr(region, 'property_mode', 'wd')
                },)

    def _update_property_entry(self, container, region, id):
        ###MLOG_TAB("PropStorage::_update_property_entry", id)
        if self._has_property_entry(container, region, id):
            new_props=()
            if self.property_type in ('selection', 'multiple selection'):
                select_variable=self._init_select_variable(container, region, id)
            for prop in container._properties:
                if prop['id']==id:
                    prop['type']=self.property_type
                    prop['label']=getattr(region, 'property_label', '')
                    prop['mode']=getattr(region, 'property_mode', 'wd')
                    if self.property_type in ('selection', 'multiple selection'):
                        prop['select_variable']=select_variable
                    new_props=new_props+(prop,)
                    MLOG_TAB("PropStorage::_update_property_entry::prop", prop)
                else:
                    new_props=new_props+(prop,)
            container._properties=new_props
        else:
            self._create_property_entry(container, region, id)

    def _setProperty(self, container, region, id, value, force_update=0):
        ###MLOG_TAB("PropStorage::_setProperty")
        container._wrapperCheck(value)
        if not self.valid_property_id(container, region, id):
            raise 'Bad Request', 'Invalid or duplicate property id'

        self._update_property_entry(container, region, id) #Does both add and update

        if self.property_type in ('multiple selection', 'tokens', 'lines') and type(value) not in (ListType, TupleType):
            raise "Value should be a list"
            value=[value]

        if force_update or not hasattr(aq_base(container), id):
            setattr(container, id, value)

    def getPropertyInfoFor(self, container, region):
        id=region.storagecontainerid
        for prop in container._properties:
            if prop['id']==id:
                return prop
        return None


    def compareHistoryItem(self, container, region, item):
        ####MLOG_TAB("PropStorage::compareHistoryItem", self.storage_type)
        res=[]
        add=res.append
        if item.storage_type != self.storage_type:   add('storage_type')
        if item.property_type != self.property_type: add('property_type')
        if item.property_mode != region.property_mode: add('property_mode')
        if item.property_label != region.property_label: add('property_label')
        if item.property_select_variable != region.property_select_variable: add('property_select_variable')

        if item.last_changed != region.lastChanged(): add('lastChanged')
        return res

    def validityCheck(self, container, region):
        #MLOG_TAB("PropStorage::validityCheck", self.storage_type)
        prop=self.getPropertyInfoFor(container, region)
        return prop and hasattr(container, region.storagecontainerid)


    def do_initilized(self, container, region):
        ####MLOG_TAB("PropStorage::do_initilized", self.storage_type)
        if getattr(region, 'default', None) is None: self.default=''
        self.do_add(container, region, region.getDefault())


    def do_add(self, container, region, value):
        ###MLOG_TAB("PropStorage::do_add", self.storage_type)
        self._setProperty(container, region, region.storagecontainerid, value, force_update=1)
        self._initRSHistoryItem(container, region)
        self._setLastChanged(container, region, region.lastChanged())


    def do_get(self, container, region, default=None):
        ####MLOG_TAB("PropStorage::do_get", self.storage_type, region.storagecontainerid, default)
        return getattr(container, region.storagecontainerid, default)

    def do_transform(self, container, region, item):
        ###MLOG_TAB("PropStorage::do_transform", self.storage_type)
        old_storage=item.getStorage()
        if old_storage:
            value=old_storage.getValueForType(container, region, self.value_type)
            #try:
            #    value=old_storage.getValueForType(container, region, self.value_type)
            #except:
            #    value=region.getDefault()
        else:
            value=self.do_get(self, container, region, default=region.getDefault())
        self.do_add(container, region, value)


    def do_transform_property(self, container, region, item):
        self._update_property_entry(container, region, region.storagecontainerid)

    def manage_initilized(self, container, region):
        item=self.getRSHistoryItem(container, region)
        if item:
            changes=self.compareHistoryItem(container, region, item)
            if 'storage_type' in changes or 'property_type' in changes:
                self.do_transform(container, region, item)
            elif 'property_label' in changes or 'property_mode' in changes or 'property_select_variable' in changes:
                self.do_transform_property(container, region, item)
        else:
            self.do_add(container, region, region.getDefault())
        self._initRSHistoryItem(container, region)
        self._setLastChanged(container, region, region.lastChanged())

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, region, value=None):
        if self.hasChanged(editablecontainer, region):
            self.manage_initilized(editablecontainer, region)
        if value is None:
            value=self.do_get(editablecontainer, region, region.getDefault())
        self.do_add(editablecontainer, region, value)

        #_storage = getattr(aq_base(editablecontainer),'_' + name + '_storage',None)
        #if _storage is not None and _storage!=self.id:
        #    storage = self.getStorage(_storage)
        #    if storage is not None:
        #        storage.manage_delete(name, editablecontainer)
        #    else: raise "StorageException", "Need to have storage [%s] installed to be able to work with this region." % (_storage,)

    security.declarePrivate('manage_load')
    def manage_load(self, container, region, default=_marker):
        if self.hasChanged(container, region) or not self.validityCheck(container, region):
            self.manage_initilized(container, region)
        value = self.do_get(container, region, default=default)
        if value is _marker: return region.getDefault()
        return value

    def manage_delete(self, container, region):
        #Remove a property, if it's allowed
        if self.manage_delete_allowed(container, region):
            #Remove select_Variable properties if the exist
            prop=self.getPropertyInfoFor(container, region)
            if prop and prop.has_key('select_variable') and container.hasProperty(prop['select_variable']):
                container._delProperty(prop['select_variable'])
            id=region.storagecontainerid
            if hasattr(container, id): delattr(container,id)
            container._properties=tuple(filter(lambda i, n=id: i['id'] != n, container._properties))


    def manage_compare(self, container, region, obj):
        local_value=self.manage_load(container, region, _marker)
        if local_value is _marker: return 0
        return obj==local_value

    def manage_copy(self, container, region, parent):
        local_value=self.manage_load(container, region, _marker)
        if local_value is _marker:
            return
        if hasattr(parent,'_setProperty'):
            if parent.hasProperty(region.storagecontainerid,):
                if parent.getPropertyType(region.storagecontainerid) == self.property_type:
                    parent._updateProperty(region.storagecontainerid, value)
                    return
                else:
                    if 'd' in filter(lambda x,id=region.storagecontainerid: x['id']==id ,parent._properties)[0]['mode']:
                        container._delProperty(region.storagecontainerid)
                    else:
                        raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (region.storagecontainerid,container.getPropertyType(region.storagecontainerid) ,self.property_type)
            if type_converters.has_key(self.property_type):
                value=type_converters[self.property_type](value)
            self._setProperty(parent, region, region.storagecontainerid, value)
        else:
            setattr(parent, region.storagecontainerid, local_value)

    def manage_diff(self, container, region, obj):
        raise 'Not implemented'

    def render_html_view(self, container, region):
        return "%s (%s)" % (self.manage_load(container, region, region.getDefault()) or '--', self.property_type)


    _default_property_config=HTML("""\
        <dtml-with region>
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table>
          <tr>
            <td class="form-label">Property Lable</td>
            <td><input type="text" name="property_label" value="&dtml.missing-property_label;"></td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def manage_view_storage_conf(self, region, REQUEST):
        return self._default_property_config(self, region=region, REQUEST=REQUEST)

    def manage_save_storage_conf(self, region, REQUEST):
        region.property_mode = REQUEST.get('property_mode',getattr(region,'property_mode', self.property_mode))
        region.property_label = REQUEST.get('property_label',getattr(region,'property_label', self.property_label))
        region.property_select_variable = REQUEST.get('property_select_variable',getattr(region,'property_select_variable', self.property_select_variable))

Globals.InitializeClass(PropertyStorage)


class DatePropertyStorage(PropertyStorage):
    """A DatePropertyStorage Object """
    property_type = 'date'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            cols=max(atoi(req.dtpref_cols)-15,20)

        return """<input type="text" name="%s:date" value="%s" size="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    , cols
                    )


Globals.InitializeClass(DatePropertyStorage)


class StringPropertyStorage(PropertyStorage):
    """A StringPropertyStorage Object """
    property_type = 'string'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            cols=max(atoi(req.dtpref_cols)-15,20)

        return """<input type="text" name="%s:string" value="%s" size="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    , cols
                    )

Globals.InitializeClass(StringPropertyStorage)


class TextPropertyStorage(PropertyStorage):
    """A TextPropertyStorage Object """
    property_type = 'text'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        rows=20
        cols="100%"
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            rows=max(atoi(req.dtpref_rows),20)
            if req.dtpref_cols[-1]=='%':
                cols= str(min(100, max(25,int(req.dtpref_cols[:-1]))))+'%'
            else:
                cols=str(max(35,int(req.dtpref_cols)))

        return """<textarea name="%s:text" rows="%s" cols="%s">%s</textarea>"""  % (
                      name
                    , rows
                    , cols
                    , self.manage_load(name, editablecontainer, region.default)
                    )


Globals.InitializeClass(TextPropertyStorage)


class BooleanPropertyStorage(PropertyStorage):
    """A BooleanPropertyStorage Object """
    property_type = 'boolean'

    security = ClassSecurityInfo()

    def _saveProperty(self, name, value, editablecontainer):
        setattr(aq_base(editablecontainer),'_' + name + '_storage',self.id)
        if editablecontainer.hasProperty(name):
            if editablecontainer.getPropertyType(name) == self.property_type:
                editablecontainer._updateProperty(name, value)
                return
            else:
                if 'd' in filter(lambda x,name=name: x['id']==name ,editablecontainer._properties)[0]['mode']:
                    editablecontainer._delProperty(name)
                else:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (name,editablecontainer.getPropertyType(name) ,self.property_type)
        editablecontainer._setProperty(name, not not value, self.property_type)

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, region, value=None):
        _storage = getattr(aq_base(editablecontainer),'_' + name + '_storage',None)
        if _storage is not None and _storage!=self.id:
            storage = self.getStorage(_storage)
            if storage is not None:
                storage.manage_delete(name, editablecontainer)
            else: raise "StorageException", "Need to have storage [%s] installed to be able to work with this region." % (_storage,)

        self._saveProperty(name,value,editablecontainer)

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="hidden" name="%s:boolean:default" value="0">
               <input type="checkbox" name="%s:boolean" value="1" %s>"""  % (
                      name
                    , name
                    , self.manage_load(name, editablecontainer, region.default) and 'checked' or ''
                    )


Globals.InitializeClass(BooleanPropertyStorage)


class IntPropertyStorage(PropertyStorage):
    """A IntPropertyStorage Object """
    property_type = 'int'

    def render_html_view(self, name, editablecontainer, region):

        return """<input type="text" name="%s:int" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )

Globals.InitializeClass(IntPropertyStorage)


class FloatPropertyStorage(PropertyStorage):
    """A FloatPropertyStorage Object """
    property_type = 'float'

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="text" name="%s:float" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )

Globals.InitializeClass(FloatPropertyStorage)


class LongPropertyStorage(PropertyStorage):
    """A LongPropertyStorage Object """
    property_type = 'long'

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="text" name="%s:long" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )


Globals.InitializeClass(LongPropertyStorage)

class LinesPropertyStorage(PropertyStorage):
    """A LinesPropertyStorage Object """
    property_type = 'lines'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        rows=20
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            rows=max(atoi(req.dtpref_rows),20)
            if dtpref_cols[-1]=='%':
                cols= str(min(100, max(25,int(dtpref_cols[:-1])+dc)))+'%'
            else:
                cols=str(max(35,int(dtpref_cols)+dc))

        lines=self.manage_load(name, editablecontainer, region.default)
        if type(lines)!=StringType:
            lines = '\n'.join(lines)
        return """<textarea name="%s:lines" rows="%s" cols="%s">%s</textarea>"""  % (
                      name
                    , rows
                    , cols
                    , lines)
Globals.InitializeClass(LinesPropertyStorage)


class TokensPropertyStorage(PropertyStorage):
    """A TokensPropertyStorage Object """
    property_type = 'tokens'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            cols=max(atoi(req.dtpref_cols)-15,20)

        tokens=self.manage_load(name, editablecontainer, region.default)
        if type(tokens)!=StringType:
            tokens = ' '.join(tokens)
        return """<input type="text" name="%s:tokens" value="%s" size="%s">"""  % (name, tokens, cols)


Globals.InitializeClass(TokensPropertyStorage)


class SelectionPropertyStorage(PropertyStorage):
    """A SelectionPropertyStorage Object """
    property_type = 'selection'

    security = ClassSecurityInfo()

    def _saveProperty(self, name, value, editablecontainer):
        setattr(aq_base(editablecontainer),'_' + name + '_storage',self.id)
        if type(value) not in (ListType, TupleType):
            value=str(value)
            value=value.replace('\r','')
            value=value.split('\n')
            active_value=getActiveListValues(value)
            if active_value: active_value=active_value[0]
            else: active_value=''

        if editablecontainer.hasProperty(name):
            if editablecontainer.hasProperty(name) and editablecontainer.getPropertyType(name) == self.property_type:
                editablecontainer._updateProperty(name, active_value)
                return
            else:
                if 'd' in filter(lambda x,name=name: x['id']==name ,editablecontainer._properties)[0]['mode']:
                    editablecontainer._delProperty(name)
                else:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (name,editablecontainer.getPropertyType(name) ,self.property_type)


        if not hasattr(editablecontainer, POSSIBLE_LINES+name) or editablecontainer.hasProperty(POSSIBLE_LINES+name) and editablecontainer.getPropertyType(POSSIBLE_LINES+name) != 'lines':
            if editablecontainer.hasProperty(POSSIBLE_LINES+name) and editablecontainer.getPropertyType(POSSIBLE_LINES+name) != 'lines':
                try:
                    editablecontainer._delProperty(POSSIBLE_LINES+name)
                except:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (POSSIBLE_LINES+name,editablecontainer.getPropertyType(POSSIBLE_LINES+name) ,'lines')

            editablecontainer._setProperty(POSSIBLE_LINES+name, getAllListValues(value), 'lines')

        if type_converters.has_key(self.property_type):
            active_value=type_converters[self.property_type](active_value)

        editablecontainer._setProperty(name, POSSIBLE_LINES+name, self.property_type)
        editablecontainer._updateProperty(name, active_value)


    def render_html_view(self, name, editablecontainer, region):
        #value=self.manage_load(name, editablecontainer, region.default)
        #listmethod=region.default
        return "%s (%s)" % (self.manage_load(name, editablecontainer, region.default) or '--', self.property_type)



Globals.InitializeClass(SelectionPropertyStorage)


class MultiSelectionPropertyStorage(PropertyStorage):
    """A MultiSelectionPropertyStorage Object """
    property_type = 'multiple selection'

    security = ClassSecurityInfo()

    def _saveProperty(self, name, value, editablecontainer):
        setattr(aq_base(editablecontainer),'_' + name + '_storage',self.id)
        if type(value) not in (ListType, TupleType):
            value=str(value)
            value=value.replace('\r','')
            value=value.split('\n')
            active_value=getActiveListValues(value)
            if active_value: active_value=active_value
            else: active_value=[]

        if editablecontainer.hasProperty(name):
            if editablecontainer.getPropertyType(name) == self.property_type:
                editablecontainer._updateProperty(name, getAllListValues(value))
                return
            else:
                if 'd' in filter(lambda x,name=name: x['id']==name ,editablecontainer._properties)[0]['mode']:
                    editablecontainer._delProperty(name)
                else:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (name,editablecontainer.getPropertyType(name) ,self.property_type)
        if type_converters.has_key(self.property_type):
            value=type_converters[self.property_type](value)


        if not hasattr(editablecontainer, POSSIBLE_LINES+name) or editablecontainer.hasProperty(POSSIBLE_LINES+name):
            if editablecontainer.hasProperty(POSSIBLE_LINES+name) and editablecontainer.getPropertyType(POSSIBLE_LINES+name) != 'lines':
                try:
                    editablecontainer._delProperty(POSSIBLE_LINES+name)
                except:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (POSSIBLE_LINES+name,editablecontainer.getPropertyType(POSSIBLE_LINES+name) ,'lines')
            editablecontainer._setProperty(POSSIBLE_LINES+name, getAllListValues(value), 'lines')
        editablecontainer._setProperty(name, POSSIBLE_LINES + name, self.property_type)
        editablecontainer._updateProperty(name, getActiveListValues(value))

    security.declarePrivate('manage_load')
    def manage_load(self,name,editablecontainer,default=None):
        value = getattr(aq_base(editablecontainer), name, default)
        if type(value) in (ListType, TupleType):
            return '\n'.join(value)
        return value

    def render_html_view(self, name, editablecontainer, region):
        return "%s (%s)" % (self.manage_load(name, editablecontainer, region.default) or '--', self.property_type)


Globals.InitializeClass(MultiSelectionPropertyStorage)



def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(DatePropertyStorage,     'DatePropertyStorage')
    context.registerEasyStorage(StringPropertyStorage,   'StringPropertyStorage')
    context.registerEasyStorage(TextPropertyStorage,     'TextPropertyStorage')
    context.registerEasyStorage(IntPropertyStorage,     'IntPropertyStorage')
    context.registerEasyStorage(FloatPropertyStorage,   'FloatPropertyStorage')
    context.registerEasyStorage(LongPropertyStorage,   'LongPropertyStorage')
    context.registerEasyStorage(BooleanPropertyStorage, 'BooleanPropertyStorage')
    context.registerEasyStorage(LinesPropertyStorage, 'LinesPropertyStorage')
    context.registerEasyStorage(TokensPropertyStorage, 'TokensPropertyStorage')

    context.registerEasyStorage(SelectionPropertyStorage,   'SelectionPropertyStorage')
    context.registerEasyStorage(MultiSelectionPropertyStorage,   'MultiSelectionPropertyStorage')

    #context.registerEasyStorage(KeywordsPropertyStorage,'KeywordsPropertyStorage')

