# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/Storage/Storage.py,v 1.13 2004/03/20 16:24:18 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#


__doc__="""Storage

$Colliberty: Storage.py,v 0.0.1 2001/05/14 22:06:00 johanc Exp $"""
__version__='$Revision: 1.13 $'[11:-2]


import Globals
from Products.EasyEditor.EasyEditorRegistry import EasyStorage
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from ExtensionClass import Base

from Products.EasyCore.EasyConverters import getConverterFromTo, getTypeDefaultValue


from Products.EasyCore import MLOG_TAB


_marker=[]

DELETE_NOT_ALLOWED= (
    'title',
    'nav_title',
    'subjects',
    'description',
    'language',
    'expiration_date',
    'effective_date',
    'creation_date',
    )

RESERVEDIDS=('storage_type','property_type','property_mode','property_label','property_label','property_select_variable','last_changed','default','acquisition_blocks_on_creation','templateid','storagecontainerid','id','storage','value','value_type',)

class RSHistory(Globals.Persistent):
    """\
    RSHistory stands for Region Storage History
    """
    def __init__(self):
        self._history={}

    def addHistoryItem(self, item):
        history=self._history
        ###templateid=getattr(item, 'templateid', None)
        storagecontainerid=item.storagecontainerid
        history[storagecontainerid]=item
        self._history=history

    def getHistoryItem(self, storage_id, default=None):
        return self._history.get(storage_id, default)

    def getItemsByTemplateId(self, template_id):
        items=[]
        for id, item in self._history.items():
            if id[0]==template_id: items.append(item)
        return items

class RSHistoryItem(Globals.Persistent):
    """\
    RSHistory stands for Region Storage History
    """

    def __init__(self, editablecontainer, region, storage, **kw):
        apply(self.update, (editablecontainer, region, storage), kw)

    def update(self, editablecontainer, region, storage, **kw):
        self.schema=filter(lambda id, RI=RESERVEDIDS: id not in RI, kw.keys())
        for id in self.schema: setattr(self, id, kw[id])
        getTemplateId=getattr(aq_base(editablecontainer), 'getTemplateId', None)
        if getTemplateId and callable(getTemplateId):
            templateid = getTemplateId()
        else:
            templateid = None
        self.templateid=templateid
        self.storagecontainerid = getattr(region, 'storagecontainerid', None)
        self.id = getattr(region, 'id', None)
        self.storage = getattr(region, 'storage', None)
        self.default = getattr(region, 'default', None)
        self.acquisition_blocks_on_creation = getattr(region, 'acquisition_blocks_on_creation', None)
        self.last_changed = getattr(region, '_last_changed', None)

        self.property_type = getattr(storage, 'property_type', None)
        self.property_mode = getattr(storage, 'property_mode', None)
        self.property_label = getattr(storage, 'property_label', None)
        self.property_select_variable = getattr(storage, 'property_select_variable', None)
        self.storage_type = getattr(storage, 'storage_type', None)


    def setValue(self, value, value_type=None):
        self.value=value
        self.value_type=value_type

    def getValue(self, value_type=None, default=_marker):
        if self.value_type==value_type:
            return self.value
        if default is _marker: raise "Invalid ValueType"
        return default

    def getStorage(self):
        return EasyStorage.get(self.storage, None)



class EasyStorageBase(Base):

    security = ClassSecurityInfo()
    storage_type="Storage Type Unknown"
    value_type='string'

    def __init__(self,id):
        self.id = id

    def hasStorage(self,storageid):
        return EasyStorage.has_key(storageid)

    def getStorage(self,storageid,default=None):
        return EasyStorage.get(storageid, default)

    def storageIds(self,spec=None):
        return EasyStorage.keys()

    def storageValues(self,spec=None):
        return EasyStorage.values()

    def storageItems(self,spec=None):
        return  EasyStorage.items()

    def manage_save(self, editablecontainer, region, value=None):
        if self.hasChanged(editablecontainer, region):
            self.manage_initilized(editablecontainer, region)
        if value is None:
            value=self.do_get(editablecontainer, region, region.getDefault())
        setattr(editablecontainer, region.storagecontainerid, value)

    def manage_load(self, editablecontainer, region, default=None):
        return self.do_get(editablecontainer, region, default=default)

    def do_get(self, editablecontainer, region, default=None):
        return getattr(aq_base(editablecontainer), region.storagecontainerid, default)


    ## TESTS

    def compareHistoryItem(self, editablecontainer, region, item):
        res=[]
        add=res.append
        if item.storage_type != self.storage_type:   add('storage_type')
        if item.last_changed != region.lastChanged(): add('lastChanged')
        return res


    def validityCheck(self, editablecontainer, region):
        return hasattr(editablecontainer, region.storagecontainerid)

    ## Actions

    def do_add(self, editablecontainer, region, value):
        setattr(editablecontainer, region.storagecontainerid, value)
        self._initRSHistoryItem(editablecontainer, region)
        self._setLastChanged(editablecontainer, region, region.lastChanged())

    def do_initilized(self, editablecontainer, region):
        if getattr(region, 'default', None) is None: self.default=''
        self.do_add(editablecontainer, region.storagecontainerid, region.getDefault())

    def do_transform(self, editablecontainer, region, item):
        old_storage=item.getStorage()
        if old_storage:
            value=old_storage.getValueForType(editablecontainer, region, self.value_type)
            old_storage.manage_delete(editablecontainer, region)
            self.do_add(editablecontainer, region, value)
            return
        self.do_initilized(editablecontainer, region)


    ## Remote Value Transformation

    def getValueForType(self, editablecontainer, region, value_type):
        """\
        Returns the value of the requested type if possible otherwice default is returned.
        """
        MLOG_TAB("Storage:getValueForType:: FROM (self.value_type)", self.value_type)
        MLOG_TAB("Storage:getValueForType:: TO   (value_type)", value_type)
        if hasattr(self, 'do_get_converters') and callable(self.do_get_converters):
            converter=self.do_get_converters(self.value_type, value_type)
        else:
            converter=getConverterFromTo(self.value_type, value_type)
        if converter:
            value=self.do_get(editablecontainer, region, None)
            ###Add try:?
            if value: return converter(value)
        return region.getDefault()


    ## XXX Interface

    def manage_initilized(self, editablecontainer, region):
        if self.hasChanged(editablecontainer, region) or not self.validityCheck(editablecontainer, region):
            self.do_transform()

    def manage_delete(self, editablecontainer, region):
        if hasattr(editablecontainer, region.storagecontainerid):
            delattr(editablecontainer, region.storagecontainerid)
        self._setLastChanged(editablecontainer, region, region.lastChanged())




    def manage_delete_allowed(self, editablecontainer, region):
        allowed = region.storagecontainerid not in DELETE_NOT_ALLOWED
        return allowed

    def manage_clone(self, editablecontainer, region, parent):
        raise 'Not implemented'

    def manage_copy(self, editablecontainer, region, parent):
        raise 'Not implemented'

    def manage_compare(self, editablecontainer, region, obj):
        raise 'Not implemented'

    def manage_diff(self, editablecontainer, region, obj):
        raise 'Not implemented'

    def render_html_view(self, editablecontainer, region):
        raise 'Not implemented'

    def manage_modification_date(self, name, editablecontainer):
        return editablecontainer.bobobase_modification_time()

    def render_html_view(self, name, editablecontainer, region):
        raise 'Not implemented'

    security.declarePrivate('getStorageType')
    def getStorageType(self):
        return self.storage_type

    def _setLastChanged(self, editablecontainer, region, time_stamp):
        setattr(editablecontainer,'_region_%s_last_changed'%region.id, time_stamp)

    def hasChanged(self, editablecontainer, region):
        try:
            ts1=region.lastChanged()
        except:
            ts1=None
        try:
            ts2=getattr(editablecontainer, '_region_%s_last_changed'%region.id,None)
        except:
            ts2=None
        return ts1!=ts2

    ##Storage Init, Config and Change

    def manage_view_storage_conf(self, region, REQUEST):
        return '<p>No storage configuration available.</p>'

    def manage_save_storage_conf(self, region, REQUEST):
        return '<p>No storage configuration available.</p>'

    ##Default
    def manage_view_default_conf(self, region, REQUEST):
        return """<textarea name="%s" cols="%s" rows="%s">%s</textarea>""" % ('default', '50', '10', region.getDefault())

    def manage_save_default_conf(self, region, REQUEST):
        region.default = REQUEST.get('default', getattr(region, 'default', None))

    def getDefault(self, region):
        return getattr(region, 'default', None)

    ###
    def _lookupRSHistoryStorage(self, editablecontainer, region):
        history=getattr(aq_base(editablecontainer), '_storage_history', None)
        if history is None: history=RSHistory()
        return history

    def getRSHistoryItem(self, editablecontainer, region, default=None):
        history = self._lookupRSHistoryStorage(editablecontainer, region)
        return history.getHistoryItem(region.storagecontainerid, default)

    def _initRSHistoryItem(self, editablecontainer, region):
        history = self._lookupRSHistoryStorage(editablecontainer, region)
        item = history.getHistoryItem(region.storagecontainerid, None)
        if item:
            item.update(editablecontainer, region, self)
        else:
            item=RSHistoryItem(editablecontainer, region, self)
        history=self._lookupRSHistoryStorage(editablecontainer, region)
        history.addHistoryItem(item)
        editablecontainer._storage_history=history
        return item

    def _markRSHistoryItemAsDeleted(self, editablecontainer, region):
        item = self.getRSHistoryItem(editablecontainer, region)
        if not item: item=_initRSHistoryItem()
        self.saveValueInRSHistoryItem(item, editablecontainer, region)

    def saveValueInRSHistoryItem(self, item, editablecontainer, region):
        """
        This method maybe overrided to be able to save more advanced
        storages data.
        """
        value=self.do_get(editablecontainer, region, None)
        item.setValue(value, 'String')

    def getType(self):
        return self.value_type

Globals.InitializeClass(EasyStorageBase)


def getAllListValues(value):
    if type(value) not in (ListType, TupleType):
        value=str(value)
        value=value.replace('\r','')
        value=value.split('\n')
    value=map(lambda x: x.strip() , value)
    value=map(lambda x: x and x[0] in ('-','#') and x[1:].strip() or x.strip(), value)
    return value


def getActiveListValues(value):
    if type(value) not in (ListType, TupleType):
        value=str(value)
        value=value.replace('\r','')
        value=value.split('\n')
    value=map(lambda x: x.strip() , value)
    value=filter(lambda x: x[0] not in ('-','#'), value)
    return value

