# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyEditor/Styles/Styles.py,v 1.4 2003/06/27 16:28:02 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#
__version__='$Revision: 1.4 $'[11:-2]
__doc__="""Styles

$Colliberty: Styles.py,v 0.0.1 2001/06/04 15:40:00 johanc Exp $"""

import Globals
import sys
from Globals import HTML, DTMLFile, Persistent, MessageDialog
from string import join,split
from OFS.SimpleItem import SimpleItem
from Products.EasyCore import EasyCorePermissions
from AccessControl import ClassSecurityInfo
from Products.EasyEditor.Styles.StylesConf import *
from Products.EasyCore.PropertyTextParser import PropertyTextParser, PropertyTextGenerator
from webdav.WriteLockInterface import WriteLockInterface
from OFS.Cache import Cacheable
#from webdav.Lockable import ResourceLockedError
#from OFS.History import html_diff, Historical


class CSSStyle(SimpleItem, Cacheable):
    """A CSSStyle PlugIn Object """

    security = ClassSecurityInfo()
    meta_type = STYLEMETATYPE

    _v_errors=''

    __implements__ = (WriteLockInterface,)
    content_type="text/plain"

    def __init__(self, id, title, csscode):
        self.id = id
        self.title = title
        self.csscode = csscode

    manage_options = (
            {'label':'Edit',   'action':'manage_main'},
    )

    _hidden_properties = (
                            {   'id':'csscode',
                                'label':'CSS Code',
                                'type':'text'
                            },
                            {   'id':'title',
                                'label':'Title',
                                'type':'string'
                            },
                        )

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_main')
    manage_main = DTMLFile('manage_main_CSSStyle', globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_edit')
    def manage_edit(self, title, csscode, REQUEST=None):
        """does this really need a doc string?"""
        self.title = title
        self.csscode = csscode
        self.setEditorDirty()
        if REQUEST is not None:
            return self.manage_main(self,REQUEST,management_view='Edit')

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        CSSStyle.inheritedAttribute('manage_afterAdd')(self, item, container)
        if hasattr(self,'setEditorDirty') and callable(self.setEditorDirty):
            self.setEditorDirty()


    def read(self):
        error=''
        try:
            generator = PropertyTextGenerator()
            generator.renderFromInstance(self, self._hidden_properties)
            if self._v_errors:
                return "%s\n%s" %(self._v_errors, generator.getCode())
            return generator.getCode()
        except:
            try:
                error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
                generator_errors='\n'.join(map(lambda err: "# " + str(err), generator.getErrors()))
                if self._v_errors:
                    return error + "%s\n%s" % (self._v_errors, generator_errors)
                return local_errors
            except:
                pass
        return error + self._v_errors


    def munge(self, body):
        self._v_errors = ''
        try:
            parser = PropertyTextParser()
            parser.parseToInstance(self, self._hidden_properties, body)
        except:
            error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
            self._v_errors = error + '\n'.join(map(lambda err: "# " + str(err), parser.getErrors()))


    def get_size(self):
        return len(self.read())

    def content_type(self):
        return self.content_type


    #FTP Support
    security.declareProtected('View management screens', 'document_src')
    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return unprocessed document source."""
        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', 'text/plain')
        return self.read()

    ## Protocol handlers
    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests."""
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body=REQUEST.get('BODY', '')
        self.munge(body)
        self.ZCacheable_invalidate()
        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected('FTP access', 'manage_FTPget')
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.read()



manage_add_CSSStyleForm = DTMLFile('manage_add_CSSStyleForm', globals())

def manage_add_CSSStyle(self, id, csscode, title, REQUEST=None):
    """Add a CSSStyle"""
    self._setObject(id, CSSStyle(id, title, csscode))
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)

Globals.InitializeClass(CSSStyle)

def initialize(context):
    """Initialize Styles Product"""

    context.registerEasyTemplateHelperClass(
            CSSStyle,
            meta_type = STYLEMETATYPE,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_CSSStyleForm, manage_add_CSSStyle),
            icon='Styles/icons/CSSStyle.gif',
        )

