# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyNavigation/EasyNamedMenu.py,v 1.20 2003/06/15 16:21:30 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''EasyNavigator product: EasyNamedMenu.'''
__version__='1.8'

import Globals
from string import join, split
from types import DictType

from Globals import DTMLFile
import DateTime
from ComputedAttribute import ComputedAttribute
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from Acquisition import aq_get

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyNavigationConf import EASYEDITABLEMETATYPES, NAVIGATION_CATALOG_ID
from Products.EasyNavigation.EasyNavigation import NavigationBrain
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAware
from Products.EasyNavigation.EasyNavigatable import Navigatable
from OFS.SimpleItem import SimpleItem
from Products.EasyCore import EasyCorePermissions
from zLOG import LOG

class EasyNamedMenu(Navigatable, VirtualPathBase, EasyCatalogAware, EasyPresentationHelpers, SimpleItem):
    """Adds some default properties"""

    navigation_catalog = NAVIGATION_CATALOG_ID

    security = ClassSecurityInfo()

    __catalogs__=(NAVIGATION_CATALOG_ID,)

    meta_type = "EasyNamedMenu"

    isPrincipiaFolderish=1

    _dontindex=1

    _add_to_navigation_on_move = 0
    _auto_add_to_navigation = 0

    navigation_menus_data = {}

    manage_options = (
            {'label':'Navigation',     'action':'manage_navigatorForm'},
        ) + \
        SimpleItem.manage_options


    _properties = (
        {'id':'title', 'type': 'string', 'mode': 'w'},
        {'id':'navigation_title', 'type': 'string', 'mode': 'w'},
    )


    def __init__(self, id, title ):
        EasyNamedMenu.inheritedAttribute('__init__')(self, title, title)
        self.id=id

    def getAutoAddMenus(self):
        return ()

    #Override. (Why? This is not an objectManager? /LRE)
    # Because otherwise this logic would be moved to the DTML File,
    # which by the way aint a objectmanager nor has the objectmanager interface
    # a getSubObjects method.
    # This would probably make more sense to you if this method
    # was called getSubObjectsHelper?
    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getSubObjects')
    def getSubObjects(self):
        if hasattr(self, 'aq_parent'):
            self=self.aq_parent
        return map(lambda ob: (ob.physicalURL, ob.navigation_title + ' -- ' + ob.id), self.objectValues(spec=EASYEDITABLEMETATYPES))

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getMenuParent')
    def getMenuParent(self):
        if hasattr(self, 'aq_parent'): return self.aq_parent
        return self

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getMenuParentUrl')
    def getMenuParentUrl(self):
        if hasattr(self, 'aq_parent'):
            self=self.aq_parent
        return self.physicalURL


    security.declareProtected(EasyCorePermissions.View, 'getMenu')
    def getMenu(self, sort_on='order', sort_order=None, skip_root=None, query=None):
        if type(query)!= DictType: query={}
        tool = aq_get(self, NAVIGATION_CATALOG_ID, None, 1)
        if tool is not None:
            navigation_menu = self.physicalPath
            return tool.getMenu(navigation_menu,sort_on=sort_on,sort_order=sort_order,skip_root=skip_root, query=query)
        return []



Globals.InitializeClass(EasyNamedMenu)



manage_addEasyNamedMenuForm=DTMLFile('dtml/manage_addEasyNamedMenuForm', globals())

def manage_addEasyNamedMenu(self, id, title, REQUEST=None):
    """Add a EasyNamedMenu object"""
    ob=EasyNamedMenu(id, title)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)




def initialize(context):
    """Initialize EasyNamedMenu Product"""

    context.registerClass(
        EasyNamedMenu,
        permission=EasyCorePermissions.EasyNavigationChange,
        icon='icons/EasyNamedMenu.gif',
        constructors = ( manage_addEasyNamedMenuForm, manage_addEasyNamedMenu),
        meta_type = EasyNamedMenu.meta_type
    )



