# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyNavigation/EasyNavigatable.py,v 1.43 2003/10/29 18:48:22 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__ = """EasyNavigation initialization module."""
__version__='$Revision: 1.43 $'[11:-2]

import Globals
from string import join, split, lower

from Globals import DTMLFile
import DateTime,sys
from ComputedAttribute import ComputedAttribute
from AccessControl import getSecurityManager, ClassSecurityInfo
from ExtensionClass import Base
from Acquisition import aq_get, aq_parent, aq_base, aq_chain
from types import StringType

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyNavigationConf import EASYEDITABLEMETATYPES, NAVIGATION_CATALOG_ID
from Products.EasyCore.VirtualPath import VirtualPathBase, url2path, path2url, mergePaths
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAware
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyCore.BodyTabs import InitializeBodyTabs, BodyTabs

from Products.EasyCore import MLOG_TAB

class Navigatable(ManageOptionBase, VirtualPathBase, EasyCatalogAware, EasyPresentationHelpers, BodyTabs):
    """Adds some default properties"""


    security = ClassSecurityInfo()

    navigation_catalog = NAVIGATION_CATALOG_ID
    __catalogs__=(NAVIGATION_CATALOG_ID,)

    _add_to_navigation_on_move = 0
    _auto_add_to_navigation = 0

    _manage_options_navigation =(
        {'label':'Navigation',     'action':'manage_navigatorForm'},
    )

    security.declareProtected(ViewManagementScreens, 'manage_options_navigation')
    def manage_options_navigation(self):
        navigation_catalog = getattr(self, self.navigation_catalog, None)
        if navigation_catalog is not None:
            return self._manage_options_navigation

    use_manage_options = ((None, manage_options_navigation, None),)

    _properties = (
        {'id':'navigation_title', 'label':'Navigation Title', 'type': 'string', 'mode': 'w'},
    )

    navigation_menus_data = {}

    def __init__(self,title,navigation_title=''):
        self.title=title
        if navigation_title!='': self.navigation_title=navigation_title
        else: self.navigation_title=self.title
        self.navigation_menus_data={}

    security.declareProtected(EasyNavigationChange, 'manage_navigatorForm')
    manage_navigatorForm = DTMLFile('dtml/manage_NavigatorForm', globals()
                            , management_view='Navigation'
                            , body_tab={'label':'Edit','order':1, 'management_view':'Navigation'}
                            )


    def isDeletedInMove(self):
        return hasattr(aq_base(self),'_has_been_deleted')

    def isTopObjectInADelete(self):
        return hasattr(aq_base(self),'_top_object_in_a_delete')


    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        self.addAutoAdd()
        self.addLocalMenu(item.physicalPath)
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            object.manage_afterAdd(item, container)
            if s is None: object._p_deactivate()
        self.index_object()
        #Why limit to editables? if item.meta_type in EASYEDITABLEMETATYPES:
        if hasattr(container, 'reindex_object') and callable(container.reindex_object):
            container.reindex_object()
        #Clean up the _has_been_deleted flag, if set
        if hasattr(aq_base(self),'_has_been_deleted'): delattr(self,'_has_been_deleted')
        if hasattr(aq_base(self),'_top_object_in_a_delete'): delattr(self,'_top_object_in_a_delete')


    security.declarePrivate('manage_beforeBeforeDelete')
    def manage_beforeBeforeDelete(self, item, container):
        if self.isDeletedInMove(): return
        #Set _has_been_deleted flag
        self._has_been_deleted=1

        self.deleteAutoAdd()
        self.saveMenuLocally(item.physicalPath)

        for object in self.objectValues():
            if hasattr(aq_base(object), 'manage_beforeBeforeDelete'):
                try: s=object._p_changed
                except: s=0
                object.manage_beforeBeforeDelete(item, container)
                if s is None: object._p_deactivate()

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):

        if self is item:
            self._top_object_in_a_delete=1
            self.manage_beforeBeforeDelete(item, container)

        self.unindex_object()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            object.manage_beforeDelete(item, container)
            if s is None: object._p_deactivate()
        if hasattr(container, 'reindex_object') and callable(container.reindex_object):
            container.reindex_object()

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        self.removeDocumentFromAllRemoteMenus(item.getPhysicalPath())
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            object.manage_afterClone(item)
            if s is None: object._p_deactivate()
        self.index_object()


    def saveMenuLocally(self, rpath, delete=1):
        local_menu=[]
        for each in self.getMenuItems():
            ob = self.unrestrictedTraverse(each.physicalURL)
            cpath=ob.physicalPath
            npath=cpath
            if len(cpath)>=len(rpath) and cpath[:len(rpath)]==rpath:
                npath=cpath[len(rpath):]
            order = ob.navigation_menus_data[self.physicalPath][0]
            if delete: ob.deleteFromMenu(self.physicalPath)
            local_menu.append( ( npath, order) )
        self._v_local_menu=local_menu

    def addLocalMenu(self, rpath):
        local_menu=getattr(aq_base(self),'_v_local_menu', None)
        if local_menu:
            ncat=getattr(self, self.navigation_catalog, None)
            for npath, order in local_menu:
                o_npath=npath
                if len(npath)==0 or (len(npath)>0 and npath[0]!=''):
                    npath=rpath+npath
                try:
                    ob = self.unrestrictedTraverse(npath)
                    if hasattr(ob, 'addToMenu'):
                        ob.addToMenu(self.physicalPath, order)
                except: pass

    def removeDocumentFromAllRemoteMenus(self, base_path):
        """\
        Removed all menus not under the base_path.
        This is used by manage_afterClone to get rid of
        menu appearences not in the copied hierarcy.
        (Local menus (added by manage_afterAdd are retained)
        """
        nmd={}
        for menu_path, value in self.navigation_menus_data.items():
            if base_path==menu_path[:len(base_path)]:
                nmd[menu_path]=value
        self.navigation_menus_data=nmd



    def getObjectFromRelativePath(self, rpath, default=None):
        """\
        Returns a object corresponding to an absolute path (url),
        a relative path (url) or a relativ acquired object.

            Absolute Path starts with ('', ...) (URL: '/')

            Relative Path starts with ('.', ...) or ('..', ...) (URL: './' or '../')
            '.' means this object and is ignored
            '..' means this objects parent and is travesed backwards

            Acquired relative paths does not start with a '.' or '..'
            They are looked for inte the acqusition context.

        """
        rpath=url2path(rpath)
        self=self.getMenuParent() #Handles menus as subobjects (aka NamedMenus)
        if rpath[0]=='': #It's absolute
            try: return self.restrictedTraverse(rpath)
            except:
                try: return self.restrictedTraverse(mergePaths(self.getVirtualRootPath(),rpath))
                except: return default
        obj=self
        if rpath[0]!='..' and rpath[0]!='.': #Get Acquired
            if len(rpath)==1:
                return getattr(self, rpath[0], default)
            elif hasattr(self, rpath[0]):
                obj=getattr(self, rpath[0]) #fetch one with aq and travers it futher...
                rpath=rpath[1:]
            else:
                return default
        for rp in rpath:
            if rp!='.':
                if rp=='..' or rp=='parent': obj=aq_parent(obj)
                elif hasattr(aq_base(obj), rp): obj=getattr(obj, rp)
                else: return default
        return obj

    # should be public
    security.declareProtected(AccessContentsInformation, 'getAutoAddToNavigation')
    def getAutoAddToNavigation(self):
        if hasattr(self, 'getTemplateId') and hasattr(self,'EasyEditor'):
            ed = self.EasyEditor
            templateid = self.getTemplateId()
            template = ed.getTemplate(templateid, None)
            if template and hasattr(template, 'navigation_aware'):
                return not not template.navigation_aware
        return not not self._auto_add_to_navigation

    # should be public
    security.declareProtected(AccessContentsInformation, 'getAutoAddMenus')
    def getAutoAddMenus(self):
        add_to_menus = getattr(self, 'add_to_menus', None)
        if add_to_menus is None and hasattr(self, 'getTemplateId') and hasattr(self,'EasyEditor'):
            ed = self.EasyEditor
            templateid = self.getTemplateId()
            template = ed.getTemplate(templateid, None)
            if template and hasattr(template, 'add_to_menus'):
                add_to_menus=getattr(template,'add_to_menus', None)
        #print "add_to_menus", add_to_menus
        if add_to_menus and type(add_to_menus) in (ListType,TupleType) and len(add_to_menus)==1 and add_to_menus[0]=='':
            if callable(add_to_menus): add_to_menus=add_to_menus()
            add_to_menus=filter(None,add_to_menus)
            return tuple(map(lambda x: tuple(x.split(' ')), add_to_menus))
        return self.getAutoAddToNavigation() and (('..',),) or ()

    def addAutoAdd(self):
        navigation_catalog=getattr(self, self.navigation_catalog, None)
        if navigation_catalog:
            #Auto delete doesn't provide any extra functionality, i think?
            #if hasattr(aq_base(self), '_auto_deleted_menus'):
            #    #If _auto_deleted_menus is set, add menu (or not if _auto_deleted_menus=)[])
            #    #and ignore getAutoAddMenus.
            #    auto_deleted_menus=self._auto_deleted_menus
            #    delattr(self, '_auto_deleted_menus')
            #    if auto_deleted_menus:
            #        for menu in auto_deleted_menus:
            #            mobj=self.getObjectFromRelativePath(menu)
            #            if mobj:
            #                mpath=mobj.getPhysicalPath()
            #                if not self.hasMenu(mpath):
            #                    order = navigation_catalog.getHighestCountForMenu(mpath)
            #                    self.addToMenu(mpath, order)


            #If this object has been deleted, it is most probably a move operation
            #so we should disable auto add
            if not self.isDeletedInMove() or self.isTopObjectInADelete():
                auto_add_to=self.getAutoAddMenus()
                for menus in auto_add_to:
                    added=0
                    for menu in menus:
                        mobj=self.getObjectFromRelativePath(menu)
                        if mobj:
                            mpath=mobj.getPhysicalPath()
                            if not self.hasMenu(mpath):
                                order = navigation_catalog.getHighestCountForMenu(mpath)
                                self.addToMenu(mpath, order)
                                added=1
                    if added:
                        break


    def deleteAutoAdd(self):
        auto_add_to=self.getAutoAddMenus()
        if auto_add_to:
            #Only add _auto_deleted_menus if there are
            #menus controlled by getAutoAddMenus in
            #this context.
            #This ensures that if the menus (defined in getAutoAddMenus)
            #is empty the menu will not be added to menues (controlled by getAutoAddMenus
            #in the new location). Empty menus are also sticky.
            deleted=[]
            for menus in auto_add_to:
                for menu in menus:
                    mobj=self.getObjectFromRelativePath(menu)
                    if mobj:
                        mpath=mobj.getPhysicalPath()
                        if self.hasMenu(mpath):
                            self.deleteFromMenu(mpath)
                            deleted.append(menu)
            self._auto_deleted_menus=deleted




    # Navigation Interface

    security.declareProtected(View, 'getMenu')
    def getMenu(self
                    , object_path=None
                    , sort_on='order'
                    , sort_order=None
                    , skip_root=None
                    , query=None
                ):
        """\
        Use the EasyNavigation service to returns the list of
        menu entries for the menu object_path.
        """
        if type(query)!=type({}): query={}
        tool = aq_get(self, NAVIGATION_CATALOG_ID, None, 1)
        if tool is None: return []
        return tool.getMenu(
                      object_path=object_path
                    , sort_on=sort_on
                    , sort_order=sort_order
                    , skip_root=skip_root
                    , query=query
                    )

    security.declareProtected(View, 'getNavigationTrail')
    def getNavigationTrail(self
                                , object_path=None
                                , path_in=None
                                , current_path=None
                                , sort_on=None
                                , sort_order=None
                                , submenu=None
                                , skip_root=None
                                , query = None
                ):
        """\
        Use the EasyNavigation service to returns a list of
        the brain for the hierarcical path of the object
        If 'submenu' is true submenu are retrieved.
        """
        if type(query)!=type({}): query={}
        tool = aq_get(self, NAVIGATION_CATALOG_ID, None, 1)
        if tool is None: return []
        return tool.getMenu(
                              object_path=object_path
                            , path_in=path_in
                            , current_path=current_path
                            , sort_on=sort_on
                            , sort_order=sort_order
                            , submenu=submenu
                            , skip_root=skip_root
                            , query = query
                            )


    # MANAGEMENT METHODS
    security.declareProtected(EasyNavigationChange, 'manage_move_menu_items_first')
    def manage_move_menu_items_first(self,navigation_menu,selected_urls=[],all_urls=[],REQUEST=None):
        """Update all objects that is in the menu "navigation_menu" for this object"""
        order = 0
        navigation_menu_path = url2path(navigation_menu)
        for url in selected_urls:
            o = self.restrictedTraverse(url2path(url))
            o.changeOrder(navigation_menu_path, order)
            order = order + 1
        for url in all_urls:
            if url not in selected_urls:
                o = self.restrictedTraverse(url2path(url))
                o.changeOrder(navigation_menu_path, order)
                order = order + 1
        self.reindex_object()
        if REQUEST is not None:
            message="Order of menu [%s] updated" % (navigation_menu,)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Navigation')

    security.declareProtected(EasyNavigationChange, 'manage_move_menu_items_last')
    def manage_move_menu_items_last(self,navigation_menu,selected_urls=[],all_urls=[],REQUEST=None):
        """Update all objects that is in the menu "navigation_menu" for this object"""
        order = 0
        navigation_menu_path = url2path(navigation_menu)
        for url in all_urls:
            if url not in selected_urls:
                o = self.restrictedTraverse(url2path(url))
                o.changeOrder(navigation_menu_path, order)
                order = order + 1
        for url in selected_urls:
            o = self.restrictedTraverse(url2path(url))
            o.changeOrder(navigation_menu_path, order)
            order = order + 1
        self.reindex_object()
        if REQUEST is not None:
            message="Order of menu [%s] updated" % (navigation_menu,)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Navigation')

    security.declareProtected(EasyNavigationChange, 'manage_move_menu_items_up')
    def manage_move_menu_items_up(self,navigation_menu,selected_urls=[],all_urls=[],REQUEST=None):
        """Update all objects that is in the menu "navigation_menu" for this object"""
        order = 0
        first_urls = []
        navigation_menu_path = url2path(navigation_menu)
        for url in all_urls:
            if url in selected_urls: break
            order = order + 1
            first_urls.append(url)
        if order>0:
            order = order - 1
            first_urls = first_urls[:-1]
        for url in selected_urls:
            o = self.restrictedTraverse(url2path(url))
            o.changeOrder(navigation_menu_path, order)
            order = order + 1
        for url in all_urls:
            if url not in selected_urls and url not in first_urls:
                o = self.restrictedTraverse(url2path(url))
                o.changeOrder(navigation_menu_path, order)
                order = order + 1
        self.reindex_object()
        if REQUEST is not None:
            message="Order of menu [%s] updated" % (navigation_menu,)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Navigation')

    security.declareProtected(EasyNavigationChange, 'manage_move_menu_items_down')
    def manage_move_menu_items_down(self,navigation_menu,selected_urls=[],all_urls=[],REQUEST=None):
        """Update all objects that is in the menu "navigation_menu" for this object"""
        order = 0
        first_urls = []
        navigation_menu_path = url2path(navigation_menu)
        next_break = do_break = 0
        for url in all_urls:
            if do_break: break;
            if url not in selected_urls:
                o = self.restrictedTraverse(url2path(url))
                o.changeOrder(navigation_menu_path, order)
                order = order + 1
                first_urls.append(url)
                if next_break: do_break=1
            if url == selected_urls[-1:][0]:
                next_break=1;
        for url in selected_urls:
            o = self.restrictedTraverse(url2path(url))
            o.changeOrder(navigation_menu_path, order)
            order = order + 1
        for url in all_urls:
            if url not in selected_urls and url not in first_urls:
                o = self.restrictedTraverse(url2path(url))
                o.changeOrder(navigation_menu_path, order)
                order = order + 1
        self.reindex_object()
        if REQUEST is not None:
            message="Order of menu [%s] updated" % (navigation_menu,)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Navigation')

    def getMenuItemsAsText(self, show_physical=1, relative=0):
        if relative:
            return '\n'.join(map(lambda o: o.virtualURL, self.getMenuItems()))
        if not show_physical:
            return '\n'.join(map(lambda o: o.virtualURL, self.getMenuItems()))
        return '\n'.join(map(lambda o: o.physicalURL, self.getMenuItems()))

    _manage_change_order_text = DTMLFile('dtml/manage_change_order_text', globals()
                            , management_view='Navigation'
                            , body_tab={'label':'Text Edit','order':2, 'management_view':'Navigation', 'url':'manage_change_order_text'}
                            )

    security.declareProtected(EasyNavigationChange, 'manage_change_order_text')
    def manage_change_order_text(self, navigation_menu=None, additional_urls=[], REQUEST=None):
        """\
        Support for fast managing and ordering of menus in a textarea.
        """
        valid = []
        invalid = []
        message=''

        if navigation_menu and additional_urls:
            valid, invalid = self.manage_add_to_menu(navigation_menu=navigation_menu, additional_urls=additional_urls)
            if valid!=[]:
                message=message+"The documents [%s] has been added to menu [%s]" % (join(valid,', '),navigation_menu,)
            else:
                message=message+"No valid urls was submitted."
            if invalid!=[]:
                message=message+"<br>The following urls was not valid documents: [%s]" % (join(invalid,', '),)

        if REQUEST is not None:
            return self._manage_change_order_text(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            invalid_urls=invalid
                            )

    def manage_delete_all(self, navigation_menu=None, confirm=None, REQUEST=None):
        """\
        Delete all urls from menu
        """
        if confirm:
            if navigation_menu is None:
                navigation_menu_path = self.physicalPath
            else:
                navigation_menu_path = url2path(navigation_menu)
            valid = []
            for each in self.getMenuItems():
                try:
                    o = self.unrestrictedTraverse(each.physicalURL)
                    o.deleteFromMenu(navigation_menu_path)
                    valid.append(self.physicalURLToVirtualURL(url))
                except: pass
            self.reindex_object()

        if REQUEST is not None:
            if valid!=[]:
                message="The documents [%s] has been deleted from the menu." % (join(valid,', '),)
            else:
                message="No object deleted."
            return self._manage_change_order_text(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Text Interface')

    security.declareProtected(EasyNavigationChange, 'manage_add_to_menu')
    def manage_add_to_menu(self, navigation_menu=None, local_urls=[], additional_urls=[], REQUEST=None):
        #argument: order=None,  has been removed
        """\
        Adds another object to navigation_menu (which could
        be this object or some "named" menu.
        """
        valid = []
        invalid = []

        selected_unique_urls =[]
        for url in local_urls:
            if url not in selected_unique_urls: selected_unique_urls.append(url)
        for url in additional_urls:
            #Don't add empty lines ('' means the Root but you must write 'root', 'top' or 'home' to add the Root.)
            if url!='':
                if url=="/" or lower(url)=="root" or lower(url)=="top" or lower(url)=="home":
                    url = self.getSiteRootURL()
                if url not in selected_unique_urls: selected_unique_urls.append(url)
        if navigation_menu is None:
            navigation_menu_path = self.physicalPath
        else:
            navigation_menu_path = url2path(navigation_menu)

        if hasattr(self, self.navigation_catalog):
            #Don't do anything if there isn't a Navigation Catalog

            #Validate object links
            valid_objects=[]
            for url in selected_unique_urls:
                obj = self.getObjectFromRelativePath(url, None)
                if obj is not None:
                    valid_objects.append(obj)
                    valid.append(obj.virtualURL)
                else:
                    invalid.append(url)
            #Remove already existing (To reorder them)
            for o in valid_objects:
                if o.hasMenu(navigation_menu_path):
                    o.deleteFromMenu(navigation_menu_path)
            #Add sorted to the menu
            order = getattr(self, self.navigation_catalog).getHighestCountForMenu(navigation_menu_path)
            for obj in valid_objects:
                obj.addToMenu(navigation_menu_path, order)
                order = order + 1
            self.reindex_object()

        if REQUEST is not None:
            if valid!=[]:
                message="The documents [%s] has been added to menu [%s]" % (join(valid,', '),navigation_menu,)
            else:
                message="No valid urls was submitted."
            if invalid!=[]:
                message=message+"<br>The following urls was not valid documents: [%s]" % (join(invalid,', '),)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            invalid_urls=invalid,
                            management_view='Navigation')
        return valid, invalid

    security.declareProtected(EasyNavigationChange, 'manage_remove_from_menu')
    def manage_remove_from_menu(self,navigation_menu=None,selected_urls=[],all_urls=[],REQUEST=None):
        """Remove documents from the navigation menu"""
        if navigation_menu is None:
            navigation_menu_path = self.physicalPath
        else:
            navigation_menu_path = url2path(navigation_menu)
        valid = []
        for url in selected_urls:
            try:
                o = self.restrictedTraverse(url2path(url))
                o.deleteFromMenu(navigation_menu_path)
                valid.append(self.physicalURLToVirtualURL(url))
            except: pass
        self.reindex_object()
        if REQUEST is not None:
            message="The documents [%s] was removed from the menu" % (join(valid,', '),)
            return self.manage_navigatorForm(
                            self,
                            REQUEST,
                            manage_tabs_message=message,
                            management_view='Navigation')

    security.declareProtected(EasyNavigationChange, 'manage_navigation')
    def manage_navigation(self, navigation_title, title, REQUEST=None):
        """Updates the navigator object"""
        id = self.id
        if callable(id): id=id()
        if not id: id = 'root'
        self.title=title
        if navigation_title!='': self.navigation_title=navigation_title
        else: self.navigation_title=id
        self.reindex_object()
        if REQUEST:
            message="Content changed."
            return self.manage_navigatorForm(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Navigation')


    #Menu Manipulation Methods
    security.declarePrivate('changeOrder')
    def changeOrder(self, navigation_menu, order):
        #This function sets the order number for this object in the specified navigation_menu
        try: order=int(order)
        except: order=0
        nmd = self.navigation_menus_data
        if nmd.has_key(navigation_menu):
            navdata = nmd[navigation_menu]
            newnavdata = (order, ) + navdata[1:]
            nmd[navigation_menu] = newnavdata
            self.navigation_menus_data = nmd
        self.reindex_object()

    security.declareProtected(EasyNavigationChange, 'addToMenu')
    def addToMenu(self, navigation_menu, order):
        #ORDER must be required othervise the order can get corrupted!
        #order must be a proper order supplied by the called!
        # Adds this object to the menu of another object
        #try: order=int(order)
        #except: order=0
        nmd = self.navigation_menus_data
        if not nmd.has_key(navigation_menu):
            nmd[navigation_menu] = (order, )
            self.navigation_menus_data = nmd
        self.reindex_object()

    security.declareProtected(AccessContentsInformation, 'hasMenu')
    def hasMenu(self, navigation_menu):
        return self.navigation_menus_data.has_key(navigation_menu)

    security.declareProtected(EasyNavigationChange, 'deleteFromMenu')
    def deleteFromMenu(self, navigation_menu):
        # Removes this object to the menu of another object
        nmd = self.navigation_menus_data
        del nmd[navigation_menu]
        self.navigation_menus_data = nmd
        self.reindex_object()

    security.declarePrivate('hasSubDocuments')
    def hasSubDocuments(self, spec=None):
        if spec is None:
            spec = EASYEDITABLEMETATYPES
        return self.objectIds(spec=spec) != []

    security.declarePrivate('showMenuPathOrName')
    def showMenuPathOrName(self,menu):
        """Returns the menu name either as a string or as a url (string)"""
        if type(menu)==type(()) or type(menu)==type(()):
            return join(menu,'/')
        return menu

    security.declarePrivate('isMenuPathOrName')
    def isMenuPathOrName(self,menu):
        """Returns the menu name either as a string or as a url (string)"""
        if type(menu)==type(()) or type(menu)==type(()):
            return 1
        return 0

    security.declarePrivate('navigation_menus')
    def navigation_menus(self):
        return self.navigation_menus_data.keys()

    security.declareProtected(EasyNavigationServiceAdd, 'manage_makeVirtualRoot')
    def manage_makeVirtualRoot(self,REQUEST=None):
        """Turn an object to a Virtual Root"""
        if hasattr(self,'aq_parent') and hasattr(self.aq_parent,'getPhysicalPath'):
            parentpath = self.aq_parent.getPhysicalPath()
        else:
            parentpath = 'root'
            self._isEasyRoot = 1
        if hasattr(self, self.navigation_catalog):
          highcount = getattr(self, self.navigation_catalog).getHighestCountForMenu(parentpath)
        else:
          highcount = 0
        self.navigation_menus_data =  { parentpath: (highcount + 1,) }
        self.index_object()
        if REQUEST:
            message="Document has been made a Virtual Root."
            return self.manage_navigatorForm(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Navigation')


    # Remote Dokument Navigation

    security.declareProtected(AccessContentsInformation, 'getMenuItems')
    def getMenuItems(self):
        tool = aq_get(self, NAVIGATION_CATALOG_ID, None, 1)
        if tool is None: return []
        physical_path=self.physicalPath
        all_menu_items = tool.searchResults(navigation_menus=[physical_path])
        #Post sort
        all_menu_items =list(all_menu_items)
        def sortmethod( x,y, nav_menu):
            return cmp(x.navigation_menus_data[nav_menu][0],
                       y.navigation_menus_data[nav_menu][0])
        all_menu_items.sort( lambda x,y, nav_menu=physical_path,
                         sortmethod=sortmethod: sortmethod(x,y,nav_menu))
        return all_menu_items

    security.declareProtected(AccessContentsInformation, 'getSubObjects')
    def getSubObjects(self):
        return map(lambda ob: (ob.physicalURL, ob.navigation_title + ' -- ' + ob.id), self.objectValues(spec=EASYEDITABLEMETATYPES))

    security.declareProtected(AccessContentsInformation, 'getMenuParent')
    def getMenuParent(self):
        return self

    security.declareProtected(AccessContentsInformation, 'getMenuParentUrl')
    def getMenuParentUrl(self):
        return self.physicalURL

    security.declareProtected(EasyNavigationChange, 'manage_hard_remove')
    def manage_hard_remove(self, url=None, REQUEST=None):
        """ """
        removed='Nothing have happend.'
        if url is not None:
            path=self.path2url(url)
            if path in self.navigation_menus_data.keys():
                self.deleteFromMenu(path)
                removed='%s has been removed' % (str(path))
            else:
                path=self.virtualURLToPhysicalPath(url)
                if path in self.navigation_menus_data.keys():
                    self.deleteFromMenu(path)
                    removed='%s has been removed' % (str(path))
        nmd=join( map( lambda (x, y): str(x)+": "+str(y), self.navigation_menus_data.items() ), '<br>' )
        return "<h3>URL: %s</h3><p>%s</p>" % (removed,nmd)

    render_investigate_menus = DTMLFile('dtml/investigate_menus/render_investigate_menus', globals())

    security.declareProtected(EasyNavigationChange, 'manage_investigate_menus')
    def manage_investigate_menus(self, menus=None,REQUEST=None):
        """\
        Investigate menus relations.
        """
        if not menus:
            menu_objs={'Here':self}
        else:
            menu_objs={}
            for url in menus:
                try:
                    ob=self.unrestrictedTraverse(url)
                    if hasattr(aq_base(ob), 'getMenuItems'):
                        menu_objs[url]= ('',ob)
                except: pass
        menu_obj_links={}
        for url, obj in menu_objs.items():
            local_menu={}
            for each in obj.getMenuItems():
                # Store a reference to the objects that are in this menu, and their order.
                try:
                    ob = self.unrestrictedTraverse(each.physicalURL)
                    navigation_menus_data = ob.navigation_menus_data
                    local_menu[each.physicalURL]=('',ob.navigation_menus_data)
                except:
                    local_menu[each.physicalURL]=('Failed to look up: %s'%(each.physicalURL,),None)
            menu_obj_links[url]=local_menu.copy()
        return self.render_investigate_menus(self, REQUEST
                            , menu_objs=menu_objs.items()
                            ,menu_obj_links=menu_obj_links.items()
                        )


    security.declareProtected(EasyNavigationChange, 'manage_menus_data')
    def manage_menus_data(self,REQUEST=None):
        """Return the navigation_menus_data for debugging."""
        if REQUEST:
            return "<h1>navigation_menus_data!</h1><p>%s</p>" % (self.navigation_menus_data,)

    # temporary fix-it-up method
    security.declareProtected(EasyNavigationChange, 'manage_do_reindex')
    def manage_do_reindex(self,REQUEST=None):
        """Reindex the document."""
        self.reindex_object()
        if REQUEST:
            return "<h1>Reindexed!</h1>"



Globals.InitializeClass(Navigatable)
InitializeBodyTabs(Navigatable)



