# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyNotification/EasyNotifiable.py,v 1.5 2003/07/16 13:23:32 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__='''EasyNotifiable product: EasyNotifiable mixin class.'''
__version__='$Revision: 1.5 $'[11:-2]


import Globals
from Globals import DTMLFile
from string import join
from AccessControl import ClassSecurityInfo
from Products.EasyCore.EasyCorePermissions import *

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyNotification.EasyNotification import EASYNOTIFICATIONID
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs


from Products.EasyCore import MLOG_TAB

class EasyNotifiable(EasyPresentationHelpers, BodyTabs):
    """EasyNotifiable management interface"""

    manage_options =()

    security = ClassSecurityInfo()

    security.declareProtected(EasyNotificationManage, 'manage_notifications_form')
    manage_notifications_form = DTMLFile('dtml/manage_notifications_form',globals()
             , management_view='Publish'
             , body_tab={'label':'Notifications','order':3, 'management_view':'Publish'}
             )


    ##SEC_CHANGE: All notifiables management view are now protected by the EasyNotificationManage
    ##            permission instead of the EasyCorePermissions.EasyWorkflowManage.
    security.declareProtected(EasyNotificationManage, 'manage_add_notification_users')
    def manage_add_notification_users(self, addusers=[], REQUEST=None):
        """
        Add user(s) to the local notification list if it exists.
        """
        notlist = getattr(self.aq_base, 'notification_list', None)
        if notlist is None:
            if REQUEST is not None:
                return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message="Can't add users to an acquired or empty notification list.",
                    management_view='Notifications')
            return None
        if addusers == []:
            if REQUEST is not None:
                return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message="There wasn't any users selected..",
                    management_view='Notifications')
            return None
        for user in addusers:
            if not notlist.has_key(user):
                notlist[user] = ()
        self.notification_list = notlist
        message="User(s) [%s] has been added to the Local Notification list." % (join(addusers,", "),)
        if REQUEST is not None:
            return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Notifications')

    security.declareProtected(EasyNotificationManage, 'manage_create_notification_list')
    def manage_create_notification_list(self, REQUEST=None):
        """
        Creates a notification list from the acqusition if it exists
        or a empty acqusition list will be created.
        """
        if hasattr(self.aq_base, 'notification_list'):
            message = "Notification list already exists."
        else:
            notlist = getattr(self, 'notification_list', None)
            if notlist is not None:
                self.notification_list = notlist.copy()
                message = "A Local Notification List created from the acquiered Notification List."
            else:
                self.notification_list= {}
                message = "An empty Local Notification List was created."
        if REQUEST is not None:
            return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Notifications')


    security.declareProtected(EasyNotificationManage, 'manage_remove_notification_list')
    def manage_remove_notification_list(self, REQUEST=None):
        """
        Remove the local notification list.
        """
        if not hasattr(self.aq_base, 'notification_list'):
            message = "A Local Notification list doesn't exist."
        else:
            del self.notification_list
            message = "The Local Notification list has been removed exists."
        if REQUEST is not None:
            return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Notifications')

    security.declareProtected(EasyNotificationManage, 'manage_remove_notification_users')
    def manage_remove_notification_users(self, checkusers=[], REQUEST=None):
        """
        Remove user(s) from the local notification list if it exists.
        """
        notlist = getattr(self.aq_base, 'notification_list', None)
        if notlist is None:
            if REQUEST is not None:
                return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message="Can't remove users from an acquired or empty notification list.",
                    management_view='Notifications')
            return None
        if checkusers == []:
            if REQUEST is not None:
                return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message="There wasn't any users selected..",
                    management_view='Notifications')
            return None
        for user in checkusers:
            if notlist.has_key(user):
                del notlist[user]
        self.notification_list = notlist
        message="User(s) [%s] has been removed from the Local Notification list." % (join(checkusers,", "),)
        if REQUEST is not None:
            return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Notifications')

    security.declareProtected(EasyNotificationManage, 'manage_change_notification')
    def manage_change_notification(self, users=None, REQUEST=None):
        """
        Updates the local notification list if it exists.
        """
        notlist = getattr(self.aq_base, 'notification_list', None)
        if notlist is None:
            if REQUEST is not None:
                return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message="Can't change an acquired or empty notification list.",
                    management_view='Notifications')
            return None
        for user in users:
            if hasattr(user, 'notify_on'):
                notlist[user.id] = user.notify_on
            else:
                notlist[user.id] = ()
        self.notification_list = notlist
        message="Notification settings has been changed."
        if REQUEST is not None:
            return self.manage_notifications_form(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Notifications')

    security.declarePrivate('getNotificationList')
    def getNotificationList(self, REQUEST=None):
        notlist = getattr(self, 'notification_list', {})
        if type(notlist) == type([]):
            new_notlist =  {}
            for key, value in notlist:
                new_notlist[key] = value
            self.notification_list = new_notlist
            return new_notlist
        return notlist

    security.declarePrivate('getAcquiredNotificationList')
    def getAcquiredNotificationList(self, REQUEST=None):
        if hasattr(self, 'aq_parent'):
            return getattr(self.aq_parent, 'notification_list', {})

    security.declarePrivate('isNotificationListAcquired')
    def isNotificationListAcquired(self, REQUEST=None):
        if hasattr(self.aq_base, 'notification_list'):
            return 0
        if not hasattr(self, 'aq_parent') or not hasattr(self.aq_parent, 'notification_list'):
            #If the isn't a local not-list and it can't be acquired, create a local one.
            self.notification_list={}
            return 0
        return 1

    security.declarePrivate('hasAcquirableNotificationList')
    def hasAcquirableNotificationList(self, REQUEST=None):
        return hasattr(self, 'aq_parent') and hasattr(self.aq_parent, 'notification_list')

    security.declarePrivate('notifyOnWorkflowChange')
    def getUserIdToNotify(self, notification):
        notlist = getattr(self, 'notification_list', {})
        send_notification_to = []
        for notify_user, notification_settings in notlist.items():
            if notification in notification_settings:
                send_notification_to.append(notify_user)
        return send_notification_to


    security.declarePrivate('manage_notify')
    def manage_notify(self, notification, fromuser, tousers,  namespace):
        notify_service = getattr(self, EASYNOTIFICATIONID, None)
        if notify_service is not None:
            notify_service.manage_notify(notification, fromuser, tousers,  namespace, self)

    security.declarePrivate('getNotifiableWorkflowTransitionItems')
    def getNotifiableWorkflowTransitionItems(self):
        notify_service = getattr(self, EASYNOTIFICATIONID, None)
        if notify_service is not None:
            return notify_service.getNotifiableWorkflowTransitionItemsFor(self)
        return ()


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base=aq_base(self)
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyNotifiable)
InitializeBodyTabs(EasyNotifiable)


def initialize(context):
    """Initialize StyleManagers Product"""

    pass # register Publishable as a BaseClass

