# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyPresentationHelpers/DTMLHTMLTags.py,v 1.1 2003/06/10 09:50:18 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''DTMLHTMLTags
$Id: DTMLHTMLTags.py,v 1.1 2003/06/10 09:50:18 johanc Exp $'''
__version__='$Revision: 1.1 $'[11:-2]

import string

from DocumentTemplate.DT_Util   import *
from DocumentTemplate.DT_String import String
from DocumentTemplate.DT_Util import render_blocks

from Products.EasyCore.TagUtils import ContainerTagBase, SingleTagBase
from Products.EasyCore.TagUtils import KEEP_SPACES, REMOVE_SPACES, INTEND_HTML
from Products.EasyCore.TagUtils import  STRING_PARAM, CONDITIONAL_PARAM

from Products.EasyCore import MLOG_TAB

_marker=[]


def HTMLTagFactory(tag, base_class=ContainerTagBase, attributes={}, space_policy=REMOVE_SPACES):

    class NewClass(base_class):  pass
    NewClass.name=tag
    NewClass.attributes=attributes
    NewClass.space_policy=space_policy
    String.commands[tag]=NewClass


html_tags=(
    ('table', ContainerTagBase, {}, REMOVE_SPACES),
    ('tr', ContainerTagBase, {}, REMOVE_SPACES),
    ('td', ContainerTagBase, {}, REMOVE_SPACES),
    ('th', ContainerTagBase, {}, REMOVE_SPACES),
    ('tbody', ContainerTagBase, {}, REMOVE_SPACES),
    ('thead', ContainerTagBase, {}, REMOVE_SPACES),

    ('h1', ContainerTagBase, {}, REMOVE_SPACES),
    ('h2', ContainerTagBase, {}, REMOVE_SPACES),
    ('h3', ContainerTagBase, {}, REMOVE_SPACES),
    ('h4', ContainerTagBase, {}, REMOVE_SPACES),
    ('h5', ContainerTagBase, {}, REMOVE_SPACES),
    ('h6', ContainerTagBase, {}, REMOVE_SPACES),
    ('h7', ContainerTagBase, {}, REMOVE_SPACES),
    ('p', ContainerTagBase, {}, REMOVE_SPACES),
    ('span', ContainerTagBase, {}, REMOVE_SPACES),
    ('div', ContainerTagBase, {}, REMOVE_SPACES),
    ('pre', ContainerTagBase, {}, KEEP_SPACES),

    ('ol', ContainerTagBase, {}, REMOVE_SPACES),
    ('ul', ContainerTagBase, {}, REMOVE_SPACES),
    ('li', ContainerTagBase, {}, REMOVE_SPACES),

    ('form', ContainerTagBase, {}, REMOVE_SPACES),

    ('select', ContainerTagBase, {}, REMOVE_SPACES),
    ('option', ContainerTagBase, {}, REMOVE_SPACES),

    ('textarea', ContainerTagBase, {}, REMOVE_SPACES),
    ('button', ContainerTagBase, {}, REMOVE_SPACES),

    ('input', SingleTagBase, {}, REMOVE_SPACES),

    ('a', ContainerTagBase, {}, REMOVE_SPACES),

    ('br', SingleTagBase, {}, REMOVE_SPACES),

)

for tag, base, attributes, space_policy in html_tags:
    HTMLTagFactory(tag, base, attributes, space_policy)



class DTMLHidden(SingleTagBase):
    """\
    DTMLHidden Tag
    """
    name='hidden'

    attributes={
      'id':     {'id':'id',     'value':'',               'required':1,'type':STRING_PARAM},
      'value':  {'id':'value',  'value':'', 'default':'', 'required':0,'type':STRING_PARAM},
    }

    template="""<input type="hidden" id="%(id)s" name="%(id)s:default" value="%(value)s"/>"""

    def do_render(self, md, object):
        return self.template%{'id':self.id, 'value':self.value}


String.commands[DTMLHidden.name] = DTMLHidden


class DTMLCheckbox(SingleTagBase):
    """\
    DTMLCheckbox Tag
    """
    name='checkbox'

    attributes={
      'id':     {'id':'id',     'value':'no_name',               'required':1, 'type':STRING_PARAM},
      'value':  {'id':'value',  'value':'', 'default':'', 'required':1, 'type':STRING_PARAM},
      'active': {'id':'active', 'value':'', 'default':'', 'required':1, 'type':CONDITIONAL_PARAM},
      'default':{'id':'default',  'value':'', 'default':'', 'required':0, 'type':STRING_PARAM},
    }

    template="""<input type="hidden" id="%(id)s" name="%(id)s:default" value="%(default)s"/>"""
    template=template+"""<input type="checkbox" id="%(id)s" name="%(id)s" value="%(value)s" %(checked)s/>"""

    def do_render(self, md, object):
        active=getattr(self, 'active', _marker)
        checked = active and 'checked="1"' or ''
        return self.template%{'id':self.id, 'value':self.value, 'checked':checked, 'default':self.default}


String.commands[DTMLCheckbox.name] = DTMLCheckbox



