# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasySearchService/EasySearchService.py,v 1.12 2003/06/18 09:22:32 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasySearchService product: EasySearchService class.'''
__version__='$Revision: 1.12 $'[11:-2]

import Globals
from Globals import DTMLFile, HTML
import DateTime
from string import join, lower
from types import StringType, TupleType, IntType
from urllib import quote
from zLOG import LOG, INFO, DEBUG

from OFS.Folder import Folder

from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo

from ZTUtils.Batch import Batch

from Products.EasyPresentationHelpers.EasyParameterTracker import EasyParameterTracker
from Products.EasyCore.EasyCorePermissions import EasyViewAhead, EasySearchServiceAdd, View, AccessContentsInformation
from Products.EasyCore.EasySearchServiceConf import SEARCH_SERVICE_ID
from Products.EasyWorkflow.EasyWorkflowBase import wfPUBLISHED
from Products.EasyCore.Utils.parse_parameters import fields2parameters, parameters2fields



class EasySearchService(Folder, EasyParameterTracker):
    """ EasySearchService object
        The EasySearchService helps views that supply search features.
    """
    meta_type = SEARCH_SERVICE_ID


    security = ClassSecurityInfo()

    track_fields = (
                        {'id':'query','type':'string','default':'', },
                        {'id':'query_start','type':'string','default':'1', },
                        {'id':'batch_size','type':'string','default':'10', },
                        {'id':'max_batch_size','type':'int','default':'100', },
                        {'id':'searchtext','type':'string','default':'', },
                        {'id':'overlap','type':'string','default':'0', },
                        {'id':'orphan','type':'string','default':'0', },
                        {'id':'sorton','type':'string','default':'', },
                        {'id':'sortorder','type':'string','default':'', },
                        {'id':'pre_sorton','type':'string','default':'', },
                        {'id':'pre_sortorder','type':'string','default':'', },
                        {'id':'advanced_search','type':'int','default':1, },
                        {'id':'search_for_categories','type':'string','default':'', },
                        {'id':'search_for_subjects','type':'tokens','default':(), },
                    )

    manage_options_left = Folder.manage_options[:1]
    manage_options_right = Folder.manage_options[1:]

    manage_options = manage_options_left + \
                     (
                         {'label':'Configure', 'action':'manage_search_service'},
                     ) + \
                     manage_options_right

    _properties=(
            {'id':'title', 'type': 'string', 'mode':'w'},
            )

    def __init__(self, id, title='', catalog=None):
        self.id = id
        self.title = title
        if catalog is not None:
            self.catalog = catalog
        else:
            self.catalog = 'EasyNavigation'

    security.declarePrivate('_manage_edit_firm')
    _manage_edit_firm = DTMLFile('dtml/manage_EasySearchServiceEditForm', globals())

    security.declareProtected(EasySearchServiceAdd, 'manage_search_service')
    def manage_search_service(self, title='', submit='', track_fields=[], catalog=None, REQUEST=None):
        """Save changes for the EasySearchService"""
        if submit==" Save Changes ":
            self.title = title
            self.track_fields, self.error_fields = fields2parameters(track_fields)
            if catalog is not None:
                self.catalog = catalog

        if REQUEST is not None:
            manage_tabs_message = ""
            management_view = "Edit"
            return self._manage_edit_firm(self,REQUEST,manage_tabs_message=manage_tabs_message,management_view=management_view)

    # Site Search API
    security.declareProtected(View, 'initSearch')
    def initSearch(self, **kw):
        apply(EasyParameterTracker.initTracking, (self,), kw)
        request = getattr(self, 'REQUEST', None)
        if request and request.has_key('batch_size') and not request.has_key('batch_size_int'):
            try:
                batch_size_int = int(request.get('batch_size',0))
            except:
                batch_size_int = 0
            max_batch_size=request.get('max_batch_size',None)
            if type(max_batch_size) is IntType and batch_size_int>max_batch_size:
                batch_size_int=max_batch_size
            request['batch_size_int'] = batch_size_int
            #The batch_size and batch_size_int will differ.


    security.declareProtected(View, 'getSearchResult')
    def getSearchResult(self, query=None, physicalPathIndex=None):
        REQUEST = getattr(self,'REQUEST', None)
        catalog = getattr(self, self.catalog)
        if catalog is not None and REQUEST is not None:
            if query is None: query={}

            if physicalPathIndex and type(physicalPathIndex) == StringType:
                query['physicalPathIndex'] = {'query':physicalPathIndex }

            if not getSecurityManager().checkPermission(EasyViewAhead, self) or hasattr(self,'REQUEST') and hasattr(self.REQUEST,'anonymous'):
                now = DateTime.now()
                query['getWorkflowState'] = {'query':wfPUBLISHED }
                query['effective_date'] = {'query': now, 'range': 'max'}
                query['expiration_date'] = {'query': now, 'range': 'min'}

            if REQUEST.has_key('meta_type'): query['meta_type'] = {'query': REQUEST['meta_type']}

            if REQUEST.has_key('advanced_search') and REQUEST['advanced_search']:
                if REQUEST.has_key('search_for_categories') and REQUEST['search_for_categories']:
                    cats = filter(None, REQUEST['search_for_categories'])
                    if len(cats) > 0:
                        query['category'] = {'query': REQUEST['search_for_categories']}

                if REQUEST.has_key('search_for_subjects') and REQUEST['search_for_subjects']:
                    query['subject'] = {'query': REQUEST['search_for_subjects']}

            if REQUEST.has_key('searchtext') and REQUEST['searchtext']!='':
                if REQUEST['searchtext']=='*':
                    query['PrincipiaSearchSource'] = {'query': ''}
                else:
                    query['PrincipiaSearchSource'] = {'query': REQUEST['searchtext']}
            else:
                REQUEST.set('previous',None)
                REQUEST.set('next',None)
                REQUEST.set('search_query', query)
                REQUEST.set('search_batch_first', 0)
                REQUEST.set('search_batch_last', 0)
                REQUEST.set('search_batch_size', 0)
                REQUEST.set('search_batches', ())#Change from None
                REQUEST.set('search_result', ()) #Change from None
                REQUEST.set('search_length', 0)
                return []

            if REQUEST.has_key('extra_query'):
                query.update( REQUEST['extra_query'])

            pre_sorton = REQUEST.get('pre_sorton',None)
            pre_sortorder = REQUEST.get('pre_sortorder',None)
            if pre_sorton:
                query['sort_on']=pre_sorton
                if pre_sortorder is not None:
                    query['sort_order']=pre_sortorder

            # Do Search
            search_result = catalog(query)

            if not getSecurityManager().checkPermission(EasyViewAhead, self) or hasattr(self,'REQUEST') and hasattr(self.REQUEST,'anonymous'):
                # Ugly hack, merging two queries, since Zope 2.5.1 sorts empty strings as higher than numbers
                query['effective_date'] = {'query': ''}
                search_result = search_result + catalog(query)

            search_length = len(search_result)

            query_start = REQUEST.get('query_start', 1)
            try: query_start=int(query_start)
            except: query_start=1

            batch_size = REQUEST.get('batch_size', 10)
            try: batch_size=int(batch_size)
            except: batch_size=search_length
            max_batch_size=REQUEST.get('max_batch_size',None)
            if type(max_batch_size) is IntType and batch_size>max_batch_size:
                batch_size=max_batch_size
                REQUEST['batch_size_int']=max_batch_size
                #The batch_size and batch_size_int will differ.

            #Post preparations
            search_batches = []
            if batch_size>0 and batch_size<search_length:
                previous = query_start - batch_size
                previous_size = batch_size
                if previous<1: previous = None

                next = query_start + batch_size
                next_size = batch_size
                if next>search_length: next = None


                rest = search_length%batch_size
                number_of_batches = search_length/batch_size + 1
                for i in range(0,number_of_batches-1):
                    start_index = i * batch_size + 1
                    stop_index = i * batch_size + batch_size
                    search_batches.append(
                        {
                            'batch_number': i+1,
                            'start_index': start_index,
                            'stop_index': stop_index,
                            'is_in_batch': (query_start>=start_index and query_start<=stop_index),
                        })
                if rest!=0:
                    start_index = (number_of_batches - 1) * batch_size + 1
                    stop_index = (number_of_batches - 1) * batch_size + rest
                    search_batches.append(
                        {
                            'batch_number': number_of_batches,
                            'start_index': start_index,
                            'stop_index': stop_index,
                            'is_in_batch': (query_start>=start_index and query_start<=stop_index),
                        })

                batch_first=max(query_start-1,0)
                batch_first=max(batch_first-batch_first%batch_size,0)
                batch_last=max(batch_first+batch_size-1,0)
                #Batch(sequence, size, start=0, end=0,orphan=0, overlap=0)
                search_result=Batch(search_result, batch_size, start=batch_first)
                batch_size=len(search_result)

            else:
                previous = None
                previous_size=0
                next = None
                next_size=0
                batch_first=0
                batch_last=search_length-1
                batch_size=search_length



            REQUEST.set('previous',previous)
            REQUEST.set('previous_size',previous_size)
            REQUEST.set('next',next)
            REQUEST.set('next_size',next_size)
            REQUEST.set('search_query',query)
            REQUEST.set('search_batch_first', batch_first)
            REQUEST.set('search_batch_last',  batch_last)
            REQUEST.set('search_batch_size',  batch_size)
            REQUEST.set('search_batches', search_batches)
            REQUEST.set('search_result',  search_result)
            REQUEST.set('search_length',  search_length)
            return search_result

    #Merge reversable_sort_url_arguments and sort_url_arguments to handle both reverasable and
    #non reversable results.
    security.declareProtected(AccessContentsInformation, 'reversable_sort_url_arguments')
    def reversable_sort_url_arguments(self,name):
        REQUEST = self.REQUEST
        pre_sorton = REQUEST.get('pre_sorton',None)
        pre_sortorder = REQUEST.get('pre_sortorder',None)
        if pre_sorton==name:
            if pre_sortorder and lower(pre_sortorder) in ['reverse','descendent']:
                pre_sortorder = ''
            else: pre_sortorder = 'reverse'
        return """pre_sorton=%s&pre_sortorder=%s""" % (name,pre_sortorder)

    security.declareProtected(AccessContentsInformation, 'sort_url_arguments')
    def sort_url_arguments(self,name):
        return """sorton=%s""" % (name,)

    security.declareProtected(AccessContentsInformation, 'is_sorted_on')
    def is_sorted_on(self,name):
        REQUEST = self.REQUEST
        return (REQUEST.has_key('pre_sorton') and REQUEST['pre_sorton']==name) or (REQUEST.has_key('sorton') and REQUEST['sorton'] ==name)

    security.declareProtected(AccessContentsInformation, 'is_sorted_in_reverse')
    def is_sorted_in_reverse(self,name):
        REQUEST = self.REQUEST
        if not REQUEST.get('pre_sortorder',None): return 0
        return lower(REQUEST.get('pre_sortorder','')) in ('reverse','descendent')

    security.declareProtected(AccessContentsInformation, 'getPossibleItemsFor')
    def getPossibleItemsFor(self, name, additional_items=(), additional_values=(), exclude_values=(), cap=0, capword=0, underscore2space=0):
        catalog = getattr(self, self.catalog)
        pos_items = ()
        if catalog is not None:
            pos_values = tuple(catalog.uniqueValuesFor(name))
            if exclude_values!=():
                pos_values = filter(lambda key, evs=exclude_values: key not in evs, pos_values)
            if type(additional_values) == TupleType:
                pos_values = pos_values + additional_values
            pos_items = self.values2items(pos_values, cap=cap, capword=capword, underscore2space=underscore2space)
            if type(additional_items) == TupleType:
                pos_items = pos_items + additional_items
        return pos_items



Globals.InitializeClass(EasySearchService)

manage_EasySearchServiceAddForm = DTMLFile('dtml/manage_EasySearchServiceAddForm', globals())

def manage_EasySearchServiceAdd(self, id, title='', submit='', REQUEST=None):
    """Add a EasySearchService object"""

    c=EasySearchService(id, title)
    self._setObject(id, c)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_workspace')


def initialize(context):
    """Initialize EasySearchService Product"""

    context.registerClass(
        EasySearchService,
        permission=EasySearchServiceAdd,
        icon='icons/EasySearchService.gif',
        constructors = ( manage_EasySearchServiceAddForm, manage_EasySearchServiceAdd),
        meta_type = SEARCH_SERVICE_ID
    )


