# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyVHM/EasyVHM.py,v 1.7 2004/01/14 14:38:56 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyVHM module

Defines the EasyVHM class

See CHANGES.TXT for Credits.

"""
__version__='$Revision: 1.7 $'[11:-2]

import Globals
from Globals import DTMLFile, Persistent
from OFS.SimpleItem import Item
from Acquisition import Implicit, ImplicitAcquisitionWrapper, aq_base, aq_parent
from ExtensionClass import Base
from string import split, strip, join, find, lower, replace
from ZPublisher import BeforeTraverse
import os

from AccessControl import ClassSecurityInfo
try:
    from Products.EasyCore.EasyCorePermissions import EasyVHMManage
except:
    EasyVHMManage='Easy VHM Manage'

class EasyVHM(Persistent, Item, Implicit):
    """Provide a simple drop-in solution for virtual hosting.
    """
    meta_type='EasyVHM'
    priority = 25
    title = ''
    isPrincipiaFolderish=1

    default_port=80
    domain_path = {}
    path_domain = {}
    domain_port = {}

    manage_options=({'label':'Edit', 'action':'manage_main'},)

    security = ClassSecurityInfo()

    def __init__(self, default_port=80):
        #EasyVHM.inheritedAttribute('__init__')(self)
        #TODO: Calculate default domain name
        self.default_port=default_port
        self.domain_path = {}
        self.path_domain = {}
        self.domain_port = {}
        self.object_version = '1.8'

    security.declareProtected(EasyVHMManage, 'manage_main')
    _manage_main = DTMLFile('dtml/manage_main', globals())
    def manage_main(self, records=None, default_port=None, REQUEST=None):
        """Saves changes to mapping and returns _manage_edit with status messages.

        Sets domain to path mappings. Returns a status message.
        """
        if default_port:
            try:
                default_port = int(default_port)
                self.default_port = default_port
            except:
                pass

        if records is None:
            manage_tabs_message = ''
        else:
            manage_tabs_message = "Mappings updated"
            new_domain_path = {}
            new_domain_port = {}
            new_path_domain = {}

            for dp in records:

                if len(dp.path) > 0  and dp.path[0] != '/':
                    dp.path = '/' + dp.path

                if dp.domain != '' and dp.path != '': #Not an empty mapping
                    if dp.port == '': dp.port = self.default_port

                    new_domain_path[dp.domain] = dp.path
                    new_domain_port[dp.domain] = dp.port
                    if not new_path_domain.has_key(dp.path):
                        new_path_domain[dp.path] = dp.domain

            self.domain_path = new_domain_path
            self.domain_port = new_domain_port
            self.path_domain = new_path_domain

        self.message = {}
        for domain in self.domain_path.keys():
            msg = self.check_path(domain, self.domain_path[domain])
            if msg is None: msg = ''
            self.message[domain] = msg


        if REQUEST is not None:
            return self._manage_main(client=self,manage_tabs_message=manage_tabs_message,REQUEST=REQUEST)


    def check_path(self,domain,path):
        if not path:
           return "Path is empty."
        if path[-1] == '/':
           if len(path) == 1:
              return None
           else:
              path = path[:-1]
        try:
            leaf_object = self.unrestrictedTraverse(path)
        except:
            return "The path %s was not found." % path

        if not getattr(aq_base(leaf_object),'isAnObjectManager', None):
            return "The path, %s , must end in a container: %s isn't one." % ( path, split(path,'/')[-1] )

        self._p_changed = 1
        return None

    def display_rules(self):
        """Returns list of tuples of access rule domains and paths"""
        retlist = []
        for domain in self.domain_path.keys():
            retlist.append({'domain': domain, 'port': self.domain_port[domain], 'path': self.domain_path[domain], 'message': self.message[domain]})
        return retlist

    #Returns a domain for a path
    def get_site_root_path(self, path):

        if path==('',) or self.path_domain.has_key(join(path,'/')):
            return path

        urls = self.path_domain.keys()
        index=2 #Always compare at least the first two entries (The first is always '' and therefore matches).
        first = 1 #This is instead of a do-while. Makes the loop run at least once.
        while first or len(urls)>1:
            first = 0
            new_urls = []
            for url in urls:
                p = tuple(split(url,'/'))
                if p[:index]==path[:index]:
                    new_urls.append(url)
            urls=new_urls
            index=index+1

        if len(urls):
            return tuple(split(urls[0], '/'))
        return ('',)

    def get_virtual_path(self, p_path):
        r_path = self.get_site_root_path(p_path)
        return ('',) + p_path[len(r_path):]

    def get_site_root_url(self, path, include_protocol=0):
        domain = self.get_domain(path)
        default_port = ''
        if self.default_port!=80:
            default_port = ':' + str(self.default_port)
        protocol =''
        if include_protocol or self.default_port!=80:
            protocol = self.getCurrentProtocol() + '://'
        return protocol + domain + default_port

    def getCurrentProtocol(self):
        req=getattr(self,'REQUEST',None)
        if req:
            https=req.get('HTTPS',None)
            if (https and https.lower()=='on') or (req.has_key('SERVER_PORT_SECURE') and req.get['SERVER_PORT_SECURE'] == "1"):
                return 'https'
        return 'http'

    def get_domain(self, path):
        while len(path)>1:
            if self.path_domain.has_key(join(path,'/')):
                return self.path_domain.get(join(path,'/'))
            path = path[:-1]

        if path==('',) and self.path_domain.has_key('/'):
            return self.path_domain.get('/')

        return 'localhost'

    def get_root_rule(self, path):
        domain = self.get_domain(path)
        return (domain, self.domain_path.get(domain,'/'))

    def split_physical_path(self, p_path):
        v_root_path = self.get_site_root_path(p_path)
        v_path = ('',) + p_path[len(v_root_path):]
        return v_root_path, v_path

    def is_root(self, p_path):
        v_root_path = self.get_site_root_path(p_path)
        return v_root_path==p_path

    #Returns a domain for a path
    def get_path(self, domain):
        return self.path_domain.get(lower(domain), None)


    def addToContainer(self, container):
        container._setObject(self.id, self)
        self.manage_afterAdd(self, container)

    def manage_beforeDelete(self, item, container):
        if item is self:
            BeforeTraverse.unregisterBeforeTraverse(container, self.meta_type)

    def manage_afterAdd(self, item, container):
        if item is self:
            id = self.id
            if callable(id): id = id()

            # We want the original object, not stuff in between
            container = container.this()
            hook = BeforeTraverse.NameCaller(id)
            BeforeTraverse.registerBeforeTraverse(container, hook,
                                                  self.meta_type,
                                                  self.priority)
    def _setId(self, id):
        id = str(id)
        if id != self.id:
            BeforeTraverse.unregisterBeforeTraverse(container,
            self.meta_type)
            hook = BeforeTraverse.NameCaller(id)
            BeforeTraverse.registerBeforeTraverse(container, hook,
                                                  self.meta_type,
                                                  self.priority)

    def __call__(self, client, request, response=None):
        '''Traversing at home'''
        stack = request['TraversalRequestNameStack']
        if stack and stack[-1] == 'VirtualHostBase':
            stack.pop()
            protocol = stack.pop()
            host = stack.pop()
            if ':' in host:
                host, port = split(host, ':')
                request.setServerURL(protocol, host, port)
            else:
                request.setServerURL(protocol, host)
        else:
            host = split(request['HTTP_HOST'], ':')[0]
            if getattr(self, 'domain_path', None):
                real_host=None
                path=None
                l_host = lower(host)

                if self.domain_path.has_key(l_host):
                    real_host=l_host
                    path = self.domain_path[real_host]
                else:
                    for hostname in self.domain_path.keys():
                        if hostname[:2] == '*.':
                            if find(l_host, hostname[2:]) != -1:
                                real_host = hostname
                                path = self.domain_path[real_host]

                if real_host and path:
                    if path != '/':
                        list_path = split(path, '/')[1:]
                        stack.append('/')
                        stack.append(self.id)
                        for i in range(len(list_path)):
                            stack.append(list_path.pop())
                    request.setServerURL('http', real_host, str(self.domain_port[real_host]))
        # Find and convert VirtualHostRoot directive
        # If it is followed by one or more path elements that each
        # start with '_vh_', use them to construct the path to the
        # virtual root.
        vh = -1
        for ii in range(len(stack)):
            if stack[ii] == 'VirtualHostRoot':
                if vh >= 0:
                    pp = ['']
                    for jj in range(vh, ii):
                        pp.insert(1, stack[jj][4:])
                    stack[vh:ii + 1] = [join(pp, '/'), self.id]
                elif ii > 0 and stack[ii - 1][:1] == '/':
                    stack[ii] = self.id
                else:
                    stack[ii] = self.id
                    stack.insert(ii, '/')
                break
            elif vh < 0 and stack[ii][:4] == '_vh_':
                vh = ii


    def __bobo_traverse__(self, request, name):
        '''Traversing away'''
        if name[:1] != '/':
            return getattr(self, name)
        parents = request.PARENTS
        parents.pop() # I don't belong there

        if len(name) > 1:
            request.setVirtualRoot(split(name[1:], '/'))
        else:
            request.setVirtualRoot([])
        return parents.pop() # He'll get put back on


Globals.InitializeClass(EasyVHM)


def manage_addEasyVHM(self, REQUEST=None):
    """ """
    vhm = EasyVHM()
    vhm.id = 'EasyVHM'
    vhm.addToContainer(self.this())
    if REQUEST:
        return self.manage_main(self,REQUEST)
    else:
        return vhm



def initialize(context):
    """Initialize EasyEditable Product"""

    context.registerClass (
            EasyVHM,
            permission=EasyVHMManage,
            icon='icons/EasyVHM.gif',
            constructors=(manage_addEasyVHM, manage_addEasyVHM),
            meta_type = EasyVHM.meta_type
            )
