# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasyMasterSlaveCopy.py,v 1.4 2003/09/25 11:32:48 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyPublishSync mixin class"""
__version__='$Revision: 1.4 $'[11:-2]


import Globals
from Globals import DTMLFile
from string import lower, join
import DateTime
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from ExtensionClass import Base
from Acquisition import aq_base
from types import TupleType, ListType, StringType

from OFS.SimpleItem import SimpleItem
from Products.EasyNotification.EasyNotifiable import EasyNotifiable
from Products.EasyCore.VirtualPath import VirtualPathBase, exclude, stripPath, mergePaths

from Products.EasyCore.EasyObjectLink import EasyObjectLink, DefaultLinkPolicy, SingleLinkPolicy

from Products.EasyEditor.EasyDialogSupport import DialogSupport

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyEditor.BrowserRegistry import BrowserRegistry

from EasySyncSecurity import EasyPublisherManage


from Products.EasyCore.Utils.DebugUtils import MLOG_TAB, C_MLOG_TAB


MASTER_LINK=('MasterLink','SlaveLink')
SLAVE_LINK=('SlaveLink','MasterLink')

class RegistrationException(Exception): pass


class EasyMasterCopy(EasyObjectLink, SimpleItem, DialogSupport, VirtualPathBase, EasyPresentationHelpers):
    """
    Object of this class marks the root folder of an master copy.

    Master copies is the base copy that are used by SlaveCopies, which
    are defined by the EasySlaveCopy.
    """

    meta_type="EasyMasterCopy"

    isMasterCopy=1

    security = ClassSecurityInfo()

    _link_policys = {MASTER_LINK:DefaultLinkPolicy()}

    syncronized_meta_types=()

    def __init__(self, id):
        EasyObjectLink.__init__(self)
        self.id = self.__name__ = str(id)

    manage_options = (
            {'label':'Configure',     'action':'manage_main'},
            {'label':'Meta Types',     'action':'manage_syncronized'},
            ) + SimpleItem.manage_options


    security.declareProtected(EasyPublisherManage, 'manage_main')
    manage_main = DTMLFile("dtml/masterslave/manage_master_copy",globals())

    _manage_syncronized = DTMLFile("dtml/masterslave/manage_syncronized", globals())
    def manage_syncronized(self, meta_type_ids=None, REQUEST=None):
        """Manage Master Copy settings"""
        management_view='Meta Types'
        manage_tabs_message=''
        if type(meta_type_ids) in (TupleType, ListType):
            self.syncronized_meta_types = tuple(meta_type_ids)
            manage_tabs_message='Syncronized Meta Type where updated.'
        return self._manage_syncronized(self,REQUEST
                                    , management_view=management_view
                                    , manage_tabs_message=manage_tabs_message
                                )

    security.declarePrivate('getLinkedSlaveCopies')
    def getLinkedSlaveCopies(self):
        return self.getPaths(MASTER_LINK)

    security.declarePrivate('getSlaveVirtualUrlItems')
    def getSlaveVirtualUrlItems(self):
        items=[]
        slaves = self.getPaths(MASTER_LINK)
        for slave_path in slaves:
            path=slave_path[:-1]
            slaveObj = self.restrictedTraverse( path, None )
            if slaveObj:
                items.append((slaveObj.id, slaveObj.virtualURL))
        return items

    security.declarePrivate('getSlaveVirtualUrls')
    def getSlaveVirtualUrls(self):
        return map(lambda path: path[:-1], self.getPaths(MASTER_LINK))

    security.declarePublic('getSyncronizedMetaTypes')
    def getSyncronizedMetaTypes(self):
        return self.syncronized_meta_types

    security.declarePublic('isMetaTypesSynced')
    def isMetaTypesSynced(self, meta_type):
        return meta_type in self.syncronized_meta_types

    def manage_afterClone(self, item):
        self.deleteAllLinkByLinkName(MASTER_LINK)




Globals.InitializeClass(EasyMasterCopy)


def addEasyMasterCopy(self, REQUEST=None):
    """Add an EasyMasterCopy object"""
    ob = EasyMasterCopy('EasyMasterCopy')
    self._setObject('EasyMasterCopy', ob)
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')



class EasySlaveCopy(EasyObjectLink, SimpleItem, DialogSupport, VirtualPathBase, EasyPresentationHelpers):
    """
    Object of this class marks the root folder of an master copy.

    Master copies is the base copy that are used by slave copies, which
    are defined by the EasySlaveCopy.
    """

    meta_type="EasySlaveCopy"

    isSlaveCopy=1

    manage_options = ({'label':'Configure',     'action':'manage_main'},) + SimpleItem.manage_options

    security = ClassSecurityInfo()

    _link_policys = {SLAVE_LINK:SingleLinkPolicy()}


    def __init__(self, id, master_link=None):
        EasyObjectLink.__init__(self)
        self.id = self.__name__ = str(id)
        if self.isPathOrUrl(master_link):
            master_path = self.url2path(master_link)
            self.setMasterLink(master_path)

    security.declarePrivate('setMasterLink')
    def setMasterLink(self, virtual_path):
        if virtual_path[-1]!='EasyMasterCopy':
            virtual_path = virtual_path + ('EasyMasterCopy',)
        self.deleteAllLinkByLinkName(SLAVE_LINK)
        return self.createLink(SLAVE_LINK, self.virtualPathToPhysicalPath(virtual_path))

    security.declarePrivate('removeMasterLink')
    def removeMasterLink(self):
        self.deleteAllLinkByLinkName(SLAVE_LINK)

    security.declarePrivate('master_link_status')
    def master_link_status(self):
        return self.hasLink(SLAVE_LINK)

    security.declarePrivate('getLink')
    def getLink(self):
        paths=self.getPaths(SLAVE_LINK)
        if len(paths)==0: return ()
        return paths[0]

    security.declareProtected(EasyPublisherManage, 'manage_main')
    _manage_main = DTMLFile("dtml/masterslave/manage_slave_copy",globals())

    def manage_main(self,
                    master_root_url=None,
                    REQUEST=None):
        """Change Slave Configurations"""
        management_view='Configure'
        manage_tabs_message=""
        if master_root_url:
            virtual_path=tuple(master_root_url.split('/'))
            virtual_path=exclude(stripPath(virtual_path), not_allowed=('', 'EasyMasterCopy'))
            try:
                self.setMasterLink(virtual_path)
                manage_tabs_message="MasterCopy referrence has been changed to [%s]." % self.physicalPathToVirtualURL(self.getLink())
            except RegistrationException, e:
                self.removeMasterLink()
                manage_tabs_message=e
        elif master_root_url=='':
            self.removeMasterLink()
        if REQUEST is not None:
            return self._manage_main(self
                                    , management_view=management_view
                                    , manage_tabs_message=manage_tabs_message
                                    , REQUEST=REQUEST)

    security.declarePublic('getMasterVirtualUrl')
    def getMasterVirtualUrl(self):
        return self.physicalPathToVirtualURL(self.getLink())

    security.declarePublic('getEasyMasterCopyObj')
    def getEasyMasterCopyObj(self):
        return self.restrictedTraverse( self.getLink(), None )


Globals.InitializeClass(EasySlaveCopy)

def addEasySlaveCopy(self,REQUEST=None):
    """Add an EasySlaveCopy object"""
    ob = EasySlaveCopy('EasySlaveCopy')
    self._setObject('EasySlaveCopy', ob)
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')



def initialize(context):
    """Initialize EasySync Product"""

    context.registerClass (
            EasyMasterCopy,
            permission=EasyPublisherManage,
            icon='icons/EasyMasterCopy.gif',
            constructors=(addEasyMasterCopy,
                          addEasyMasterCopy),
             meta_type = EasyMasterCopy.meta_type
            )

    context.registerClass (
            EasySlaveCopy,
            permission=EasyPublisherManage,
            icon='icons/EasySlaveCopy.gif',
            constructors=(addEasySlaveCopy,
                          addEasySlaveCopy),
             meta_type = EasySlaveCopy.meta_type
            )

