# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasyNavigationDiff.py,v 1.4 2003/03/27 11:12:04 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyNavigationDiff mixin class"""
__version__='$Revision: 1.4 $'[11:-2]

import Globals
from Acquisition import Implicit, aq_base, aq_get
from types import DictType, TupleType, ListType, StringType
from Products.EasyCore.VirtualPathHelpers import path2url, url2path, comparePaths, absolute2relative
from ComputedAttribute import ComputedAttribute

from Products.EasyCore.Utils.DebugUtils import MLOG_TAB

class NavigationInfo:


    def __init__(self, brain=None, local_base=None):
        if brain is not None:
            self.path=absolute2relative(local_base,brain.physicalPath)
            self.physicalPath=brain.physicalPath
            self.virtualURL=brain.virtualURL
            self.modification_time=None
            self.id=brain.id
            self.title=brain.title
            self.navigation_title=brain.navigation_title
        else:
            self.physicalPath=''
            self.virtualURL=None
            self.path=''
            self.modification_time=None
            self.id=''
            self.title=''
            self.navigation_title='&#151;'

    def getLocalId(self):
        if not self.path: return ''
        if self.path and self.path[0]<>'':
            return self.path[-1]
        return "Remote"

    def getPath(self):
        return '/'.join(self.path)

    def isLocalLink(self):
        return self.path and self.path[0]<>''

    def isEmpty(self):
        return self.virtualURL is None

    def getNavigationTitle(self):
        return self.navigation_title or self.id or '&#151;'

    def physicalURL(self):
        if self.physicalPath:
            return '/'.join(self.physicalPath)
        return ''


MISSINGNAVIGATIONINFO=NavigationInfo()

class NavigationDiff(Implicit):
    """\
    NavigationDiff is the result of an Diff operation.

    NavigationDiff takes three arguments at initiation:
        local, remote, old_remote, which are NavigationInfo instances

    """

    order=None
    remote=None
    old_remote=None
    local=None

    __allow_access_to_unprotected_subNavigations__=1

    def __init__(self, order, local, remote, old_remote):
        self.order=order
        self.local =local
        self.remote=remote
        self.old_remote=old_remote

    def getOrder(self):
        return self.order

    def checked(self, selected_ids=()):
        #if self.getURL() in selected_ids: return 'checked'
        return ''

    def getLocal(self):
        return self.local

    def getRemote(self):
        return self.remote

    def getOldRemote(self):
        return self.old_remote

    def isAddedRemotely(self):
        return not not (not self.old_remote.physicalPath and self.remote.physicalPath)

    def isRemovedRemotely(self):
        return not not (self.old_remote.physicalPath and not self.remote.physicalPath)

    def isMissingRemotely(self):
        return not self.remote.physicalPath

    def isMissingLocally(self):
        return not self.local.physicalPath

    def diffrentLocally(self):
        if self.remote.virtualURL is None: return 1
        if self.local.virtualURL is None: return 1
        return 0

    def masterHasChanged(self):
        bool1 = (self.old_remote.virtualURL is None and self.remote.virtualURL is not None )
        bool2 = (self.old_remote.virtualURL is not None and self.remote.virtualURL is None )
        bool3 = self.old_remote.virtualURL != self.remote.virtualURL
        return bool1 or bool2 or bool3

    def __repr__(self):
        return "<NavigationDiff: %s >" % self.order

