# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasyObjectDiff.py,v 1.3 2003/03/25 16:25:01 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyPublishSync mixin class"""
__version__='$Revision: 1.3 $'[11:-2]

import Globals
from Acquisition import Implicit, aq_base, aq_get
from types import DictType, TupleType, ListType, StringType
from Products.EasyCore.VirtualPathHelpers import path2url, url2path, comparePaths

class MissingObject:
    id=''
    meta_type=''
    modification_time=''
    icon=''
    #isPrincipiaFolderish=''
    objectIds=()
    isTranslated=None
    isSyncronized=None
    isAnObjectManager=0
    physicalURL=None
    virtualURL=None

    def __init__(self, obj=None, spec=None):
        self.spec=None

MISSINGOBJECT=MissingObject()

class ObjectInfo(MissingObject):

    def __init__(self, obj=None, spec=None):
        self.spec=spec
        if obj is not None:
            #return MissingObject
            #baseobj = aq_base(obj)
            if hasattr(obj, 'getId'):
                id=obj.getId()
            else:
                id=obj.id
                if callable(id): id=id()
            self.id=id
            self.meta_type=getattr(obj,'meta_type','Unknown Meta Type')
            mtime=getattr(obj, 'bobobase_modification_time', None)
            if callable(mtime): mtime=mtime()
            self.modification_time=mtime
            if hasattr(obj, 'getIcon'):
                icon=obj.getIcon()
            else:
                icon=obj.icon
                if callable(icon): icon=icon()
            self.icon=icon
            self.isAnObjectManager=getattr(obj,'isAnObjectManager',0)
            #dict['isPrincipiaFolderish']=getattr(obj,'isPrincipiaFolderish',0)
            if hasattr(obj, 'objectIds'):
                self.objectIds = obj.objectIds(spec=spec)
            else:
                self.objectIds = ()
            self.physicalPath=obj.getPhysicalPath()
            self.physicalURL=path2url(self.physicalPath)
            self.virtualURL=getattr(obj, 'virtualURL', '')


class ObjectDiff(Implicit):
    """\
    ObjectDiff is the result of an Diff operation.

    ObjectDiff takes three arguments at initiation:
        local, remote, old_remote, which are ObjectInfo instances

    """


    remote=None
    old_remote=None
    local=None

    __allow_access_to_unprotected_subobjects__=1

    def __init__(self, id, local, remote, old_remote):
        self.id=id
        self.local =local
        self.remote=remote
        self.old_remote=old_remote

    def getLocal(self):
        return self.local

    def getRemote(self):
        return self.remote

    def getOldRemote(self):
        return self.old_remote

    def isAddedRemotely(self):
        return not self.old_remote.id and self.remote.id

    def isRemovedRemotely(self):
        return not not (self.old_remote.id and not self.remote.id)

    def isMissingRemotely(self):
        return not self.remote.id

    def isMissingLocally(self):
        return not self.local.id

    def hasChangedRemotely(self):
        if not self.remote.id: return 0
        if not self.local.id: return 0
        return not (self.old_remote.id and self.remote.id)\
               or  (self.old_remote.id and not self.remote.id)\
               or  self.old_remote.modification_time != self.remote.modification_time

    def isTranslated(self):
        return not not self.old_remote.isTranslated

    #DEPRECATED???
    def isSyncronized(self):
        return not not self.old_remote.isSyncronized


    def __repr__(self):
        return "<ObjectDiff: %s >" % self.id

    def getSyncStatusText(self):
        #FIX. Should hasChangedRemotely return anything sensable?
        res=[]
        if self.isMissingLocally(): res.append('Missing Locally')
        if self.hasChangedRemotely(): res.append('') #res.append('Changed Remotely')
        if self.isRemovedRemotely(): res.append('Removed Remotely')
        if self.isMissingRemotely(): res.append('Missing Remotely')
        return ', '.join(res)


