# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasySync.py,v 1.33 2004/01/14 14:38:56 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#


__doc__="""EasyPublishSync mixin class"""
__version__='$Revision: 1.33 $'[11:-2]


import Globals, Acquisition
from Globals import DTMLFile
from string import lower, join
import DateTime
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from ExtensionClass import Base
from Acquisition import Implicit, aq_base, aq_get, aq_chain
from types import TupleType, ListType, StringType

from ComputedAttribute import ComputedAttribute

from Products.EasyNotification.EasyNotifiable import EasyNotifiable
from Products.EasyCore.VirtualPath import VirtualPathBase

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyCorePermissions import EasySynchronizeManage, EasySynchronizeView

from Products.EasyWorkflow.EasyDiff import MasterDataCopy
from Products.EasyCore.EasyObjectLink import EasyObjectLink, DefaultLinkPolicy, SingleLinkPolicy
from Products.EasyCore.VirtualPathHelpers import url2path, comparePaths, absolute2relative
from Products.EasyCore.Utils.Utils import distinct
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs, BodyTabsInfo

try:
    from Products.EasyCore.XMLUtils.HTMLDiffer import HTMLDiffer
except:
    HTMLDiffer=None


from zLOG import LOG, INFO
from Products.EasyCore.Utils.DebugUtils import MLOG_TAB

LINK_BACK_TO_SLAVE=('MasterDocument','SlaveDocument')
LINK_TO_MASTER=('SlaveDocument','MasterDocument')
_marker=[]

class EasySyncAware(EasyObjectLink, BodyTabs, VirtualPathBase, EasyPresentationHelpers):
    """Mix-in class for Sync Aware Objects"""

    security = ClassSecurityInfo()
    bodytabs = BodyTabsInfo()

    _link_policys = {LINK_BACK_TO_SLAVE:DefaultLinkPolicy(),
                     LINK_TO_MASTER:    SingleLinkPolicy()}

    #easysyncaware_options = (
    #            {'label':'SyncContent', 'action':'manage_sync_content',},
    #            {'label':'SyncProperties', 'action':'manage_sync_properties',},
    #            {'label':'SyncNavigation', 'action':'manage_sync_navigation',},
    #            {'label':'SyncSubObjects', 'action':'manage_sync_subobjects',},
    #        )

    easysyncaware_options = (
                {'label':'Sync', 'action':'manage_sync_content',},
            )

    _master_data_copy=None

    def __init__(self):
        EasyObjectLink.__init__(self)


    # SLAVE INTERFACE

    security.declarePublic('getMasterDocumentPath')
    def getMasterDocumentPath(self, marker=None):
        """
        """
        master_path=self.getPaths(LINK_TO_MASTER, suppress_exception=1)
        if len(master_path)>0:
            return master_path[0]
        masterlink = self.getEasySlaveCopy()
        if masterlink:
            loot_path = masterlink.getPhysicalPath()[:-1]
            local_path = self.getPhysicalPath()
            virtual_path = local_path[len(loot_path):]
            rpath = self.mergePaths(masterlink.getLink()[:-1], virtual_path)
            return rpath
        return marker

    security.declarePublic('getMasterDocument')
    def getMasterDocument(self, path=None, marker=None):
        """
        Returns the path to the master document
        first by looking for a MASTERLING and then
        by comparing the virtual paths from master/
        slave roots.
        """
        if path is None:
            path = self.getMasterDocumentPath()
        if path is not None:
            return self.restrictedTraverse( path, marker )
        return marker

    security.declarePublic('getMasterDocumentURL')
    def getMasterDocumentURL(self, marker=''):
        """
        """
        path = self.getMasterDocumentPath()
        if path:
            return self.physicalPathToVirtualURL(path)
        return marker


    security.declarePublic('hasMasterDocument')
    def hasMasterDocument(self):
        """
        Returns true if ther is a valid master document.
        """
        path = self.getMasterDocumentPath()
        if path is not None:
            return self.restrictedTraverse( path, None ) is not None
        return 0

    security.declareProtected(EasySynchronizeManage, 'showSyncronized')
    showSyncronized = DTMLFile('dtml/sync/showSyncronized',globals())

    #MASTER SYNCAWARE INTERFACE
    security.declarePublic('getEasySlaveCopy')
    def getEasySlaveCopy(self, default=None):
        return getattr(self, 'EasySlaveCopy', default)

    security.declarePublic('getEasyMasterCopy')
    def getEasyMasterCopy(self, default=None):
        return getattr(self, 'EasyMasterCopy', default)

    security.declareProtected(EasySynchronizeManage, 'showMasterCopy')
    showMasterCopy = DTMLFile('dtml/sync/showMasterCopy',globals())

    security.declarePublic('isTranslated')
    def _isTranslated(self):
        master_data_copy=getattr(self, '_master_data_copy', None)
        if master_data_copy is None: return 0
        return 1
    isTranslated=ComputedAttribute(_isTranslated, 1)

    security.declarePublic('isSyncronized')
    def _isSyncronized(self):
        return self._isHardLinked() or self._isSoftLinked()
    isSyncronized=ComputedAttribute(_isSyncronized, 1)

    security.declarePublic('isSoftLinked')
    def _isSoftLinked(self):
        return hasattr(self, 'EasySlaveCopy')
    isSoftLinked=ComputedAttribute(_isSoftLinked, 1)

    security.declarePublic('isHardLinked')
    def _isHardLinked(self):
        return len(self.getPaths(LINK_TO_MASTER, suppress_exception=1))
    isHardLinked=ComputedAttribute(_isHardLinked, 1)

    security.declarePublic('isSlaveCopy')
    def _isSlaveCopy(self):
        return hasattr(self, 'EasySlaveCopy')
    isSlaveCopy=ComputedAttribute(_isSlaveCopy, 1)

    security.declarePublic('isMasterCopy')
    def _isMasterCopy(self):
        return hasattr(self, 'EasyMasterCopy')
    isMasterCopy=ComputedAttribute(_isMasterCopy, 1)


    security.declarePublic('masterHasMoved')
    def _masterHasMoved(self):
        master_data_copy=self.getMasterDataCopy()
        if not master_data_copy.hasMasterCopy():
            return -1
        if not self.isHardLinked: return -1
        master_obj=self.getMasterDocument()
        if master_obj is None: return -1

        return master_data_copy.masterHasMoved(master_obj)

    masterHasMoved=ComputedAttribute(_masterHasMoved, 1)

    def hasEverSyncedToMasterCopy(self):
        return self.getMasterDataCopy().hasMasterCopy()


    #SYNCRONIZATION STATE HARDLINKS and TRANSLATION

    def softlink2hardlink(self):
        if self.isSoftLinked:
            self.createLink(LINK_TO_MASTER, self.getMasterDocumentPath())
            self.setPathOfMaster()

    def manage_create_hardlink(self, master_url, ishardlinked, management_view='Sync', REQUEST=None):
        """Create a hard Link"""
        if ishardlinked:
            self.deleteAllLinkByLinkName(LINK_TO_MASTER)
            manage_tabs_message='Hard link to %s has been removed.' % master_url
        else:
            self.createLink(LINK_TO_MASTER, self.virtualURLToPhysicalPath(master_url))
            self.setPathOfMaster()
            manage_tabs_message='Hard link to %s has been created.' % master_url

        if REQUEST is not None:

            if management_view=='SyncProperties':
                manage_sync_view=self.manage_sync_properties
            elif management_view=='SyncContent':
                manage_sync_view=self.manage_sync_content
            elif management_view=='SyncSubObjects':
                manage_sync_view=self._manage_sync_subobjects
            elif management_view=='SyncNavigation':
                manage_sync_view=self._manage_sync_navigation
            return manage_sync_view(self, REQUEST
                            , manage_tabs_message=manage_tabs_message
                            , management_view=management_view
                        )

#
# Stranges thing. This method (doSyncronize) disturbs the
# EasyPropertyContainer from being acquired from DTML
# (without the correct permissions)
# Really strange
# It has something to do with: security.declareProtected('doSyncronize')

    security.declarePrivate('_createMasterCopy')
    def _createMasterCopy(self, master_obj, master_data_copy=None):
        master_data_copy = master_data_copy or MasterDataCopy()
        if not self.isHardLinked:
            master_data_copy.copyPath(master_obj)
        master_data_copy.copyProperties(master_obj)
        master_data_copy.copyRegions(master_obj)
        #WHY??#master_data_copy.copyContent(master_obj)
        master_data_copy.copyNavigation(master_obj)

        return master_data_copy

    security.declarePrivate('doSyncronize')
    def doSyncronize(self, object=None, master_data_copy=None):
        """Make syncronization"""
        master_data_copy=master_data_copy or self.getMasterDataCopy()
        master_obj=object or self.getMasterDocument()
        self._createMasterCopy(master_obj, master_data_copy)

        #if master_obj is not None and master_data_copy is not None:
        #    if not self.isHardLinked:
        #        master_data_copy.copyPath(master_obj)
        #    master_data_copy.copyProperties(master_obj)
        #    master_data_copy.copyRegions(master_obj)
        #    #WHY??#master_data_copy.copyContent(master_obj)
        #    master_data_copy.copyNavigation(master_obj)



    def getMasterDataCopy(self):
        _master_data_copy=getattr(self, '_master_data_copy', None)
        if _master_data_copy is None:
            _master_data_copy = MasterDataCopy()
        self._master_data_copy = _master_data_copy
        return _master_data_copy

    def setPathOfMaster(self):
        master_data_copy=self.getMasterDataCopy()
        master_obj=self.getMasterDocument()
        if master_obj is not None and master_data_copy is not None:
            master_data_copy.copyPath(master_obj)


    security.declareProtected(EasySynchronizeManage, 'manage_sync_subobjects')

    _manage_sync_subobjects = DTMLFile('dtml/sync/manage_sync_subobjects',globals()
        , management_view='Sync'
        , body_tab={'order':3, 'label':'Sub Objects', 'url':'manage_sync_subobjects', 'selectable':1, 'management_view':'Sync'}
        )
    def manage_sync_subobjects(self, selected_ids=[], copy_document_id=None, action=None, REQUEST=None):
        """\
        Handle SubObject syncronization actions.
        """
        management_view='Sync'
        manage_tabs_message=''
        if action=='Copy':
            for document_id in selected_ids:
                self.manage_copy_document(document_id)
            manage_tabs_message='[%s] has been copied from the master document.' % ','.join(selected_ids)
        elif copy_document_id is not None:
            self.manage_copy_document(copy_document_id)
            manage_tabs_message='%s has been copied from the master document.' % copy_document_id

        if REQUEST is not None:
            return self._manage_sync_subobjects(self, REQUEST
                            , manage_tabs_message=manage_tabs_message
                            , management_view=management_view
                        )



    #UpdateTab

    security.declareProtected(EasySynchronizeManage, 'manage_sync_navigation')
    _manage_sync_navigation = DTMLFile('dtml/sync/manage_sync_navigation',globals()
        , management_view='Sync'
        , body_tab={'order':4, 'label':'Navigation', 'url':'manage_sync_navigation', 'selectable':1, 'management_view':'Sync'}
        )
    def manage_sync_navigation(self, action=None, REQUEST=None):
        """Synchronize Navigation Menus"""
        valid_objects=[]
        valid=[] #.append(obj.virtualURL)
        invalid=[]
        management_view='Sync'
        manage_tabs_message=''
        if action=='Copy Menu':
            self.manage_delete_all(confirm=1)
            urls=[]
            local_base=self.getPhysicalPath()
            remote_obj=self.getMasterDocument()
            remote_base=remote_obj.getPhysicalPath()
            for brain in remote_obj.getMenuItems():
                remote_path=absolute2relative(remote_base,brain.physicalPath)
                if remote_path[0]!='': remote_path=local_base+remote_path
                urls.append('/'.join(remote_path))
            self.manage_add_to_menu(local_urls=urls)
            manage_tabs_message='The Navigation Menus have been synchronized.'
        if REQUEST is not None:
            return self._manage_sync_navigation(self, REQUEST
                            , manage_tabs_message=manage_tabs_message
                            , management_view=management_view
                        )


    security.declareProtected(EasySynchronizeManage, 'manage_copy_document')
    #_manage_copy_document = DTMLFile('dtml/sync/manage_copy_document',globals())
    def manage_copy_document(self, id, REQUEST=None):
        """\
        Copy document from Master Section to here.
        """
        if self.isSyncronized:
            master_obj=self.getMasterDocument()
            if hasattr(aq_base(master_obj),id):
                obj=master_obj._getOb(id)
                if obj is None: raise KeyError, "Object %s doesn't exist." % id
                if hasattr(aq_base(obj), 'cloneTo'):
                    obj.cloneTo(self)
                    obj=getattr(self,id) #Get object from locally
                else:
                    cp=master_obj.manage_copyObjects(id)
                    self.manage_pasteObjects(cp)
                    obj=getattr(self,id) #Get object from locally

                #Reset navigation!!!
                master_obj.saveMenuLocally(master_obj.getPhysicalPath(), delete=0)
                delattr(master_obj,'local_menu')
                obj.navigation_menus_data = {}
                #Reset Publish History!!!
                obj.resetPublishHistory()
                #obj.addLocalMenu(menu)
                obj.reindex_object()
                obj.softlink2hardlink()


    security.declareProtected(EasySynchronizeManage, 'manage_sync_properties')
    manage_sync_properties = DTMLFile('dtml/sync/manage_sync_properties',globals()
        , management_view='Sync'
        , body_tab={'order':2, 'label':'Properties', 'selectable':1, 'management_view':'Sync'}
        )

    security.declareProtected(EasySynchronizeManage, 'manage_sync_content')
    manage_sync_content = DTMLFile('dtml/sync/manage_sync_content',globals()
        , management_view='Sync'
        , body_tab={'order':1, 'label':'Contents', 'selectable':1, 'management_view':'Sync'}
        )


#
    # MASTER INTERFACE

    security.declareProtected(EasySynchronizeManage, 'getSyncronizedMetaTypes')
    def getSyncronizedMetaTypes(self):
        if self.isSlaveCopy and self.isSyncronized:
            master = self.getEasySlaveCopy().getEasyMasterCopyObj()
            if master:
                return master.getSyncronizedMetaTypes()
        return ()

    def getSlaveDocumentPaths(self):
        """
        Returns the paths (list) to all linked slave object.

        The link needs to be hard for the notification to work.
        """
        if self.isMasterCopy:
            if self.isHardLinked:
                slave_paths=self.getPaths(LINK_BACK_TO_SLAVE, suppress_exception=1)
                return slave_paths
            else:
                #find possible soft links
                easymastercopy = self.getEasyMasterCopy()
                if easymastercopy:
                    #calculate new paths to match slave root plus local path
                    #(diff from master root and current object)
                    new_paths=[]
                    loot_path = easymastercopy.getPhysicalPath()[:-1]
                    local_path = self.getPhysicalPath()
                    virtual_path = local_path[len(loot_path):]
                    for path in easymastercopy.getSlaveVirtualUrls():
                        new_paths.append(self.mergePaths(path, virtual_path))
                    return new_paths
        return []

        master_path=self.getPaths(LINK_TO_MASTER, suppress_exception=1)
        if len(master_path)>0:
            return master_path[0]
        masterlink = self.getEasySlaveCopy()
        if masterlink:
            loot_path = masterlink.getPhysicalPath()[:-1]
            local_path = self.getPhysicalPath()
            virtual_path = local_path[len(loot_path):]
            rpath = self.mergePaths(masterlink.getLink()[:-1], virtual_path)
            return rpath
        return marker


    def getSlaveDocuments(self):
        """
        Returns a list of all the linked slave object.

        The link needs to be hard for the notification to work.
        """
        slave_obs=[]
        if self.isMasterCopy:
            for slave_path in self.getSlaveDocumentPaths():
                slave=self.unrestrictedTraverse( slave_path, _marker )
                if slave is not _marker: slave_obs.append(slave)
        return slave_obs

    security.declarePublic('getEasySlaveVirtualUrlItems')
    def getEasySlaveVirtualUrlItems(self, default=None):
        slave_docs=self.getSlaveDocumentPaths()
        if slave_docs:
            pp2vu=self.physicalPathToVirtualURL
            return map(lambda path,pp2vu=pp2vu: (path[-1], pp2vu(path)),slave_docs)
        return []
        mastercopy = self.getEasyMasterCopy()
        #try: return self.getEasyMasterCopy().getSlaveItems()
        #except: pass
        #return []
        master_path=self.getPaths(LINK_TO_MASTER, suppress_exception=1)
        if len(master_path)>0:
            return master_path[0]
        masterlink = self.getEasySlaveCopy()
        if masterlink:
            loot_path = masterlink.getPhysicalPath()[:-1]
            local_path = self.getPhysicalPath()
            virtual_path = local_path[len(loot_path):]
            rpath = self.mergePaths(masterlink.getLink()[:-1], virtual_path)
            return rpath
        return marker



    # MASTER NOTIFICATION INTERFACE

    #This is not realy secure!!!
    security.declarePrivate('doNotifyOnMasterWorkflowChange')
    def doNotifyOnMasterWorkflowChange(self, old_state, new_state, history_entry):
        """Handles notification for the master object."""
        if self.isMasterCopy:
            for slave_obj in self.getSlaveDocuments():
                if hasattr(slave_obj,'notifyOnMasterWorkflowChange'):
                    slave_obj.notifyOnMasterWorkflowChange(self, old_state, new_state, history_entry)
        else:
            pass


    # SLAVE NOTIFICATION INTERFACE

    security.declarePrivate('notifyOnMasterWorkflowChange')
    def notifyOnMasterWorkflowChange(self, master_obj, old_state, new_state, history_entry):
        LOG("NotImplemented", INFO, "notifyOnMasterWorkflowChange in EasySync.py has not been implemented yet.")


        #notify_users = self.getUserIdToNotify(notification)
        #not_ns= {   'id':master_obj.id,
        #            'title':master_obj.title,
        #            'description':master_obj.description,
        #            'subject': ', '.join(master_obj.Subject()),
        #            'date': history_entry['Time'],
        #            'changed_by': history_entry['User'],
        #            'new_state':history_entry['State'],
        #            'old_state':history_entry['OldState'],
        #            'absolute_url': master_obj.absolute_url(),
        #            'comment': history_entry['Comment'],
        #        }
        #self.manage_notify('notify_master_changed', fromuser, tousers,  namespace)


    # DIFF OBJECTMANAGERS

    def getObjectDiff(self, spec=None):
        #Possible custom diff, choose what two object to diff (not just linked diff).
        #doSyncronize(self, object=None, master_data_copy=None)
        #MasterDataCopy()
        if self.isSyncronized:
            meta_types=spec or self.getSyncronizedMetaTypes()
            master_data_copy=self.getMasterDataCopy()
            return master_data_copy.getDiffObjects(self, spec=meta_types)
        return []


    render_property_value_diff=DTMLFile('dtml/sync/render_property_value_diff',globals())

    def getPropertyDiff(self, local_obj=None, exclude='webdav'):
        if local_obj is None: local_obj=self
        if self.isSyncronized:
            master_data_copy=getattr(local_obj, '_master_data_copy', None)
            if master_data_copy is not None:
                return master_data_copy.getDiffPropertySheets(local_obj, exclude=exclude)
        return []

    def getRegionDiff(self, local_obj=None):
        if local_obj is None: local_obj=self
        if self.isSyncronized:
            master_data_copy=getattr(local_obj, '_master_data_copy', None)
            if master_data_copy is not None:
                return master_data_copy.getDiffRegions(local_obj)
        return []

    def getNavigationDiff(self, local_obj=None):
        if local_obj is None: local_obj=self
        if self.isSyncronized:
            master_data_copy=getattr(local_obj, '_master_data_copy', None)
            if master_data_copy is not None:
                return master_data_copy.getDiffNavigation(local_obj)
        return []


    def diffHTML(self, html1, html2):
        if HTMLDiffer:
            differ=HTMLDiffer(html1, html2)
            differ.processDiff()
            return differ.getBodyContent()
        #Should have a non diffing differ here.
        return html2

    #Clean up Interface

    security.declareProtected(EasySynchronizeManage, 'manage_release_hardlinks')
    def manage_release_hardlinks(self, types=(), recursive=0, REQUEST=None):
        """\
        Release all hard links.
        If recursive is true it traverse it's sub objects.

        http://virttest1.torped.se/master/manage_release_hardlinks?types=slaves&types=masters&recursive=1
        """

        if type(types)==StringType:types=[types]

        if 'masters' in types:
            failed=self.deleteAllLinkByLinkName(LINK_TO_MASTER)

        if 'slaves' in types:
            failed=self.deleteAllLinkByLinkName(LINK_BACK_TO_SLAVE)

        if recursive:
            for obj in self.objectValues():
                if hasattr(aq_base(obj), 'manage_release_hardlinks'):
                    obj.manage_release_hardlinks(types=types, recursive=1)

        if REQUEST is not None:
            return self.manage_sync_content(self, REQUEST
                            , manage_tabs_message="Hard links removed%s." % str(recursive and ' recursive' or '')
                            , management_view='Sync'
                        )

    security.declareProtected(EasySynchronizeManage, 'manage_create_hardlinks')
    def manage_create_hardlinks(self, recursive=0, REQUEST=None):
        """\
        Create hardlinks for all matching objects

        http://virttest1.torped.se/master/manage_create_hardlinks?recursive=1
        """

        if self.isSoftLinked and not self.isHardLinked:
            self.createLink(LINK_TO_MASTER, self.getMasterDocumentPath())
            self.setPathOfMaster()

        if recursive:
            for obj in self.objectValues():
                if hasattr(aq_base(obj), 'manage_create_hardlinks'):
                    obj.manage_create_hardlinks(recursive=1)

        if REQUEST is not None:
            return self.manage_sync_content(self, REQUEST
                            , manage_tabs_message="Create hard links %s." % str(recursive and ' recursive' or '')
                            , management_view='Sync'
                        )


    security.declareProtected(EasySynchronizeManage, 'manage_syncronize_all')
    def manage_syncronize_all(self, recursive=0, REQUEST=None):
        """\
        Syncronize all existing objects.

        http://virttest1.torped.se/english/manage_syncronize_all?recursive=1
        http://virttest1.torped.se/master/manage_syncronize_all?recursive=1
        """

        self.doSyncronize()

        if recursive:
            for obj in self.objectValues():
                if hasattr(aq_base(obj), 'manage_syncronize_all'):
                    obj.manage_syncronize_all(recursive=1)

        if REQUEST is not None:
            return self.manage_sync_content(self, REQUEST
                            , manage_tabs_message="Object have been syncronized%s." % str(recursive and ' recursive' or '')
                            , management_view='Sync'
                        )

    ##def manage_afterAdd(self, item, container):
    ##    self.recreateRemoteLinks()

    ##def manage_beforeDelete(self, item, container):
    ##    self.disableRemoteLinks()

    def manage_afterClone(self, item):
        self.deleteAllLinkByLinkName(LINK_BACK_TO_SLAVE)

Globals.InitializeClass(EasySyncAware)
InitializeBodyTabs(EasySyncAware)

def initialize(context):
    """Initialize EasySync Product"""

    pass
