# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasyWorkflow.py,v 1.10 2003/07/23 16:03:49 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyWorkflow mixin class"""
__version__='$Revision: 1.10 $'[11:-2]


import Globals
from Globals import DTMLFile
import DateTime
from urllib import quote
from AccessControl import getSecurityManager, ClassSecurityInfo
from Acquisition import aq_inner, aq_parent

from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.VirtualPath import VirtualPathBase
##from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.EasyPresentationHelpers.EasyParameterTracker import EasyParameterTracker
from Products.EasyCore.EasyTypeRegistry import EasyTypeRegistry
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs

from EasyWorkflowBase import *

from Products.EasyCore import MLOG_TAB


class WorkflowManagement(EasyParameterTracker, BodyTabs, VirtualPathBase):
    """Workflow management interface"""

    security = ClassSecurityInfo()

    manage_options = (
                        {'label': 'Workflow',   'action': 'manage_workflow',},
                     )

    track_fields = (
                        {'id':'sfilterall',             'type':'string',    'default':'showall', },
                        {'id':'rfilternon',             'type':'string',    'default':'shownone', },
                        {'id':'skey',                   'type':'string',    'default':'getWorkflowState', },
                        {'id':'rkey',                   'type':'string',    'default':'',           },
                        {'id':'togglekey',              'type':'string',    'default':'showall', },
                        {'id':'filteronauthor',         'type':'int',    'default':1, },
                        {'id':'filteronpublished_by',   'type':'int',    'default':0, },
                        {'id':'filterondraft',          'type':'int',    'default':1,           },
                        {'id':'filteronsubmitted',      'type':'int',    'default':1, },
                        {'id':'filterondenied',         'type':'int',    'default':1, },
                        {'id':'filteronreviewed',       'type':'int',    'default':1, },
                        {'id':'filtercolumns',          'type':'lines',     'default':(
                                                                                    'type',
                                                                                    'published',
                                                                                    'state',
                                                                                    'title',
                                                                                    'author',
                                                                                    'effective_date',
                                                                                    'expiration_date'
                                                                                ), },
                        {'id':'b_start',                'type':'int',       'default':1, },
                        {'id':'b_size',                 'type':'int',       'default':20, },
                        {'id':'b_orphan',               'type':'int',       'default':3, },
                        {'id':'show_types',             'type':'lines',     'default':EasyTypeRegistry.getWorkflowableTypes(), },
                    )

    security.declareProtected(EasyWorkflowView,'manage_workflow')
    manage_workflow = DTMLFile('dtml/manage_workflow', globals()
        , management_view='Workflow'
        , body_tab={'order':1, 'label':'Workflow', 'selectable':1, 'management_view':'Workflow'})

    security.declareProtected(EasyWorkflowView,'manage_configure')
    manage_configure = DTMLFile('dtml/manage_configure', globals()
        , management_view='Workflow'
        , body_tab={'order':2, 'label':'Configure', 'selectable':1, 'management_view':'Workflow'})

    security.declareProtected(EasyWorkflowView,'manage_statistics')
    manage_statistics = DTMLFile('dtml/manage_statistics', globals()
        , management_view='Workflow'
        , body_tab={'order':3, 'label':'Statistics', 'selectable':1, 'management_view':'Workflow'})



    security.declarePublic('getWorkflowManager')
    def getWorkflowManager(self):
        return self

    security.declarePrivate('getWorkflowCatalog')
    def getWorkflowCatalog(self):
        return self.EasyNavigation

    security.declarePrivate('getWorkflowScoop')
    def getWorkflowScoop(self, obj):
        return aq_parent(obj).getPhysicalPath()

    security.declareProtected(EasyWorkflowView,'getWorkflowDocuments')
    def getWorkflowDocuments(self,
            filteronauthor='',
            filteronpublished_by='',
            filterondraft='',
            filteronsubmitted='',
            filterondenied='',
            filteronreviewed='',
            REQUEST=None):
        if REQUEST is None: REQUEST=self.REQUEST
        searchquery = {}
        show_types=REQUEST.get('show_types',None)
        if not show_types: return []
        searchquery['meta_type'] =  {'query': show_types}
        workflowfilter = []
        current_user = getSecurityManager().getUser().getId()
        if filteronauthor and filteronauthor == 'showme':
            searchquery['author'] = {'query': current_user}
        if filteronpublished_by and filteronpublished_by == 'showme':
            searchquery['approved_by'] = {'query': current_user}
        if filterondraft:
            workflowfilter.append(wfDRAFT)
        if filteronsubmitted:
            workflowfilter.append(wfSUBMITTED)
        if filterondenied:
            workflowfilter.append(wfDENIED)
        if filteronreviewed:
            workflowfilter.append(wfPUBLISHED)
        if workflowfilter:
            searchquery['getWorkflowState'] ={'query':  workflowfilter}
        else:
            return []
        searchquery['physicalPathIndex'] = {'query': '/'.join(self.getWorkflowScoop(self))}
        return self.getWorkflowCatalog()(searchquery)


    security.declareProtected(EasyWorkflowView,'getPublishStateFor')
    ##IFC:before: getPublishState(self, workflowstate, effective_date, expiration_date)
    ######:after: getPublishStateFor(self, workflowstate, effective_date, expiration_date)
    def getPublishStateFor(self, workflowstate, effective_date, expiration_date):
        if (effective_date!=NullDate and effective_date>DateTime.now()) or workflowstate!=wfPUBLISHED:
            return psUNPUBLISHED
        if expiration_date!=NullDate and expiration_date < DateTime.now():
            return psARCHIVED
        return psPUBLISHED

    security.declareProtected(EasyWorkflowView,'getPublishIconFor')
    def getPublishIconFor(self, publishstate, default=''):
        try: iconsrc,message = pslookup[publishstate][0], pslookup[publishstate][1]
        except: return default
        if iconsrc:
            result="""<img src="%s" border="0" """ % (iconsrc,)
            result=result+"""alt="%s" """ % (publishstate,)
            if message: result=result+"""title="%s" """ % (message,)
            result=result+"/>"
            return result
        else: return default

    security.declareProtected(EasyWorkflowView,'getWorkflowIconFor')
    def getWorkflowIconFor(self, id, default=''):
        try: iconsrc, message = wflookup[id][0], wflookup[id][1]
        except: return default
        if iconsrc:
            result="""<img src="%s" border="0" """ % (iconsrc,)
            result=result+"""alt="%s" """ % (id,)
            if message: result=result+"""title="%s" """ % (message,)
            result=result+"/>"
            return result
        else: return default

    security.declareProtected(EasyWorkflowView,'manage_general_ids_form')
    manage_general_ids_form = DTMLFile('dtml/manage_general_ids_form', globals())

    security.declareProtected(EasyWorkflowView,'getValidInvalidBrains')
    def getValidInvalidBrains(self, urls, new_state):
        paths = self.urls2paths(urls)
        valid, invalid = self.getValidFromStates(new_state)
        searchquery = {}
        searchquery['physicalPath'] = {'query': paths }
        searchquery['getWorkflowState'] = {'query': valid}
        valid_res = self.getWorkflowCatalog()(searchquery)
        searchquery['getWorkflowState'] = {'query': invalid}
        invalid_res = self.getWorkflowCatalog()(searchquery)
        return valid_res, invalid_res

    security.declareProtected(EasyWorkflowView,'getCatalogBrainsForUrls')
    def getCatalogBrainsForUrls(self, urls):
        paths = self.urls2paths(urls)
        searchquery = {}
        searchquery['physicalPath'] = {'query': paths}
        res = self.getWorkflowCatalog()(searchquery)
        return res

    security.declareProtected(EasyWorkflowSubmit,'manage_submit_ids_form')
    def manage_submit_ids_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Submit Id form"""
        kw={}
        kw['new_state'] = wfSUBMITTED
        kw['valid_result'], kw['invalid_result'] = self.getValidInvalidBrains(urls, kw['new_state'])
        return apply(self.manage_general_ids_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowRetract,'manage_retract_ids_form')
    def manage_retract_ids_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Retract Id form"""
        kw={}
        kw['new_state'] = wfDRAFT
        kw['valid_result'], kw['invalid_result'] = self.getValidInvalidBrains(urls, kw['new_state'])
        return apply(self.manage_general_ids_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowPublish,'manage_publish_accept_ids_form')
    def manage_publish_accept_ids_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Publish/Accept Id form"""
        kw={}
        kw['new_state'] = wfPUBLISHED
        kw['valid_result'], kw['invalid_result'] = self.getValidInvalidBrains(urls, kw['new_state'])
        return apply(self.manage_general_ids_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowDeny,'manage_deny_revoke_ids_form')
    def manage_deny_revoke_ids_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Deny/Revoke Id form"""
        kw={}
        kw['new_state'] = wfDENIED
        kw['valid_result'], kw['invalid_result'] = self.getValidInvalidBrains(urls, kw['new_state'])
        return apply(self.manage_general_ids_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowView,'manage_general_dates_form')
    manage_general_dates_form = DTMLFile('dtml/manage_general_dates_form', globals())

    security.declareProtected(EasyWorkflowChangeDate,'manage_change_publish_dates_form')
    def manage_change_publish_dates_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Publish Date form"""
        kw={}
        kw['date'] = 'effective_date'
        kw['valid_result'] = self.getCatalogBrainsForUrls(urls)
        return apply(self.manage_general_dates_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowChangeDate,'manage_change_archive_dates_form')
    def manage_change_archive_dates_form(self, urls, client=None, REQUEST=None):
        """Setup keyword arguments for Archive Date form"""
        kw={}
        kw['date'] = 'expiration_date'
        kw['valid_result'] = self.getCatalogBrainsForUrls(urls)
        return apply(self.manage_general_dates_form, (client,REQUEST), kw)

    security.declareProtected(EasyWorkflowView,'manage_change_workflow_states')
    #SEQURITY
    #Sequrity check in changeWorkflowState, we need a stronger solution here
    #Transitions are dictionaries now they should be object with sequrity assertions.
    def manage_change_workflow_states(self, urls, new_state, comment='', REQUEST=None, RESPONSE=None):
        """Change workflow states"""
        paths = self.urls2paths(urls)
        changed = []
        you_dont_have_access = []
        for path in paths:
            obj = self.unrestrictedTraverse(path)
            if callable(obj.id): id = obj.id()
            else: id = obj.id
            # CHECKTHIS: LENNART
            try:
                obj.changeWorkflowState(new_state,comment)
            except 'Unauthorized':
                you_dont_have_access.append(id)
            else:
                changed.append(id)
        message=""
        if changed:
            message = "The workflow state of the documents have been set to '%s'<br>" % (new_state, )
        if you_dont_have_access:
            message = message  + "You didn't have the right privilege to change the workflow state for the following documents: %s<br>" % (you_dont_have_access.join(', '))
        if REQUEST and RESPONSE:
            return RESPONSE.redirect('manage_workflow?manage_tabs_message=%s&management_view=Workflow' % (quote(message)))

    security.declareProtected(EasyWorkflowView,'manage_change_dates_for_ids')
    #SEQURITY
    #Sequrity check in changePublishState, we need a stronger solution here
    #Transitions are dictionaries now they should be object with sequrity assertions.
    def manage_change_dates_for_ids(self, selectall, date, urls=[], alldate=None, dates=[], comment='', REQUEST=None, RESPONSE=None):
        """Save date changes for all documents in urls."""
        if selectall:
            for url in urls:
                obj = self.unrestrictedTraverse(self.url2path(url))
                if date=='effective_date':
                    obj.changePublishState(effective_date=alldate, expiration_date=DONTCHANGEDATE, comment=comment)
                elif date=='expiration_date':
                    obj.changePublishState(effective_date=DONTCHANGEDATE, expiration_date=alldate, comment=comment)
        else:
            for rec in dates:
                if rec.url in urls:
                    obj = self.unrestrictedTraverse(self.url2path(rec.url))
                    if date=='effective_date':
                        obj.changePublishState(effective_date=rec.date, expiration_date=DONTCHANGEDATE, comment=comment)
                    elif date=='expiration_date':
                        obj.changePublishState(effective_date=DONTCHANGEDATE, expiration_date=rec.date, comment=comment)
        if REQUEST and RESPONSE:
            return RESPONSE.redirect('manage_workflow?manage_tabs_message=%s&management_view=Workflow' % (quote("Dates changed")))




    ##Interigation Interface...

    security.declareProtected(EasyWorkflowView,'getWorkflowItems')
    def getWorkflowItems(self, meta_type=None, workflow_state=None):
        searchquery = {}
        if workflow_state is not None: searchquery['getWorkflowState'] = {'query': workflow_state}
        if meta_type is None: meta_type=self.getMetaTypeKeys()
        searchquery['meta_type'] =  {'query': meta_type}
        searchquery['physicalPathIndex'] = {'query': '/'.join(aq_parent(self).getPhysicalPath())}
        return self.getWorkflowCatalog()(searchquery)

    security.declareProtected(EasyWorkflowView,'getWorkflowStates')
    def getWorkflowStates(self):
        return wflookup.keys()

    security.declarePublic('getMetaTypeKeys')
    def getMetaTypeKeys(self):
        return EasyTypeRegistry.getWorkflowableTypes()

    security.declarePublic('getMetaTypeItems')
    def getMetaTypeItems(self):
        return map(lambda x: (x, x) ,EasyTypeRegistry.getWorkflowableTypes())

    security.declarePublic('getPossibleColumn')
    def getPossibleColumn(self):
        return (('type','Type'),
                ('published','Published'),
                ('state','State'),
                ('title','Title'),
                ('virtualURL','URL'),
                ('author','Author'),
                ('published_by','Published By'),
                ('owner','Owner'),
                ('stateby','State By'),
                ('statedate','State Date'),
                ('creation_date','Created Date'),
                ('effective_date','Effective Date'),
                ('expiration_date','Expiration Date'),
            )


Globals.InitializeClass(WorkflowManagement)
InitializeBodyTabs(WorkflowManagement)


def initialize(context):
    """Initialize StyleManagers Product"""

    pass # register Publishable as a BaseClass

