# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/EasyWorkflow/EasyWorkflowBase.py,v 1.14 2003/11/15 12:56:39 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__='''EasyWorkflow product: WorkflowBase mixin class.'''
__version__='$Revision: 1.14 $'[11:-2]


import Globals
from ExtensionClass import Base
import DateTime
from AccessControl import getSecurityManager, ClassSecurityInfo

from Products.EasyCore.EasyCorePermissions import *

NullDate = ""
DONTCHANGEDATE = "DONTCHANGEDATE"


#DEFAULT WORKFLOW

def transition_publish(ob ,comment, effective_date=None, expiration_date=None):
    if hasattr(ob, 'isSyncronized') and ob.isSyncronized: ob.doSyncronize()
    return ob.changeWorkflowState(wfPUBLISHED, comment)

transition_accept = transition_publish

def transition_deny(ob ,comment, effective_date=None, expiration_date=None):
    return ob.changeWorkflowState(wfDENIED, comment)

transition_revoke=transition_deny

def transition_submit(ob ,comment, effective_date=None, expiration_date=None):
    return ob.changeWorkflowState(wfSUBMITTED, comment)

def transition_submit_now(ob ,comment, effective_date=None, expiration_date=None):
    res = ob.changeWorkflowState(wfSUBMITTED, comment)
    resdate = ob.changePublishState(DateTime.now(), DONTCHANGEDATE, comment)
    return (res, resdate)

def transition_publish_now(ob ,comment, effective_date=None, expiration_date=None):
    res = ob.changeWorkflowState(wfPUBLISHED, comment)
    resdate = ob.changePublishState(DateTime.now(), DONTCHANGEDATE, comment)
    if hasattr(ob, 'isSyncronized') and ob.isSyncronized: ob.doSyncronize()
    return (res, resdate)

def transition_archive_now(ob ,comment, effective_date=None, expiration_date=None):
    res = ob.changeWorkflowState(wfPUBLISHED, comment)
    resdate = changePublishState(DONTCHANGEDATE, DateTime.now(), comment)
    return (res, resdate)

def transition_change_publish_date(ob ,comment, effective_date=None, expiration_date=None):
    return ob.changePublishState(DateTime.now(), DONTCHANGEDATE, comment)

def transition_change_archive_date(ob ,comment, effective_date=None, expiration_date=None):
    return ob.changePublishState(DONTCHANGEDATE, DateTime.now(), comment)

def transition_retract(ob ,comment, effective_date=None, expiration_date=None):
    return ob.changeWorkflowState(wfDRAFT, comment)



# Publish state constants:
psUNPUBLISHED="Unpublished"
psPUBLISHED="Published"
psARCHIVED="Archived"


pslookup = {
    psUNPUBLISHED:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_UnPublished",
        "This document is currently not published",
      ),
    psPUBLISHED:
          (
        "/misc_/EasyWorkflow/EasyWorkflow_Published",
        "This document is published",
      ),
    psARCHIVED:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_Archived",
        "This document is archived",
      ),
}

#workflowstate constants
wfDRAFT = 'Draft'
wfSUBMITTED = 'Submitted'
wfDENIED = 'Denied'
wfPUBLISHED = 'Published'


# A dictionary where the workflowstat constant maps to a tuple of information about that state
# The tuple contains ( icon, description text, default roles, short description text )
wflookup = {
    wfDRAFT:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_Draft",
        "This is a draft document",
        ('Author', 'Reviewer','Manager'),
        "Draft",
      ),
    wfSUBMITTED:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_Submitted",
        "This document has been submitted for review",
        ('Author', 'Reviewer','Manager'),
        "Submitted",
      ),
    wfDENIED:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_Denied",
        "This document has been denied for publishing",
        ('Reviewer','Manager'),
        "Denied",
      ),

    wfPUBLISHED:
      (
        "/misc_/EasyWorkflow/EasyWorkflow_Reviewed",
        "This document has been accepted for publishing",
        ('Reviewer','Manager'),
        "Accepted",
      ),
}


wf_transitions = {
    wfDRAFT:
      (
        {
            'new_state':wfSUBMITTED,
            'label_text':"Submit for review",
            'action_text':"Submit",
            'permissions':(EasyWorkflowSubmit,),
            'permissions_operator':"or",
            'transition_method': transition_submit,
            'notifications': ('notify_submit',)
        },
        {
            'new_state':wfPUBLISHED,
            'label_text':"Publish",
            'action_text':"Publish",
            'permissions':(EasyWorkflowPublish,),
            'permissions_operator':"or",
            'transition_method': transition_publish,
            'notifications': ('notify_publish',) #,'notify_master_changed')
        },
      ),
    wfSUBMITTED:
      (
        {
            'new_state':wfSUBMITTED,
            'label_text':"Submit Again for review",
            'action_text':"Submit Again",
            'permissions':(EasyWorkflowSubmit,),
            'permissions_operator':"or",
            'transition_method': transition_submit,
            'notifications': ('notify_submit',)
        },
        {
            'new_state':wfPUBLISHED,
            'label_text':"Accept",
            'action_text':"Accept",
            'permissions':(EasyWorkflowPublish,),
            'permissions_operator':"or",
            'transition_method': transition_accept,
            'notifications': ('notify_publish',) #,'notify_master_changed')
        },
        {
            'new_state':wfDENIED,
            'label_text':"Deny",
            'action_text':"Deny",
            'permissions':(EasyWorkflowDeny,),
            'permissions_operator':"or",
            'transition_method': transition_deny,
            'notifications': ('notify_deny',)
        },
        {
            'new_state':wfDRAFT,
            'label_text':"Retract",
            'action_text':"Retract",
            'permissions':(EasyWorkflowRetract,),
            'permissions_operator':"or",
            'transition_method': transition_retract,
            'notifications': ('notify_retract',)
        },
      ),
    wfDENIED:
      (
        {
            'new_state':wfSUBMITTED,
            'label_text':"Submit",
            'action_text':"Submit",
            'permissions':(EasyWorkflowSubmit,),
            'permissions_operator':"or",
            'transition_method': transition_submit,
            'notifications': ('notify_submit',)
        },
        {
            'new_state':wfPUBLISHED,
            'label_text':"Publish",
            'action_text':"Publish",
            'permissions':(EasyWorkflowPublish,),
            'permissions_operator':"or",
            'transition_method': transition_publish,
            'notifications': ('notify_publish',) #,'notify_master_changed')
        },
        {
            'new_state':wfDRAFT,
            'label_text':"Retract",
            'action_text':"Retract",
            'permissions':(EasyWorkflowRetract,),
            'permissions_operator':"or",
            'transition_method': transition_retract,
            'notifications': ('notify_retract',)
        },
      ),

    wfPUBLISHED:
      (
        {
            'new_state':wfPUBLISHED,
            'label_text':"Publish Again",
            'action_text':"Publish Again",
            'permissions':(EasyWorkflowPublish,),
            'permissions_operator':"or",
            'transition_method': transition_publish,
            'notifications': ('notify_publish',) #,'notify_master_changed')
        },
        {
            'new_state':wfDENIED,
            'label_text':"Revoke",
            'action_text':"Revoke",
            'permissions':(EasyWorkflowDeny,),
            'permissions_operator':"or",
            'transition_method': transition_revoke,
            'notifications': ('notify_revoke',)
        },
        {
            'new_state':wfDRAFT,
            'label_text':"Retract",
            'action_text':"Retract",
            'permissions':(EasyWorkflowRetract,),
            'permissions_operator':"or",
            'transition_method': transition_retract,
            'notifications': ('notify_retract',)
        },
      ),
}


class WorkflowBase(Base):
    """Workflow mix-in class"""


    security = ClassSecurityInfo()

    security.declarePublic('getPublishIconForState')
    def getPublishIconForState(self, publish_state, default='', size=16):
        try: iconsrc,message = pslookup[publish_state][0], pslookup[publish_state][1]
        except: return default
        result="""<img src="%s" border="0" """ % (iconsrc,)
        result=result+"""alt="%s" """ % (publish_state,)
        if message: result=result+"""title="%s" """ % (message,)
        result=result+'width="%s" height="%s"/>' % (str(size), str(size))
        return result

    security.declarePublic('getPublishStateMessageForState')
    def getPublishStateMessageForState(self, publish_state):
        result = pslookup[publish_state][1]
        return result

    security.declarePublic('getWorkflowIconForState')
    def getWorkflowIconForState(self, workflow_state, default='', size=16):
        try: iconsrc, message = wflookup[workflow_state][0], wflookup[workflow_state][1]
        except: return default
        result="""<img src="%s" border="0" """ % (iconsrc,)
        result=result+"""alt="%s" """ % (workflow_state,)
        if message: result=result+"""title="%s" """ % (message,)
        result=result+'width="%s" height="%s"/>' % (str(size), str(size))
        return result

    security.declarePublic('getWorkflowMessageForState')
    def getWorkflowMessageForState(self, workflow_state, default=''):
        try: message = wflookup[workflow_state][1]
        except: return default
        return message

    security.declarePublic('getWorkflowTransitionFor')
    def getWorkflowTransitionFor(self, old_state, new_state, marker=None):
        for transition in wf_transitions[old_state]:
            if transition['new_state'] == new_state:
                return transition
        return marker

    security.declarePublic('getWorkflowMessageForState')
    def getWorkflowNameForState(self, workflow_state, default=''):
        try: return wflookup[workflow_state][3]
        except: return default

    security.declarePublic('isTransitionAllowed')
    def isTransitionAllowed(self, old_state, new_state):
        for transition in wf_transitions[old_state]:
            if transition['new_state'] == new_state:
                return 1
        return 0

    security.declarePublic('getValidFromStates')
    def getValidFromStates(self, new_state):
        valid = []
        invalid = []
        for key, transitions in wf_transitions.items():
            for transition in transitions:
                if transition['new_state'] == new_state:
                    if key not in valid: valid.append(key)
            if key not in valid: invalid.append(key)
        return valid, invalid


    security.declarePrivate('getNotificationsForTransition')
    def getNotificationsForTransition(self, old_state, new_state):
        if wf_transitions.has_key(old_state):
            for trans in wf_transitions[old_state]:
                if trans['new_state'] == new_state:
                    return trans['notifications']
        return ()

    security.declarePrivate('notifyOnWorkflowChange')
    def notifyOnWorkflowChange(self, REQUEST=None):
        pass

    security.declarePrivate('_checkPermission')
    def _checkPermission(self, permissions, logic_operator='or'):
        #getSecurityManager().checkPermission can handle ProxyRoles
        #so this check needs a real roles to work.
        #This means problems when accessing through Python Scripts or
        #DTML Methods as users without correct permission.
        logic_operator = logic_operator.lower()
        if type(permissions)==type(''): permissions = (permissions,)
        checkPermission=getSecurityManager().checkPermission
        if logic_operator=='and': result = 1
        else: result = 0
        for perm in permissions:
            if logic_operator=='and': result = result and checkPermission(perm, self)
            else: result = result or checkPermission(perm, self)
        return result

Globals.InitializeClass(WorkflowBase)
