#####################################################################
#
# Xron -- Zope Scheduled Event Product
# Copyright (C) 2000 Loren Stafford
#
# Derived from the ZScheduler product version 0.0.7 in accord
# with the terms of the ZScheduler's license.
#
# Based on code by Martijn Pieters, (c) 1999 Antraciet B.V.
# Used with explicit permission.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#         Free Software Foundation, Inc.,
#         59 Temple Place - Suite 330,
#         Boston, MA    02111-1307, USA.
#
#####################################################################
""" Scheduled DTML Methods for Xron """

from ExtensionClass import Base
from Globals import InitializeClass
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.ZCatalog.CatalogAwareness import CatalogAware
from urllib import quote
import sys, string
import TimeAritmetics
import Schedule
ScheduleID=Schedule.ScheduleID

from Products.EasyCore import MLOG_TAB


class XronBase(CatalogAware, Base):

    meta_type = 'XronBase'
    default_catalog = ScheduleID

    next_update=None
    last_event_time=None
    periodyears=0
    periodmonths=0
    perioddays=0
    periodhours=0
    periodminutes=0
    periodweeks=0


    security = ClassSecurityInfo()

    def __init__(self, id, title = ''
                , periodyears=0
                , periodmonths=0
                , perioddays=0
                , periodhours=0
                , periodminutes=0
                , periodweeks=0
                , start=None
                ):
        self.id=id
        self.title=title
        self.updateIntervals(periodyears, periodmonths, perioddays, periodhours, periodminutes, periodweeks)
        self.next_update=None
        self.last_event_time=None
        if start:
            self.arm(start)

    def updateIntervals(self
                        , periodyears=0
                        , periodmonths=0
                        , perioddays=0
                        , periodhours=0
                        , periodminutes=0
                        , periodweeks=0
                        ):
        if periodyears is not None: self.periodyears=periodyears
        if periodmonths is not None: self.periodmonths=periodmonths
        if perioddays is not None: self.perioddays=perioddays
        if periodhours is not None: self.periodhours=periodhours
        if periodminutes is not None: self.periodminutes=periodminutes
        if periodweeks is not None: self.periodweeks=periodweeks

    manage_options=(
              {'label':'Schedule', 'action':'manage_schedule', },
              {'label':'Trigger', 'action':'manage_triggerXronEvent'},
              {'label':'Security', 'action':'manage_access'},
            )

    security.declareProtected('Xron Manage Event')
    manage_schedule=DTMLFile('dtml/manage_base_schedule', globals())

    #getPhysicalPath

    def manage_save_schedule(self,REQUEST=None
                                , armed=0
                                , next_update=None
                                , periodyears=None
                                , periodmonths=None
                                , perioddays=None
                                , periodhours=None
                                , periodminutes=None
                                , periodweeks=None
                            ):
        """Save Changes"""
        if next_update and next_update!='None':
            try:
                self.next_update=DateTime(next_update)
            except:
                self.next_update=DateTime()
        elif armed and not self.next_update:
            self.next_update=DateTime()
        self.updateIntervals(periodyears, periodmonths, perioddays, periodhours, periodminutes, periodweeks)
        print "Do index"
        self.index_object()
        if REQUEST:
            return self.manage_schedule(self,REQUEST,manage_tabs_message="The intervals was updated.")

    def nextEventTime(self):
        return self.next_update

    def setNextEventTime(self):
        now=DateTime()
        next=self.last_event_time or now
        if next>now: next=now()
        print "next?", next, now
        if self.periodyears:
            next=TimeAritmetics.addYears(next, self.periodyears)
        if self.periodmonths:
            next=TimeAritmetics.addMonths(next, self.periodmonths)
        if self.perioddays:
            next=TimeAritmetics.addDays(next, self.perioddays)
        if self.periodhours:
            next=TimeAritmetics.addHours(next, self.periodhours)
        if self.periodminutes:
            next=TimeAritmetics.addMinutes(next, self.periodminutes)
        if self.periodweeks:
            next=TimeAritmetics.addWeeks(next, self.periodweeks)
        self.next_update=next
        self.index_object()

    # Prevent changing the default_catalog
    manage_editCatalogerForm = None
    def manage_editCataloger(self, default):
        raise NotImplemented

    # Only index if nextEventTime returns something
    #def index_object(self):
    #    if self.next_update is not None:
    #        print "index"
    #        CatalogAware.index_object.im_func(self)
    #        # see Python Reference Manual "The standard type hierarchy"
    #        # for the built-in type im_func

    security.declareProtected('Xron Manage Event', 'reschedule')
    def reschedule(self,REQUEST=None):
        """Implement rescheduling policy.
        This method is designed to be overridden by a ZClass DTML-method;
        so, it passes parameters via REQUEST.
        input:    REQUEST.XronEventOldTime a DateTime object
        output: REQUEST.XronEventTime a DateTime object
        Calculate REQUEST.XronEventTime as a function of REQUEST.XronEventOldTime
        """
        self.setNextEventTime()

    def do_trigger(self, **kw):
        pass #Override

    #security.declareProtected('Xron Manage Event', 'trigger')
    def trigger(self, client=None, REQUEST=None, RESPONSE=None, **kw):
        """For use by the Dispatcher to fire the event.
        Use also from the browser URL box to debug your events.
        Reschedules the event, then calls the action method.
        """
        try:
            print "Trigger", self.next_update
            self.do_trigger(**kw)
            if self.next_update is not None:
                self.setNextEventTime()
        except:
            import sys, traceback, string
            type, val, tb = sys.exc_info()
            fe=traceback.format_exception (type, val, tb)
            return string.join(fe, '\n')
        print "Trigger End"

    security.declareProtected('Xron Manage Event', 'manage_triggerXronEvent')
    def manage_triggerXronEvent(self, client=None, REQUEST=None, RESPONSE=None, **kw):
        """For use by the Trigger tab on Xron management page.
        Use for testing the Xron in a request context.
        """
        aurl=REQUEST['URL1']
        furl=string.join((aurl, 'trigger'), '/')
        RESPONSE.redirect(furl)

    #security.declareProtected('Xron Manage Event', 'arm')
    def arm(self, start=None):
        if start is not None:
            self.next_update=start
            self.index_object()
        else:
            self.setNextEventTime()


    #security.declareProtected('Xron Manage Event', 'disarm')
    def disarm(self):
        """Private method for use only by the Dispatcher,
        when trigger fails.
        Marks the event as fired by nullifying its time.
        """
        self.next_update=None
        self.index_object()


InitializeClass(XronBase)


