# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/ZMIPatch/OFS_ObjectManager/__init__.py,v 1.7 2004/01/26 11:01:15 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__ = """OFS_ObjectManager."""
__version__='$Revision: 1.7 $'[11:-2]

from zLOG import LOG, INFO, BLATHER

from types import StringType

from Globals import ImageFile, DTMLFile, InitializeClass
from OFS.ObjectManager import ObjectManager
from OFS.CopySupport import CopyContainer
from DocumentTemplate.sequence import sort
from AccessControl import ClassSecurityInfo
from App.version_txt import getZopeVersion


LOG('ZMIPatch', BLATHER, 'ZMI Patch:OFS.ObjectManager', 'ZMI Patch: OFS ObjectManager: , Enable Labels in the property ManagementView.\n\t\tJohan Carlsson, 2002-10-02, version Easy Publisher 1.8\n')


zope_version=getZopeVersion()
if zope_version[0:3]>=(2,7,0) and zope_version[0:3] < (2,8,0):
    manage_main = DTMLFile('dtml/270/manage_main', globals())
    ObjectManager.manage_main = manage_main
elif zope_version[0:3]>=(2,6,1) and zope_version[0:3] < (2,7,0):
    manage_main = DTMLFile('dtml/261/manage_main', globals())
    ObjectManager.manage_main = manage_main
elif zope_version[0:3]>=(2,5,1)  and zope_version[0:3] < (2,6,1):
    manage_main = DTMLFile('dtml/251/manage_main', globals())
    ObjectManager.manage_main = manage_main
else:
    manage_main = DTMLFile('dtml/261/manage_main', globals())
    ObjectManager.manage_main = manage_main


oomPermission = 'Copy or Move'
ObjectManager.security = ClassSecurityInfo()

ObjectManager.possible_obj_sort_columns=['id','meta_type', 'get_size', 'position']

def get_object_position(self, id):
    """ Get the position of a particular object. """

    ids = self.objectIds()

    if id in ids:
        return ids.index(id)

ObjectManager.security.declareProtected(oomPermission, 'get_object_position')
ObjectManager.get_object_position=get_object_position


def move_objects_by_positions(self, ids, delta):
    """ Move objects with ids by delta. """

    if type(ids) is StringType:
        ids = (ids,)

    min_position = 0
    counter = 0

    objects = list(self._objects)

    obj_dict = {}
    for obj in objects:
        obj_dict[obj['id']] = obj

    if delta > 0:
        ids = list(ids)
        ids.reverse()
        objects.reverse()

    for id in ids:
        object = obj_dict[id]

        old_position = objects.index(object)
        new_position = max( old_position - abs(delta), min_position )

        if new_position == min_position:
            min_position += 1

        if old_position <> new_position:
            objects.remove(object)
            objects.insert(new_position, object)
            counter += 1

    if counter > 0:
        if delta > 0:
            objects.reverse()
        self._objects = tuple(objects)

    return counter

ObjectManager.security.declareProtected(oomPermission, 'move_objects_by_positions')
ObjectManager.move_objects_by_positions=move_objects_by_positions


def move_objects_up(self, ids, delta=1):
    """ Move objects with ids up by delta. """

    return self.move_objects_by_positions(ids, -delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_up')
ObjectManager.move_objects_up=move_objects_up


def move_objects_down(self, ids, delta=1):
    """ Move objects with ids down by delta. """

    return self.move_objects_by_positions(ids, delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_down')
ObjectManager.move_objects_down=move_objects_down


def move_objects_to_top(self, ids):
    """ Move objects with ids to top. """

    return self.move_objects_by_positions(ids, -len(self._objects))

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_top')
ObjectManager.move_objects_to_top=move_objects_to_top


def move_objects_to_bottom(self, ids):
    """ Move objects with ids to bottom. """

    return self.move_objects_by_positions(ids, len(self._objects))

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_bottom')
ObjectManager.move_objects_to_bottom=move_objects_to_bottom


def manage_move_objects_up(self, ids=None, delta=1, REQUEST=None):
    """ Move objects with ids up by delta (default = 1). """

    if ids:
        self.move_objects_up(ids, delta)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_up')
ObjectManager.manage_move_objects_up=manage_move_objects_up


def manage_move_objects_down(self, ids=None, delta=1, REQUEST=None):
    """ Move objects with ids down by delta (default = 1). """

    if ids:
        self.move_objects_down(ids, delta)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_down')
ObjectManager.manage_move_objects_down=manage_move_objects_down


def manage_move_objects_to_top(self, ids=None, REQUEST=None):
    """ Move objects with ids to top. """

    if ids:
        self.move_objects_to_top(ids)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_top')
ObjectManager.manage_move_objects_to_top=manage_move_objects_to_top


def manage_move_objects_to_bottom(self, ids=None, REQUEST=None):
    """ Move objects with ids to bottom. """

    if ids:
        self.move_objects_to_bottom(ids)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_bottom')
ObjectManager.manage_move_objects_to_bottom=manage_move_objects_to_bottom


def manage_orderObjects(self, key, reverse, REQUEST=None):
    """ Orders a objectmanager by key / reverse. """

    ids = [ id for id, obj in sort( self.objectItems(),
                                  ( (key, 'cmp', 'asc'), ) ) ]
    if reverse:
        ids.reverse()

    self.move_objects_to_top(ids)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_orderObjects')
ObjectManager.manage_orderObjects = manage_orderObjects
CopyContainer._old_manage_renameObject=CopyContainer.manage_renameObject


def manage_renameObject(self, id, new_id, REQUEST=None):
    """ rename a particular sub-object """

    try:
        position = self.get_object_position(id)
        result = self._old_manage_renameObject(id, new_id, REQUEST)
        newposition = self.get_object_position(new_id)
        self.move_objects_by_positions(new_id, position - newposition)
    except:
        result = self._old_manage_renameObject(id, new_id, REQUEST)

    return result

CopyContainer.manage_renameObject=manage_renameObject

InitializeClass(ObjectManager)


