# $Header: /usr/custom/cvs/public/EnergyPlusLive/lib/python/Products/ZMIPatch/UpgradeSupport/upgradeEasyPublisher.py,v 1.16 2003/07/16 19:01:49 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""
Usage:

Call manage_upgrade from the root point you want to upgrade
and follow the instructions.

To write upgrade extentions for your objects you need to implement
a upgradehook method:

security.declareProtected('Easy Upgrade Manage', 'upgradehook')
def upgradehook(self, REQUEST=None, changed=0):
    '''\
    returns changed or (changed, MessageList):
        changed=1, mean upgrade was successful
        changed=0, means nothing was upgraded.

        MessageList is a list or tuple of strings that will be
            written to the upgrade result page.

    On error an Exception is raised.
    Exception id, exception value, and traceback are all written to
    the resulting web page.

    The REQUEST is use to enable direct access to upgradehook ttw.
    For upgradehooks that should be callable ttw you need to return
    HTML instead of changed. There also needs to be a doc string.

    upgradehook should be declared protected by the "Easy Upgrade Manage"
    permission.

    '''

"""

import Globals
from os.path import join
from Products.EasyImage.EasyImage import *
from Products.EasyFile.EasyFile import *
from Products.EasyCore.EasyCorePermissions import *
from types import ListType, TupleType, StringType
import os
import time
from cStringIO import StringIO
import sys, traceback, string
from Products.EasyCore.Utils.Utils import createDTMLMethod, createDTMLFile
#from OFS.Application import Application
from OFS.ObjectManager import ObjectManager
from Products.EasyCore.Utils.Results import Results, Result, HTMLWriter, LogWriter
from Products.EasyNavigation.EasyNavigation import manage_addEasyNavigation
from Products.EasyResources.EasyResources import manage_addEasyResources
from Products.EasyNewsService.EasyNewsService import manage_addEasyNewsService
#from Products.EasyVHM.EasyVHM import manage_addEasyVHM
from Acquisition import aq_base
from zLOG import LOG, INFO, ERROR, TRACE, DEBUG, BLATHER

from Products.EasyCore.Utils.DebugUtils import MLOG_TAB, C_MLOG_TAB

ignore_id = (
    'DatabaseManagement',
    'Versions',
    'DavLockManager',
    'Products',
)
ignore_meta_type = (
    'Database Management',
    'Version Management',
    'WebDAV Lock Manager',
    'Product Management',
    'Product Help',
    'Product',
    'Debug Information',
)

# Return result: 0 = Failed upgrade, 1 = Successful upgrade, 2 = Success, no upgrade needed

app=ObjectManager
securityRoleManager = ClassSecurityInfo()
#createDTMLMethod(EasyRoot, 'ep_template', 'emi/ep_template')


#WORKERS

# Do Methods must be safe, They should never fail.

NOOP='No operation'
INDEXED='Indexed'
UPGRADE_SUCCESSFUL='Upgraded Successfully'
NO_UPGRADE_NEEDED='No Upgraded Needed'
UPGRADEHOOK_SUCCESSFUL='Called Objects Upgradhook Successfully'
UPGRADE_FAILED='Upgrade Failed'
INDEX_FAILED='Index Failed'



def do_delete_obsolete_items(root, object, results):
    if hasattr(aq_base(object), 'sessionDM') or hasattr(aq_base(object), 'session_id_mgr'):
        results.write(DEBUG, "do_delete_obsolete_items"
                     , "manage_delObjects( 'sessionDM' ) and manage_delObjects( 'session_id_mgr' )"
                     )
        result=results.getResult(object)
        try:
            if hasattr(object, 'sessionDM'):
                object.manage_delObjects( 'sessionDM' )
            if hasattr(object, 'session_id_mgr'):
                object.manage_delObjects( 'session_id_mgr' )
            result.addResult(UPGRADE_SUCCESSFUL)
            result.show(log_level=INFO)
            result.write_all()
        except:
            get_transaction().abort()
            exc = sys.exc_info()
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            result.addResult(UPGRADE_FAILED)
            result.addError("Couldn't delete 'sessionDM' and/or 'session_id_mgr': %s: %s\n\n%s\n\n%s" % (exc[0],exc[1],error_tb) )
            result.show(log_level=ERROR)
            result.write_all()
        results.setResult(result)



def do_nothing(root, object, results):
    results.write(DEBUG, "Do nothing for %s [%s]" % (object.getId(), '/'.join(object.getPhysicalPath()) ) )
    result=results.getResult(object)
    try:
        results.addMessage('Did nothing.')
        results.addResult(NO_UPGRADE_NEEDED)
        result.show(log_level=INFO)
        result.write_all()
    except:
        get_transaction().abort()
        result=results.getResult(object)
        result.addResult(UPGRADE_FAILED)
        result.addMessage('Failed to do nothing.')
        result.show(log_level=ERROR)
        result.write_all()
    results.setResult(result)

def do_upgrade_workflow(root, object, results):
    if hasattr(aq_base(object), 'upgradeWorkflow'):
        results.write(DEBUG, "Do Upgrade Workflow for %s [%s]" % (object.getId(), '/'.join(object.getPhysicalPath()) ) )
        result=results.getResult(object)
        try:
            res=object.upgradeWorkflow()
            if res:
                result.addMessage(res)
                result.show(log_level=INFO)
                result.write_all()
        except:
            get_transaction().abort()
            result.addResult(UPGRADE_FAILED)
            result.addMessage('Failed to do nothing.')
            result.show(log_level=ERROR)
            result.write_all()
        results.setResult(result)



def do_upgradehook(root, object, results):
    if hasattr(aq_base(object), 'upgradehook'):
        results.write(DEBUG, "Do Upgrade Hook for %s [%s]" % (object.getId(), '/'.join(object.getPhysicalPath()) ) )
        result=results.getResult(object)
        try:
            hook_result=object.upgradehook()
            hook_messages=()
            if type(hook_result) == TupleType:
                hook_messages = hook_result[1]
                hook_result = hook_result[0]
                if type(hook_messages)==StringType:
                    hook_messages=(hook_messages,)
            if hook_result:
                result=results.getResult(object)
                result.addResult(UPGRADEHOOK_SUCCESSFUL)
                for hook_msg in hook_messages:
                    result.addMessage(hook_msg)
                result.show(log_level=INFO)
                result.write_all()
                results.setDoReindex()
            elif Globals.DevelopmentMode:
                result=results.getResult(object)
                result.addResult(NO_UPGRADE_NEEDED)
                for hook_msg in hook_messages:
                    result.addMessage(hook_msg)
                result.show(log_level=ERROR)
                result.write_all()
        except:
            get_transaction().abort()
            exc = sys.exc_info()
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            result.addResult(UPGRADE_FAILED)
            result.addError("Upgradehook failed: %s: %s\n\n%s" % (exc[0],exc[1],error_tb) )
            result.show(log_level=ERROR)
            result.write_all()
        results.setResult(result)

def do_reindex(root, object, results):
    index_object = getattr(aq_base(object), 'index_object', None)
    if index_object and callable(index_object):
        results.write(DEBUG, "Do reindex for %s [%s]" % (object.getId(), '/'.join(object.getPhysicalPath()),))
        result=results.getResult(object)
        try:
            object.index_object()
            result.setReindexed()
            result.show(log_level=INFO)
            result.write_all()

        except:
            get_transaction().abort()
            result.addResult(INDEX_FAILED)
            exc = sys.exc_info()
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            result.addError("Reindex Failed: %s: %s\n\n%s" % (exc[0],exc[1],error_tb) )
            result.show(log_level=ERROR)
            result.write_all()
        results.setResult(result)


#FrameWork

def traverseObjects(root, object, results, methods=None, done=None):
    """This method only traverse the object hierachy and calles
    methods in the methods tuple.
    Don't edit this method to change the upgrade scenarios, use upgrade insteed.
    """
    if done is None: done=[]
    #Call the upgrade methods:
    if type(methods) == TupleType:
        for method in methods:
            if callable(method):
                method(root, object, results)
                get_transaction().commit(1)
    elif callable(methods):
            methods(root, object, results)
            get_transaction().commit(1)

    #Traverse subobjects, and upgrade them too.
    if hasattr(aq_base(object) ,'objectValues'):
        for subobject in object.objectValues():
            id=subobject.getId()
            if subobject not in done:
                done.append(subobject)
                meta_type=getattr(subobject,'meta_type',None)
                if hasattr(subobject, 'objectValues') and id not in ignore_id and meta_type is not None and meta_type not in ignore_meta_type:
                    traverseObjects(root, subobject, results, methods=methods, done=done)
            else:
                results.write(DEBUG, "Object %s has already been upgrade." % id)

#Main Method

def upgrade(self, root_object=None, log_level=ERROR, c_upgrade=0, c_reindex=0, c_clear_catalogs=1, REQUEST=None):
    """This method defines the Upgrade Scenario.
    Add upgrade scenarios by adding calls to traverseObjects.

    Each call to traverseObjects traverses the object hierarchy
    and calles the upgrade methods on all object.

    Its always wice to run a traverseObjects with do_index to re index all objects.
    """
    if root_object is None:
        root_object = self
    #html_writer=HTMLWriter( REQUEST=REQUEST
    #                      , header=self.manage_upgrade_header
    #                      , items=self.manage_upgrade_items
    #                      , footer=self.manage_upgrade_footer
    #                      )
    log_writer=LogWriter(title="Easy Publisher Upgrade")
    results=Results(log_level=log_level, writers=(log_writer,) )
    #Start of Upgrade Scenario Calls
    #traverseObjects(self, root_object, results, (do_nothing,))

    if c_clear_catalogs:
        results.write(INFO, "Start fix_catalogs")
        traverseObjects(self, root_object, results, (do_fix_catalogs,))
        results.write(INFO, "Stop fix_catalogs")
    LOG("upgradeEasyPublisher.upgrade", INFO, "c_clear_catalogs: Finished")
    if c_upgrade:
        results.write(INFO, "Start traverseObjects")
        traverseObjects(self, root_object, results,
                        ( do_upgradehook
                        , do_upgrade_workflow
                        , do_delete_obsolete_items
                        #, do_upgradeVirtualHostMonster
                        ))
        results.write(INFO, "Stop traverseObjects")
    LOG("upgradeEasyPublisher.upgrade", INFO, "c_upgrade: Finished")
    if c_reindex:
        results.write(INFO, "Start Reindex")
        if results.getDoReindex() or not c_upgrade:
            traverseObjects(self, root_object, results, (do_reindex,))
        results.write(INFO, "Stop Reindex")
    LOG("upgradeEasyPublisher.upgrade", INFO, "c_reindex: Finished")
    return results.items()



createDTMLFile(app, securityRoleManager, 'manage_upgrade', 'dtml/manage_upgrade'
                  , protection='protected'
                  , permission='Easy Upgrade Manage'
                  , globals=globals())

createDTMLFile(app, securityRoleManager, 'manage_upgrade_header', 'dtml/manage_upgrade_header'
                  , protection='protected'
                  , permission='Easy Upgrade Manage'
                  , globals=globals())


createDTMLFile(app, securityRoleManager, 'manage_upgrade_footer', 'dtml/manage_upgrade_footer'
                  , protection='protected'
                  , permission='Easy Upgrade Manage'
                  , globals=globals())

createDTMLFile(app, securityRoleManager, 'manage_upgrade_items', 'dtml/manage_upgrade_items'
                  , protection='protected'
                  , permission='Easy Upgrade Manage'
                  , globals=globals())


securityRoleManager.declareProtected('Easy Upgrade Manage', 'manage_do_upgrade')
def manage_do_upgrade(self, log_level=ERROR, c_upgrade=0, c_reindex=0, REQUEST=None):
    """Upgrade the site."""
    upgrade_report = upgrade(self, log_level=log_level, c_upgrade=c_upgrade, c_reindex=c_reindex, REQUEST=REQUEST)
    if REQUEST is not None:
        return self.manage_upgrade(self,REQUEST,upgrade_report=upgrade_report)
    return upgrade_report

app.manage_do_upgrade = manage_do_upgrade


securityRoleManager.declareProtected('Easy Upgrade Manage', 'manage_reindex_site')
def manage_reindex_site(self, root_object=None, log_level=ERROR, REQUEST=None):
    """\
    """
    if root_object is None:
        root_object = self
    #html_writer=HTMLWriter( REQUEST=REQUEST
    #                      , header=self.manage_upgrade_header
    #                      , items=self.manage_upgrade_items
    #                      , footer=self.manage_upgrade_footer
    #                      )
    log_writer=LogWriter(title="Easy Publisher Reindexing")
    results=Results(log_level=log_level, writers=(log_writer, ) )

    results.write(INFO, "Start Reindex")
    traverseObjects(self, root_object, results, (do_reindex,))
    results.write(INFO, "Stop Reindex")

    if REQUEST is not None:
        return self.manage_upgrade(self,REQUEST,upgrade_report=results.items())
    return results.items()

app.manage_reindex_site = manage_reindex_site




securityRoleManager.declareProtected('Easy Upgrade Manage', 'do_fix_catalogs')
def do_fix_catalogs(root, object, results):
    if getattr(object, 'meta_type', None) in ('EasyNavigation','EasyResources', 'EasyNewsService', 'EasyApplication', 'EasyApplicationConfigure'):
        result=results.getResult(object)
        try:
            elapse = time.time()
            c_elapse = time.clock()

            object.refreshCatalog(clear=1)

            elapse = time.time() - elapse
            c_elapse = time.clock() - c_elapse

            result.addResult("The catalog %s was reindex in %s seconds (%s seconds)." % ('/'.join(object.getPhysicalPath()), str(elapse),str(c_elapse)))

            result.show(log_level=INFO)
            result.write_all()

        except:
            get_transaction().abort()
            result.addResult("Fix Catalog Failed")
            exc = sys.exc_info()
            f=StringIO()
            traceback.print_exc(100,f)
            error_tb=f.getvalue()
            result.addError("Can't create subobject: %s: %s\n\n%s" % (exc[0],exc[1],error_tb) )
            result.show(log_level=ERROR)
            result.write_all()
        results.setResult(result)

app.do_fix_catalogs = do_fix_catalogs

securityRoleManager.declareProtected('Easy Upgrade Manage', 'manage_fix_catalogs')
def manage_fix_catalogs(self, root_object=None, log_level=ERROR, REQUEST=None):
    """\
    """
    if root_object is None:
        root_object = self
    #html_writer=HTMLWriter( REQUEST=REQUEST
    #                      , header=self.manage_upgrade_header
    #                      , items=self.manage_upgrade_items
    #                      , footer=self.manage_upgrade_footer
    #                      )
    log_writer=LogWriter(title="Easy Publisher Reindexing")
    results=Results(log_level=log_level, writers=(log_writer,) )

    results.write(INFO, "Start fix catalogs")
    traverseObjects(self, root_object, results, (do_fix_catalogs,))
    results.write(INFO, "Stop fix catalogs")

    if REQUEST is not None:
        return self.manage_upgrade(self,REQUEST,upgrade_report=results.items())

    return results.items()

app.manage_fix_catalogs = manage_fix_catalogs


app.security = securityRoleManager
Globals.InitializeClass(app)
