##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Standard keychain generators.

$Id: keygen.py,v 1.2 2004/01/09 14:05:32 johanc Exp $
"""

from apelib.core.interfaces import IKeychainGenerator
from apelib.core.exceptions import MappingError


class NullKeychainGenerator:
    """A null keychain generator refuses to generate any keychains."""

    __implements__ = IKeychainGenerator

    def makeKeychain(self, event, name, stored):
        raise MappingError("Null keychain generator")


class PathKeychainGenerator:
    """Path-based keychain generator
    """
    __implements__ = IKeychainGenerator

    def makeKeychain(self, event, name, stored):
        if name is None:
            raise MappingError('Path keychains require a name')
        if '/' in name:
            raise ValueError, '%s is not a legal name' % name
        parent_keychain = event.getKeychain()
        k = parent_keychain[-1]
        if k.endswith('/'):
            k = k + name
        else:
            k = '%s/%s' % (k, name)
        return parent_keychain[:-1] + (k,)

