##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""SQL keychain generator

$Id: keygen.py,v 1.2 2004/01/09 14:05:33 johanc Exp $
"""

from apelib.core import interfaces, exceptions

import sqlbase


class SQLKeychainGenerator (sqlbase.SQLGatewayBase):

    __implements__ = (interfaces.IKeychainGenerator,
                      interfaces.IDatabaseInitializer)

    table_base_name = 'key_seq'
    column_defs = ()

    def init(self, event):
        conn = event.getConnection(self.conn_name)
        first_time = 0
        try:
            rows = self.execute(event, 'sequence_check', 1)
            if len(rows) == 0:
                first_time = 1
        except conn.error:
            conn.db.rollback()
            self.execute(event, 'sequence_create')
            first_time = 1
        if first_time:
            self.execute(event, 'sequence_insert')
        if event.clearing():
            self.execute(event, 'sequence_clear')
        conn.db.commit()

    def makeKeychain(self, event, name, stored):
        if not stored:
            raise exceptions.ConfigurationError(
                'Keychains generated by SQLKeychain must be stored')
        if interfaces.ISDEvent.isImplementedBy(event):
            # Request that the other side do the work (for ZEO)
            n = event.getKeyedObjectSystem().newKey()
        else:
            self.execute(event, 'sequence_update')
            n = self.execute(event, 'sequence_read', 1)[0][0]
        return event.getKeychain()[:-1] + (long(n),)

